/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef LINKVIEW_H
#define LINKVIEW_H

#if 0 /* some versions of c++ */
using namespace std;
#endif

#ifndef EXTERN
#  define EXTERN extern
#endif
#ifndef DEFAULT
#  define DEFAULT(x)
#endif

#ifdef __GNUG__
#  define std
#  undef RTTI
#  if defined EGCS || __GNUC__ >= 2 && __GNUC_MINOR__ >= 8
#    define RTTI
#  endif
#  include <iterator.h>
#  include <function.h>
#  define FUNCTION_H
#endif

#if defined RTTI && defined __cplusplus
#  include <typeinfo>
#  define DCAST(type, data)  dynamic_cast<type>(data)
#  define RCAST(type, data)  reinterpret_cast<type>(data)
#  define SCAST(type, data)  static_cast<type>(data)
#  define CCAST(type, data)  const_cast<type>(data)
#else
#  define DCAST(type, data)  ((type)(data))
#  define RCAST(type, data)  ((type)(data))
#  define SCAST(type, data)  ((type)(data))
#  define CCAST(type, data)  ((type)(data))
#endif

#if defined DEBUGGING && defined __cplusplus

#include <iomanip.h>

struct DBG_FLAGS {
    enum {
        IO        = 1 <<  0,  /* */
        Natto     = 1 <<  1,  /* Natto.cc */
        View      = 1 <<  2,  /* NattoView.cc */
        Misc      = 1 <<  3,  /* mainx.cc util.cc */
        Parser    = 1 <<  4,  /* */
        Kanji     = 1 <<  5,  /* kanji.cc */
        OpenGL    = 1 <<  6,  /* NattoViewGL.cc */
        Proto     = 1 <<  7,  /* */
        GUI       = 1 <<  8,  /* PortKit.cc */
        Motif     = 1 <<  9,  /* NattoView_Xm.cc */
        URL       = 1 << 11,  /* URL.cc */
        NetObj    = 1 << 12,  /* NetObject.cc */
        Mem       = 1 << 13,  /* memobject.cc */
        Net       = 1 << 14,  /* netstream.cc */
    };
};

EXTERN int debug_flags DEFAULT(= ~(DBG_FLAGS::Kanji | DBG_FLAGS::OpenGL));

#define STR(x)  # x
#define DBG_POS(mod, sub)  \
    setiosflags(ios::left)   << setw(6) << setfill('.') << STR(mod) << \
    resetiosflags(ios::left) << setw(5) << setfill('.') << sub << ' '
#define DBG(sub, msg)  \
    if (debug_flags & DBG_FLAGS::DBG_MODULE) cerr << \
    DBG_POS(DBG_MODULE, sub) << msg << endl

#ifdef __DMALLOC_H__

inline void * operator new  (size_t n) { return malloc(n); }
inline void * operator new[](size_t n) { return malloc(n); }
inline void   operator delete  (void * p) throw() { if (p) free(p); }
inline void   operator delete[](void * p) throw() { if (p) free(p); }

#define DBG_NEW  \
    public: \
    void* operator new  (size_t n)   { return malloc(n); } \
    void* operator new[](size_t n)   { return malloc(n); } \
    void  operator delete  (void* p) { if (p) free(p); } \
    void  operator delete[](void* p) { if (p) free(p); }

#define DBG_NEW2  \
    DBG_NEW \
    void* operator new(size_t n, self_type* p) { return ::operator new(n, p); }

#else
#define DBG_NEW
#define DBG_NEW2
#endif  /* __DMALLOC_H__ */

#else

#define DBG(x,y)
#define DBG_NEW

#endif  /* DEBUGGING */

void ExitNattoView(void);

#endif /* LINKVIEW_H */
