/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_Anchor_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "AnchorBox.h"

#include "MainFrame.h"
#include "SiteBar.h"


AnchorBox::AnchorBox(
    const char * name,
    GuiObject*   parent,
    MainFrame*   frame):

    GuiObject(name, parent), frame_(frame),
    form_(new LayoutForm("anchorbox_form", this)),
    sitebar_(new SiteBar("anchorbox_sitebar", form_, frame_))
{}


AnchorBox::~AnchorBox()
{
    delete form_;
    delete sitebar_;
}


AnchorBox_if*
AnchorBox::interface()
{
    return DCAST(AnchorBox_if*, imp()->interface());
}


void
AnchorBox::create(
    GuiConfig* config)
{
    GuiObject::create(&PortKit::create_AnchorBox, config);
}


void
AnchorBox::set_node(
    Natto::node* vp)
{
    if (!vp) return;

    string title = "NattoView - " + vp->label();
    interface()->set_title(title.c_str());
    sitebar_->set_node(vp);
    interface()->set_node(vp);
}


void
AnchorBox::activate(
    Natto::link* ep)
{
    frame_->focus_node(ep->to());
    frame_->natto()->sync();
}


void
AnchorBox::select(
    Natto::link* ep)
{
    frame_->focus_link(ep);
    frame_->natto()->sync();
}


void
AnchorBox::focus(
    Natto::link* ep)
{
    frame_->focus_node(ep->to());
    frame_->natto()->sync();
}


void
AnchorBox::scan(
    vector<Natto::link*>& links)
{
    vector<Natto::link*>::iterator ei;
    int i;
    for (ei = links.begin(), i = 0; ei != links.end(); ei++, i++) {
	interface()->unhighlight();
	interface()->highlight(i);
        frame_->focus_link(*ei);
    }
    frame_->natto()->sync();
}


void
AnchorBox::browse(
    Natto::link* ep)
{
    frame_->focus_link(ep);
    frame_->browse(ep->to());
    frame_->natto()->sync();
}


AnchorBox_if::~AnchorBox_if()
{}
