/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef ANCHORBOX_H
#define ANCHORBOX_H
#ifdef __GNUG__
#pragma interface
#endif

#include <vector>
#include "PortKit.h"
#include "Natto.h"


class AnchorBox_if: public GuiObject_if
{
 public:
    virtual ~AnchorBox_if();

    virtual void popup()                = 0;
    virtual void popdown()              = 0;
    virtual void clear()                = 0;
    virtual void highlight(int)         = 0;
    virtual void unhighlight()          = 0;
    virtual void set_node(Natto::node*) = 0;
    virtual void set_title(string)      = 0;
    DBG_NEW
};


class AnchorBox: public GuiObject
{
 public:
    AnchorBox(const char* name, GuiObject* parent, MainFrame* frame);
    virtual ~AnchorBox();

    // as GuiObject
    virtual AnchorBox_if* interface();
    virtual void          create(GuiConfig*);

    // operations
    virtual void popup()   { interface()->popup(); }
    virtual void popdown() { interface()->popdown(); }
    virtual void clear()   { interface()->clear(); }
    virtual void set_node(Natto::node*);

    // callbacks
    virtual void activate(Natto::link*);
    virtual void select(Natto::link*);
    virtual void focus(Natto::link*);
    virtual void scan(vector<Natto::link*>&);
    virtual void browse(Natto::link*);

    // member access
    MainFrame*  frame()   { return frame_; }
    LayoutForm* form()    { return form_; }
    SiteBar*    sitebar() { return sitebar_; }

 private:
    MainFrame*  frame_;
    LayoutForm* form_;
    SiteBar*    sitebar_;

    AnchorBox(const AnchorBox&);
    AnchorBox& operator=(const AnchorBox&);
    DBG_NEW
};

#endif /* ANCHORBOX_H */
