/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef GENERICMAPPER_H
#define GENERICMAPPER_H
#ifdef __GNUG__
#pragma interface
#endif

#include "Mapper.h"

class GenericMapperBox;


class GenericMapper: public Mapper
{
 public:
    enum dimension_t {
	xaxis, yaxis, zaxis, radius, n_dimensions
    };
    enum parameter_t {
	profile, degree_of_interest, spring, file_name,
	content_length, last_modified, ratio_of_japanese, n_images, n_links,
	file_name_135, file_name_246, n_parameters
    };

    GenericMapper();
    virtual void configure(Natto*, GuiObject*);
    virtual void layout(Natto*, Natto::node*);
    virtual bool x_label(Natto*, int, float&, string&);
    virtual bool y_label(Natto*, int, float&, string&);
    void         set_title();

    parameter_t* mapping();
    void         mapping(parameter_t[n_dimensions]);

    float map_degree_of_interest(Natto::node*);
    float map_file_name(Natto::node*);
    float map_file_name_135(Natto::node*);
    float map_file_name_246(Natto::node*);
    float map_content_length(Natto::node*);
    float map_last_modified(Natto::node*);
    float map_n_images(Natto::node*);
    float map_n_links(Natto::node*);
    float map_log_n_links(Natto::node*);
    float map_ratio_of_japanese(Natto::node*);
    float map_dummy(Natto::node*);
    float map_alphabetical(string, unsigned = 0, unsigned = 1);
    float map_by_parameter(Natto::node*, parameter_t);

    void limit_file_name(string, string);

    bool label_alphabetical(int, float&, string&);
    bool label_content_length(int, float&, string&);
    bool label_last_modified(int, float&, string&);
    bool label_n_images(int, float&, string&);
    bool label_n_links(int, float&, string&);
    bool label_ratio_of_japanese(int, float&, string&);
    bool label_by_parameter(int, float&, string&, parameter_t);

    static char* dimension_names[n_dimensions];
    static char* parameter_names[n_parameters];
    static char* dimension_descriptions[n_dimensions];
    static char* parameter_descriptions[n_parameters];

 protected:
    GenericMapperBox* box_;
    time_t upto_, since_;
    parameter_t mapping_[n_dimensions];

    GenericMapper(const GenericMapper&);
    GenericMapper& operator=(const GenericMapper&);
    DBG_NEW
};

#endif /* GENERICMAPPER_H */
