/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_LinkExtractor_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "LinkExtractor.h"

#include "util.h"


bool LinkExtractor::class_initialized_ = false;
LinkExtractor::start_handlers_type LinkExtractor::start_handlers_;//(32);
LinkExtractor::end_handlers_type   LinkExtractor::end_handlers_;//(32);


LinkExtractor::LinkExtractor(
    const char* text,
    size_t      len,
    NetObject*  netobj):
    SGML(text, len), netobj_(netobj)
{
    if (!class_initialized_) class_init();
}


void
LinkExtractor::class_init()
{
    class_initialized_ = true;
    start_handlers_["HEAD"] = &start_head;
    start_handlers_["TITLE"] = &start_title;
    start_handlers_["LINK"] = &start_link;
    start_handlers_["BASE"] = &start_base;
    start_handlers_["BODY"] = &start_body;
    start_handlers_["H1"] = &start_hx;
    start_handlers_["H2"] = &start_hx;
    start_handlers_["H3"] = &start_hx;
    start_handlers_["H4"] = &start_hx;
    start_handlers_["H5"] = &start_hx;
    start_handlers_["H6"] = &start_hx;
    start_handlers_["A"] = &start_a;
    start_handlers_["IMG"] = &start_img;
    start_handlers_["AREA"] = &start_area;
    start_handlers_["FORM"] = &start_form;
    start_handlers_["INPUT"] = &start_input;
    start_handlers_["FRAME"] = &start_frame;
    start_handlers_["IFRAME"] = &start_iframe;
    start_handlers_["SCRIPT"] = &start_script;

    end_handlers_["TITLE"] = &end_title;
    end_handlers_["H1"] = &end_hx;
    end_handlers_["H2"] = &end_hx;
    end_handlers_["H3"] = &end_hx;
    end_handlers_["H4"] = &end_hx;
    end_handlers_["H5"] = &end_hx;
    end_handlers_["H6"] = &end_hx;
    end_handlers_["A"] = &end_a;
    end_handlers_["FORM"] = &end_form;
}


void
LinkExtractor::text_block(
    const char* p,
    const char* q)
{
    if (state_ & (hx_bit | title_bit))
        heading_ += string(p, q - p);
    if (state_ & a_bit)
        anchor_  += string(p, q - p);
}


void
LinkExtractor::start_element(
    string      elem,
    attrs_type& attrs)
{
    start_handlers_type::iterator i = start_handlers_.find(elem);
    if (i != start_handlers_.end()) (this->*(*i).second)(elem, attrs);
}


void
LinkExtractor::end_element(
    string elem)
{
    end_handlers_type::iterator i = end_handlers_.find(elem);
    if (i != end_handlers_.end()) (this->*(*i).second)(elem);
}


void
LinkExtractor::start_head(
    string,
    attrs_type&)
{
    state_ |= head_bit;
}


void
LinkExtractor::start_title(
    string,
    attrs_type&)
{
    state_ |= title_bit;
    heading_ = "";
}


void
LinkExtractor::start_link(
    string      elem,
    attrs_type& attrs)
{
    if (attrs.find("href") != attrs.end())
        this->hyperlink(elem, attrs["href"], attrs);
}


void
LinkExtractor::start_base(
    string,
    attrs_type& attrs)
{
    if (attrs.find("href") != attrs.end())
        this->base(attrs["href"]);
}


void
LinkExtractor::start_body(
    string      elem,
    attrs_type& attrs)
{
    if (state_ & head_bit) this->end_element("HEAD");
    state_ |= body_bit;
#if 0
    if (attrs.find("background") != attrs.end())
        this->hyperlink(elem, attrs["background"], attrs_type());
#endif
}


void
LinkExtractor::start_hx(
    string      elem,
    attrs_type& attrs)
{
    if (state_ & hx_bit) this->end_element(hx_name_);
    state_ |= hx_bit;
    hx_name_ = elem;
    heading_ = "";
}


void
LinkExtractor::start_a(
    string      elem,
    attrs_type& attrs)
{
    if (state_ & a_bit) this->end_element("A");
    state_ |= a_bit;
    a_attrs_ = attrs;
    anchor_ = "";
}


void
LinkExtractor::start_img(
    string      elem,
    attrs_type& attrs)
{
#if 0
    if (attrs.find("dynsrc") != attrs.end())
        this->hyperlink(elem, attrs["dynsrc"], attrs, attrs["alt"]);
    if (attrs.find("src") != attrs.end())
        this->hyperlink(elem, attrs["src"], attrs, attrs["alt"]);
#endif
    if (attrs.find("alt") != attrs.end()) {
        string alt = attrs["alt"];
        this->text_block(alt.c_str(), alt.c_str() + alt.length());
    }
}


void
LinkExtractor::start_area(
    string      elem,
    attrs_type& attrs)
{
    if (attrs.find("href") != attrs.end())
        this->hyperlink(elem, attrs["href"], attrs, attrs["alt"]);
}


void
LinkExtractor::start_form(
    string,
    attrs_type& attrs)
{
    if (state_ & form_bit) this->end_element("form");
    state_ |= form_bit;
    form_attrs_ = attrs;
}


void
LinkExtractor::start_input(
    string      elem,
    attrs_type& attrs)
{
    if ((state_ & form_bit) && attrs["type"] == "submit")
        this->hyperlink(
            elem, form_attrs_["action"], form_attrs_, attrs["name"]);
}


void
LinkExtractor::start_frame(
    string      elem,
    attrs_type& attrs)
{
    if (attrs.find("src") != attrs.end())
        this->hyperlink(elem, attrs["src"], attrs, attrs["name"]);
}


void
LinkExtractor::start_iframe(
    string      elem,
    attrs_type& attrs)
{
    if (attrs.find("src") != attrs.end())
        this->hyperlink(elem, attrs["src"], attrs, attrs["name"]);
}


void
LinkExtractor::start_script(
    string      elem,
    attrs_type& attrs)
{
#if 0
    if (attrs.find("src") != attrs.end())
        this->hyperlink(elem, attrs["src"], attrs, attrs["langage"]);
#endif
}


void
LinkExtractor::end_title(
    string)
{
    if (state_ & title_bit) this->title(heading_);
    state_ &= ~title_bit;
}


void
LinkExtractor::end_hx(
    string elem)
{
    if ((state_ & hx_bit) && hx_name_ == elem)
        this->heading(elem[1] - '0', heading_);
    state_ &= ~hx_bit;
}


void
LinkExtractor::end_a(
    string elem)
{
    if ((state_ & a_bit) && a_attrs_.find("href") != a_attrs_.end())
        this->hyperlink(elem, a_attrs_["href"], a_attrs_, anchor_);
    state_ &= ~a_bit;
}


void
LinkExtractor::end_form(
    string)
{
    state_ &= ~form_bit;
}


void
LinkExtractor::hyperlink(
    string      element,
    string      href,
    attrs_type& attrs,
    string      anchor)
{
    anchor = squeeze_space(anchor);
    href   = trim_space(href);

    string method = trim_space(attrs["method"]);
    string title  = squeeze_space(attrs["title"]);
    string mtype  = trim_space(attrs["mtype"]);
    string rel    = trim_space(attrs["rel"]);
    string rev    = trim_space(attrs["rev"]);
#if 0
    cout << "anchor: " << anchor << '\n'
         << "href  : " << href   << '\n'
         << "title : " << title  << '\n'
         << "mtype : " << mtype  << '\n'
         << "rel   : " << rel    << '\n'
         << "rev   : " << rev    << '\n' << endl;
#endif
    // if (href.compare("/internal-icon/", 15) != 0) return;
    netobj_->hyperlink(
        NetAnchor(href, NetAnchor::normal_link, anchor, title, mtype));
    rel = rev = "";
}


void
LinkExtractor::base(
    string href)
{
    netobj_->base(URL(href, netobj_->url()));
    // cout << "base  : " << trim_space(href) << '\n' << endl;
}


void
LinkExtractor::title(
    string str)
{
    netobj_->title(str);
    // cout << "title : " << squeeze_space(str) << '\n' << endl;
}


void
LinkExtractor::heading(
    int    level,
    string str)
{
    // cout << 'h' << level << "    : " << squeeze_space(str) << '\n' << endl;
}

