/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef LINKEXTRACTOR_H
#define LINKEXTRACTOR_H
#ifdef __GNUG__
#pragma interface
#endif

#include "SGML.h"
#include "NetObject.h"


class LinkExtractor: public SGML
{
 public:
    typedef void (LinkExtractor::*start_handler)(string, attrs_type&);
    typedef void (LinkExtractor::*end_handler)(string);
    typedef map<string, start_handler, less<string> > start_handlers_type;
    typedef map<string,   end_handler, less<string> >   end_handlers_type;

    LinkExtractor(textpos, size_t, NetObject*);

 protected:
    virtual void text_block(textpos, textpos);
    virtual void start_element(string, attrs_type&);
    virtual void end_element(string);

    virtual void start_head(string, attrs_type&);
    virtual void start_title(string, attrs_type&);
    virtual void start_link(string, attrs_type&);
    virtual void start_base(string, attrs_type&);
    virtual void start_body(string, attrs_type&);
    virtual void start_hx(string, attrs_type&);
    virtual void start_a(string, attrs_type&);
    virtual void start_img(string, attrs_type&);
    virtual void start_area(string, attrs_type&);
    virtual void start_form(string, attrs_type&);
    virtual void start_input(string, attrs_type&);
    virtual void start_frame(string, attrs_type&);
    virtual void start_iframe(string, attrs_type&);
    virtual void start_script(string, attrs_type&);

    virtual void end_title(string);
    virtual void end_hx(string);
    virtual void end_a(string);
    virtual void end_form(string);

    virtual void hyperlink(string elem, string href,
                           attrs_type& attrs, string anchor = "");
    virtual void title(string);
    virtual void heading(int, string);
    virtual void base(string);
    
    static void class_init();
    
 private:
    NetObject* netobj_;

    enum state_t {
	head_bit = 0x0001, body_bit = 0x0002,
        fremeset_bit = 0x0004, noframe_bit = 0x0008,
	title_bit = 0x0010, a_bit = 0x0020, hx_bit = 0x0040, form_bit = 0x0080,
	script_bit = 0x0100, applet_bit = 0x0200, object_bit = 0x0400 };

    int    state_;
    string hx_name_;
    string heading_;
    string anchor_;
    attrs_type a_attrs_;
    attrs_type form_attrs_;

    static bool class_initialized_;
    static start_handlers_type start_handlers_;
    static end_handlers_type   end_handlers_;

    LinkExtractor(const LinkExtractor&);
    LinkExtractor& operator=(const LinkExtractor&);
    DBG_NEW
};

#endif /* LINKEXTRACTOR_H */
