/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_MainFrame_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "MainFrame.h"

#include <fstream.h>
//#include "pipestream.h"
#include "util.h"
#include "utilgl.h"
#include "NattoViewGL.h"
#include "SpringMapper.h"
#include "GenericMapper.h"
#include "Preference.h"
#include "PortKit.h"
#include "ToolBar.h"
#include "SiteBar.h"
#include "ViewPalette.h"
#include "StatusBar.h"
#include "AnchorBox.h"


MainFrame::MainFrame(
    const char* name,
    GuiObject*  parent,
    Natto*        natto):

    GuiObject(name, parent),
    natto_(natto), last_update_(time(NULL)),
    form_(new LayoutForm("frame_form", this)),
    glframe_(new LayoutForm("glframe", form_)),
    rootview_(new NattoViewGL("glxarea", glframe_, false, true, this)),
    toolbar_(new ToolBar("toolbar", form_, this)),
    sitebar_(new SiteBar("sitebar", form_, this)),
    statusbar_(new StatusBar("statbar", form_)),
    viewpalette_(new ViewPalette("viewpalette", form_)),
    anchorbox_(NULL)
{
    this->natto()->attach(this);
}


MainFrame::~MainFrame()
{
    delete form_;
    delete glframe_;
    delete rootview_;
    delete toolbar_;
    delete sitebar_;
    delete statusbar_;
    delete viewpalette_;
    delete anchorbox_;
}


MainFrame_if*
MainFrame::interface()
{
    return DCAST(MainFrame_if*, imp()->interface());
}


void
MainFrame::create(
    GuiConfig* config)
{
    GuiObject::create(&PortKit::create_MainFrame, config);
}


void
MainFrame::rootview(
    NattoView* view)
{
    rootview_ = active_view_ = view;
}


void
MainFrame::active_view(
    NattoView* view)
{
    active_view_ = view;
}


void
MainFrame::form(
    LayoutForm* form)
{
    form_ = form;
}


void
MainFrame::glframe(
    LayoutForm* glframe)
{
    glframe_ = glframe;
}


void
MainFrame::toolbar(
    ToolBar* toolbar)
{
    toolbar_ = toolbar;
}


void
MainFrame::sitebar(
    SiteBar* sitebar)
{
    sitebar_ = sitebar;
}


void
MainFrame::statusbar(
    StatusBar* statusbar)
{
    statusbar_ = statusbar;
}


void
MainFrame::viewpalette(
    ViewPalette* viewpalette)
{
    viewpalette_ = viewpalette;
}


void
MainFrame::anchorbox(
    AnchorBox* anchorbox)
{
    anchorbox_ = anchorbox;
}


void
MainFrame::update(
    observable*,
    int priority,
    void*)
{
    if (!rootview_) return;
    if (priority <= Natto::pri_ignorable && time(NULL) - last_update_ < 1)
	return;
    rootview_->update();
    last_update_ = time(NULL);
    if (priority >= Natto::pri_sync) this->show_node_info();
    this->statusbar()->natto_size(this->natto());
}


void
MainFrame::focus_node(
    URL url)
{
    this->focus_node(natto()->find_node(url, true));
}


void
MainFrame::focus_node(
    Natto::node* vp)
{
    if (!vp) return;
    natto()->activate_node(vp);
    this->show_node_info();
}


void
MainFrame::focus_link(
    Natto::link* ep)
{
    if (ep) natto()->activate_link(ep);
}


void
MainFrame::scan(
    Natto::node* vp)
{
    if (vp) natto()->scan_node(vp, false);
}


void
MainFrame::stop_transfer()
{
    // LinkParser::kill_all();
}


#if 0
void
MainFrame::hide(
    Natto::node* vp)
{
}
#endif

void
MainFrame::browse(
    Natto::node* vp)
{
    if (!vp) vp = natto()->active_node();
    if (vp)  interface()->mozilla_open(vp->url());
}


void
MainFrame::save_file(
    const char* fname)
{
    ofstream ofs(fname);
    // if (ofs) ofs << *natto();
}


void
MainFrame::load_file(
    const char* fname)
{
    ifstream ifs(fname);
    // if (ifs) ifs >> *natto();
}


void
MainFrame::popup_anchorbox()
{
    anchorbox_->popup();
}


void
MainFrame::popup_tourbox()
{
    rootview()->popup_tourbox();
}


void
MainFrame::popup_mapperbox()
{
    natto()->mapper()->configure(natto(), this);
}


void
MainFrame::mapper_spring()
{
    Mapper* m = natto()->mapper();
    natto()->mapper(new SpringMapper(3));
    delete m;
    natto()->mapper()->layout_all(natto());
}


void
MainFrame::mapper_spring3d()
{
    Mapper* m = natto()->mapper();
    natto()->mapper(new SpringMapper(7));
    delete m;
    natto()->mapper()->layout_all(natto());
}


void
MainFrame::mapper_generic()
{
    Mapper* m = natto()->mapper();
    natto()->mapper(new GenericMapper);
    delete m;
    natto()->mapper()->layout_all(natto());
}


#include "pipestream.h"

void
MainFrame::import_icon()
{
    Natto::node* vp = natto()->active_node();
    if (!vp) return;

    char* argv[] = { "import", "-frame", "-verbose", "sgi:-", NULL };
    pipestream in("import", argv);

    GLint width, height;
    void* image = LoadSGIImage(in, width, height);
    in.close();
    unsigned char* buf = new unsigned char[64 * 64 * 4];
    gluScaleImage(GL_RGB, width, height, GL_UNSIGNED_BYTE, image,
		  64, 64, GL_UNSIGNED_BYTE, buf);
    NodeShapeGL* shape = DCAST(NodeShapeGL*, vp->shape());
    if (shape->tex_image) delete shape->tex_image;
    shape->tex_width  = 64;
    shape->tex_height = 64;
    shape->tex_image  = buf;
    delete[] image;
}


void
MainFrame::show_node_info()
{
    if (!imp()) return;

    Natto::node* vp = natto()->active_node();
    if (!vp) return;

    string title = "NattoView - " + vp->label();
    interface()->set_title(title.c_str());
    this->sitebar()->set_node(vp);
    this->anchorbox()->set_node(vp);
}


MainFrame_if::~MainFrame_if()
{}
