/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef MAINFRAME_H
#define MAINFRAME_H
#ifdef __GNUG__
#pragma interface
#endif

#include <time.h>
//#include "ptrvector.h"
#include "PortKit.h"
#include "Natto.h"

class NattoView;


class MainFrame_if: public GuiObject_if
{
 public:
    virtual ~MainFrame_if();

    virtual void set_title(string) = 0;
    virtual void mozilla_open(URL) = 0;
    DBG_NEW
};


class MainFrame: public GuiObject, public observer
{
 public:
    MainFrame(const char* name, GuiObject* parent, Natto*);
    virtual ~MainFrame();

    // as GuiObject
    virtual MainFrame_if* interface();
    virtual void          create(GuiConfig*);

    // as observer
    virtual void update(observable*, int priority, void* data);

    // operations
    virtual void focus_node(URL);
    virtual void focus_node(Natto::node*);
    virtual void focus_link(Natto::link*);
    virtual void lift_up(Natto::node* vp = NULL)  { natto()->lift_up(vp); }
    virtual void lift_down(Natto::node* vp = NULL){ natto()->lift_down(vp); }
    virtual void lift_down_all()                  { natto()->lift_down_all(); }
    virtual void scan(Natto::node* = NULL);
    virtual void browse(Natto::node* = NULL);
    virtual void stop_transfer();
    virtual void show_node_info();

    void save_file(const char* fname);
    void load_file(const char* fname);
    void quit() { ::ExitNattoView(); }

    void mapper_spring();
    void mapper_spring3d();
    void mapper_generic();
    void import_icon();

    virtual void popup_anchorbox();
    virtual void popup_tourbox();
    virtual void popup_mapperbox();
    
    // member access
    Natto*       natto()       { return natto_; }
    NattoView*   rootview()    { return rootview_; }
    NattoView*   active_view() { return active_view_; }
    LayoutForm*  form()        { return form_; }
    LayoutForm*  glframe()     { return glframe_; }
    ToolBar*     toolbar()     { return toolbar_; }
    SiteBar*     sitebar()     { return sitebar_; }
    StatusBar*   statusbar()   { return statusbar_; }
    ViewPalette* viewpalette() { return viewpalette_; }
    AnchorBox*   anchorbox()   { return anchorbox_; }

    virtual void rootview(NattoView*);
    virtual void active_view(NattoView*);
    virtual void form(LayoutForm*);
    virtual void glframe(LayoutForm*);
    virtual void toolbar(ToolBar*);
    virtual void sitebar(SiteBar*);
    virtual void statusbar(StatusBar*);
    virtual void viewpalette(ViewPalette*);
    virtual void anchorbox(AnchorBox*);

 private:
    Natto*       natto_;
    NattoView*   active_view_;
    time_t       last_update_;
    LayoutForm*  form_;
    LayoutForm*  glframe_;
    NattoView*   rootview_;
    ToolBar*     toolbar_;
    SiteBar*     sitebar_;
    StatusBar*   statusbar_;
    ViewPalette* viewpalette_;
    AnchorBox*   anchorbox_;

    MainFrame(const MainFrame&);
    MainFrame& operator=(const MainFrame&);
    DBG_NEW
};

#endif /* MAINFRAME_H */
