/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  OpenGL

#ifdef __GNUG__
#pragma implementation
int dummy_NattoViewGL_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "NattoViewGL.h"

#include <math.h>
#include <queue>
#include <GL/glu.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "utilstl.h"
#include "utilgl.h"
#include "Mapper.h"


#define invalid_name ((GLuint)(-1))

#define DEF_PERS_FOVY      40.0
#define DEF_PERS_NEAR       0.3 
#define DEF_PERS_FAR       10.0 


NattoViewGL::NattoViewGL(
    const char* name,
    GuiObject*  parent,
    bool        cmap,
    bool        dbuf,
    MainFrame*  frame):

    NattoView(name, parent, cmap, dbuf, frame),
    fill_bitmap_(NULL), fill_bytes_(0)
{}


NattoViewGL::NattoViewGL(
    const char*  name,
    GuiObject*   parent,
    NattoViewGL* view,
    int          x,
    int          y,
    int          width,
    int          height):

    NattoView(name, parent, view, x, y, width, height),
    fill_bitmap_(NULL), fill_bytes_(0)
{}


void
NattoViewGL::init_graphics()
{
    interface()->make_current();

    glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
    glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
    glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
    glHint(GL_FOG_HINT, GL_FASTEST);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
    
    glDepthFunc(GL_LEQUAL);
    glCullFace(GL_BACK);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glEnable(GL_LINE_SMOOTH);

    glLightfv(GL_LIGHT0, GL_AMBIENT,   light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE,   light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR,  light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION,  light_position);
    glLightf (GL_LIGHT0, GL_SHININESS, light_shininess);
    glEnable(GL_LIGHT0);

    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);

    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
#if 0    
    glFogi (GL_FOG_MODE,    fog_mode);
    glFogfv(GL_FOG_COLOR,   fog_color);
    glFogf (GL_FOG_DENSITY, fog_density);
#endif
    if (this->level() & 01) ClearColor(background1_color);
    else                    ClearColor(background0_color);

    PackingMode pm;
    pm.swap_bytes = GL_FALSE;
    pm.lsb_first  = GL_FALSE;
    pm.row_length = pm.skip_rows = pm.skip_pixels = 0;
    pm.alignment  = 1;
    SetUnpackingMode(pm);

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
}


void
NattoViewGL::resize(
    int x, int y,
    int w, int h)
{
    interface()->make_current();
    NattoView::resize(x, y, w, h);
    glViewport(0, 0, w, h);
}


#if 0

void
NattoViewGL::projection_matrix()
{
    GLdouble fovy   = DEF_PERS_FOVY;
    GLdouble aspect = (GLdouble) width() / height();
    gluPerspective(fovy, aspect, DEF_PERS_NEAR, DEF_PERS_FAR);
}


void
NattoViewGL::modelview_matrix()
{
    glTranslatef(trans().x, trans().y, trans().z);
    glRotatef(rot().x, 1.0, 0.0, 0.0);
    glRotatef(rot().y, 0.0, 1.0, 0.0);
    glRotatef(rot().z, 0.0, 0.0, 1.0);
    glTranslatef(-origin().x, -origin().y, -origin().z);
}

#else


void
NattoViewGL::projection_matrix()
{
    NattoView* root = frame()->rootview();
    
    GLdouble fovy   = DEF_PERS_FOVY;
#if 1
    GLdouble aspect = (GLdouble) root->height() / root->width();
    GLdouble xmax   = DEF_PERS_NEAR * tan(fovy * M_PI / 360.0);
    GLdouble xmin   = -xmax;
    GLdouble ymin   = xmin * aspect;
    GLdouble ymax   = xmax * aspect;
#else
    GLdouble aspect = (GLdouble) root->width() / root->height();
    GLdouble ymax   = DEF_PERS_NEAR * tan(fovy * M_PI / 360.0);
    GLdouble ymin   = -ymax;
    GLdouble xmin   = ymin * aspect;
    GLdouble xmax   = ymax * aspect;
#endif
    if (rootview()) {
	GLdouble dw = (ymax - ymin) / root->height();
	GLdouble dh = (xmax - xmin) / root->width();
	xmax = xmin + dh * (xpos() + width());
	xmin = xmin + dh *  xpos();
	ymax = ymin + dw * (ypos() + height());
	ymin = ymin + dw *  ypos();
    }
    DBG("mat", name() << " projection (x,y)-(w,h)=(" <<
        xpos() << ',' << ypos() << ")-(" << width() << ',' << height() << ')');

    if (this->render_mode() & ortho_proj_bit)
	glOrtho(xmin*6, xmax*6, ymin*6, ymax*6, DEF_PERS_NEAR, DEF_PERS_FAR);
    else
	glFrustum(xmin, xmax, ymin, ymax, DEF_PERS_NEAR, DEF_PERS_FAR);
}


void
NattoViewGL::modelview_matrix()
{
    NattoView* root = frame()->rootview();
    
    if (!rootview()) {
	glTranslatef(trans().x, trans().y, trans().z);
	glRotatef(rot().x, 1.0, 0.0, 0.0);
	glRotatef(rot().y, 0.0, 1.0, 0.0);
	glRotatef(rot().z, 0.0, 0.0, 1.0);
	glTranslatef(-origin().x, -origin().y, -origin().z);
    }
    else {
	const double d2r = M_PI / 180.0;
	glTranslatef(root->trans().x + trans().x,
		     root->trans().y + trans().y,
		     root->trans().z + trans().z);
	double l = hypot(origin().y, origin().x);
	double a = atan2(origin().y, origin().x);
	double x = l * cos(a + root->rot().z * d2r);
	double y = l * sin(a + root->rot().z * d2r);
	double l2 = hypot(origin().z, y);
	double a2 = atan2(origin().z, y);
	double z2 = l2 * sin(a2 + root->rot().x * d2r);
	double y2 = l2 * cos(a2 + root->rot().x * d2r);

	glTranslatef(0, y2, z2);
	glRotatef(rot().x, 1.0, 0.0, 0.0);
	glTranslatef(0, -y2, -z2);
	glRotatef(root->rot().x, 1.0, 0.0, 0.0);

	glTranslatef(x, y, 0);
	glRotatef(rot().z, 0.0, 0.0, 1.0);
	glTranslatef(-x, -y, 0);
	glRotatef(root->rot().z, 0.0, 0.0, 1.0);

	glTranslatef(- root->origin().x,
		     - root->origin().y,
		     - root->origin().z);
    }
}

#endif


void
NattoViewGL::update_prefix()
{
    interface()->make_current();
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    if (this->level() & 01) ClearColor(background1_color);
    else                    ClearColor(background0_color);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    this->projection_matrix();
}


void
NattoViewGL::draw_prefix()
{
    interface()->make_current();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    modelview_matrix();

    glInitNames();
    glPushName(invalid_name);

    glShadeModel(GL_SMOOTH);
}


void
NattoViewGL::draw_suffix()
{
    glPopName();
}


void
NattoViewGL::draw_nodes_prefix()
{
    clear_container(node_table_);
}


void
NattoViewGL::draw_a_node(
    Natto::node* vp)
{
    int n = node_table_.size();
    int name = node_bit | n;
    glLoadName(name);
    node_table_.push_back(vp);
    glPushMatrix();
    DBG("draw", this->name() << " node=" << n << " url=" << vp->url());

    if (!vp->shape()) vp->shape(new NodeShapeGL);
    NodeShapeGL* shape = DCAST(NodeShapeGL*, vp->shape());
    
    NattoView* root = frame()->rootview();
    glTranslatef(vp->x, vp->y, vp->z);
    glRotatef(180 - root->rot().z, 0.0, 0.0, 1.0);
    glRotatef(      root->rot().y, 0.0, 1.0, 0.0);
    glRotatef( 90 + root->rot().x, 1.0, 0.0, 0.0);

    if (vp->test_flags(Natto::shape_changed_bit)) {
	if (!shape->icon_list) shape->icon_list = glGenLists(1);
	glNewList(shape->icon_list, GL_COMPILE);
	this->draw_a_node_icon(vp);
	glEndList();
    }
    glCallList(shape->icon_list);

    if (!(render_mode() & draw_label_bit) /* || !status */) {
	glPopMatrix();
	glLoadName(invalid_name);
	return;
    }

    glTranslatef(0, 0, vp->r + 0.01);
    if (vp->test_flags(Natto::shape_changed_bit)) {
	if (!shape->label_list) shape->label_list = glGenLists(1);
	glNewList(shape->label_list, GL_COMPILE);
	this->draw_a_node_label(vp);
	glEndList();
    }
    glCallList(shape->label_list);
    
    glPopMatrix();
    glLoadName(invalid_name);
}


void
NattoViewGL::draw_a_node_icon(
    Natto::node* vp)
{
    NodeShapeGL* shape = DCAST(NodeShapeGL*, vp->shape());

    if ((render_mode() & draw_icon_bit) && shape->tex_image) {
	float r = vp->r;
	glTexImage2D(GL_TEXTURE_2D, 0, 4,
                     shape->tex_width, shape->tex_height, 0,
		     GL_RGB, GL_UNSIGNED_BYTE, shape->tex_image);
	glEnable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0, 0.0);  v3f( r, r/3, -r);
	glTexCoord2f(1.0, 0.0);  v3f(-r, r/3, -r);
	glTexCoord2f(1.0, 1.0);  v3f(-r, r/3,  r);
	glTexCoord2f(0.0, 1.0);  v3f( r, r/3,  r);
	glEnd();
	glDisable(GL_TEXTURE_2D);
    }
    else {
	glEnable(GL_LIGHTING);
	c3fv(vp->test_flags(Natto::active_bit)  ? active_node_color:
	     vp->test_flags(Natto::marked_bit)  ? marked_node_color:
	     vp->test_flags(Natto::scanned_bit) ? scanned_node_color:
	     /* normal node */                    normal_node_color);
	SolidSphere_icosahedron(vp->r, 1);
	// SolidCube(vp->r);
	glDisable(GL_LIGHTING);
    }
}


void
NattoViewGL::draw_a_node_label(
    Natto::node* vp)
{
    string label = vp->label(vp->test_flags(Natto::active_bit) ? 0: 8);
    if (vp->test_flags(Natto::marked_bit))
	label += '(' + vp->mark_keys.back().substr(0, 8) + ')';

    int width = 1, height = 1, base_line = 0;
    GLubyte* bitmap =
	(GLubyte*) string_bitmap(label, width, height, base_line);

    // plate
    int bytes = (width + 4 + 7) / 8 * (height + 2);
    if (bytes > fill_bytes_) {
	delete[] fill_bitmap_;
	fill_bytes_ = bytes;
	fill_bitmap_ = new GLubyte[fill_bytes_];
	memset(fill_bitmap_, 0xff, fill_bytes_);
    }
    c3fv(vp->test_flags(Natto::active_bit) ? active_plate_color:
	 vp->test_flags(Natto::marked_bit) ? marked_plate_color:
	 /* normal node */                    popup_plate_color);
    glRasterPos3i(0, 0, 0);
    glBitmap(0, 0, 0, 0, -3 -2, -base_line /* -1 */, NULL);  // Mesa bug?
    glBitmap(width + 4, height /* + 2 */, 0, 0, 0, 0, fill_bitmap_);

    // label
    c3fv(popup_text_color);
    glRasterPos3i(0, 0, 0);
    glBitmap(0, 0, 0, 0, -3, -base_line, NULL);  // Mesa bug?
    glBitmap(width, height, 0, 0, 0, 0, bitmap);
    delete[] bitmap;
}


void
NattoViewGL::draw_nodes_suffix()
{}


void
NattoViewGL::draw_links_prefix()
{
    glDepthMask(GL_FALSE);
    glEnable(GL_BLEND);
    clear_container(link_table_);
}

    
void
NattoViewGL::draw_a_link(
    Natto::link* ep)
{
    int n = link_table_.size();
    DBG("draw", this->name() << " link=" << n << " href=" << ep->href());

    int name = link_bit | n;
    glLoadName(name);
    link_table_.push_back(ep);

    if (!ep->shape()) ep->shape(new LinkShapeGL);
    
    this->draw_a_link_line(ep);
    glLoadName(invalid_name);
}

		   
void
NattoViewGL::draw_a_link_line(
    Natto::link* ep)
{
    glLineWidth(ep->test_flags(Natto::active_bit) ? active_link_width:
		/* normal link */                   normal_link_width);

    Natto::node* from = ep->from();
    Natto::node* to   = ep->to();
    GLfloat* color1 = ep->test_flags(Natto::active_bit) ?
        active_link1_color: normal_link1_color;
    GLfloat* color2 = ep->test_flags(Natto::active_bit) ?
        active_link2_color: normal_link2_color;
    glBegin(GL_LINES);
    c4fv(color1);  v3f(from->x, from->y, from->z);
    c4fv(color2);  v3f(to->x,   to->y,   to->z);
    glEnd();
}


void
NattoViewGL::draw_links_suffix()
{
    glDisable(GL_BLEND);
    glDepthMask(GL_TRUE);
}


void
NattoViewGL::draw_board()
{
    glPushMatrix();

    c3fv(strong_text_color);
    glLineWidth(normal_text_width);
    StrokeText( 0.20, -0.10, z_board(),  0.0, 0.0004,
	       natto()->mapper()->x_title());
    StrokeText(-0.05,  0.20, z_board(), 90.0, 0.0004,
	       natto()->mapper()->y_title());

    float  pos;
    string label;
    int i;
    for (i = 0; natto()->mapper()->x_label(natto(), i, pos, label); i++) {
	glLineWidth(1);
	c3fv(grid_color);
	glBegin(GL_LINES);
	v3f(pos, 1.00, z_board());
	v3f(pos, 1.05, z_board());
	glEnd();

	glLineWidth(normal_text_width);
	c3fv(normal_text_color);
	StrokeText(pos, 1.05, z_board(), 40.0, 0.0003, label);
    }
    for (i = 0; natto()->mapper()->y_label(natto(), i, pos, label); i++) {
	glLineWidth(1);
	c3fv(grid_color);
	glBegin(GL_LINES);
	v3f(1.00, pos, z_board());
	v3f(1.05, pos, z_board());
	glEnd();

	glLineWidth(normal_text_width);
	c3fv(normal_text_color);
	StrokeText(1.05, pos, z_board(), 40.0, 0.0003, label);
    }
    c3fv(grid_color);
    glBegin(GL_LINE_LOOP);
    v3f(0.0, 0.0, z_board());
    v3f(1.0, 0.0, z_board());
    v3f(1.0, 1.0, z_board());
    v3f(0.0, 1.0, z_board());
    glEnd();

    n3f(0.0, 0.0, 1.0);
    glBegin(GL_QUADS);
    c3fv(board1_color);  v3f(0.0, 0.0, z_board());
    c3fv(board2_color);  v3f(1.0, 0.0, z_board());
    c3fv(board3_color);  v3f(1.0, 1.0, z_board());
    c3fv(board4_color);  v3f(0.0, 1.0, z_board());
    glEnd();

    glPopMatrix();
}


void
NattoViewGL::draw_cursor()
{
    if (!natto()->active_node()) return;
    GLfloat x = natto()->active_node()->x;
    GLfloat y = natto()->active_node()->y;
    GLfloat z = z_board();
    c3fv(cursor_color);
    glLineWidth(cursor_width);
    glLoadName(cursor);
    glBegin(GL_LINES);
    v3f(x, y, 10.0);
    v3f(x, y, z);
    glEnd();
#if 0
    glBegin(GL_LINE_STRIP);
    v3f(0.0, y, 0.0);
    v3f(x,   y, 0.0);
    v3f(x, 0.0, 0.0);
    glEnd();
#endif
    glLoadName(invalid_name);
}


void
NattoViewGL::pick(
    int  x,
    int  y,
    vector<int>& sysobjs,
    vector<Natto::node*>& nodes,
    vector<Natto::link*>& links)
{
    GLuint sel_buf[SEL_BUF_SIZE];
    msgout << "(" << x << ", " << y << ") selected." << endl;
    DBG("pick", name() << " picking (x,y)=(" << x << ',' << y << ')');

    interface()->make_current();
    glSelectBuffer(sizeof(sel_buf), sel_buf);
    (void) glRenderMode(GL_SELECT);
    GLint viewport[4];
    // glGetIntegerv(GL_VIEWPORT, viewport);
    viewport[0] = viewport[1] = 0;
    viewport[2] = width();
    viewport[3] = height();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // gluPickMatrix(x, y, 5, 5, viewport);
    gluPickMatrix(x, y, 1, 1, viewport);
    this->projection_matrix();
    this->draw();
    glFlush();

    GLint nhit = glRenderMode(GL_RENDER);
    DBG("pick", name() << " #hits=" << nhit);
    priority_queue<Hit, vector<Hit>, greater<Hit> > hits;
    // priority_queue<vector<Hit>, greater<Hit> > hits;
    GLuint* name_ptr = sel_buf;
    for (GLint i = 0; i < nhit; i++) {
	GLuint nname = *name_ptr++;
	GLuint z     = *name_ptr++;
	/* z2 */        name_ptr++;
	for (GLuint j = 0; j < nname; j++) {
	    GLuint n = *name_ptr++;
	    if (n == invalid_name) continue;
	    Hit h;
	    h.z = z;
	    h.name = n;
	    hits.push(h);
	}
    }
    
    for (Hit h; hits.size(); hits.pop()) {
	h = hits.top();
	if (h.name & sysobj_bit) {
	    int sysobj_no = h.name ^ sysobj_bit;
	    sysobjs.push_back(sysobj_no);
	    DBG("pick", name() << " z=" << h.z << " sysobj " << sysobj_no);
	    continue;
	}
	if (h.name & node_bit) {
	    int node_no = h.name ^ node_bit;
	    nodes.push_back(node_table_[node_no]);
	    DBG("pick", name() << " z=" << h.z <<
                " node " << node_table_[node_no]->url());
	    continue;
	}
	if (h.name & link_bit) {
	    int link_no = h.name ^ link_bit;
	    links.push_back(link_table_[link_no]);
	    DBG("pick", name() << " z=" << h.z <<
                " link " << link_table_[link_no]->from()->url() <<
                " -> " << link_table_[link_no]->href());
	    continue;
	}
	DBG("pick", name() << "z=" << h.z << " unknown object " << h.name);
    }
    return;
}


void
NattoViewGL::get_raster_pos(
    Natto::node* vp,
    float&     x,
    float&     y)
{
    if (!vp) return;
    interface()->make_current();
    glRasterPos3f(vp->x, vp->y, vp->z);
    GLfloat pos[4];
    glGetFloatv(GL_CURRENT_RASTER_POSITION, pos);
    x = pos[0];
    y = pos[1];
}


void
NattoViewGL::viewable_nodes(
    vector<Natto::node*>& vnodes)
{
    GLfloat matrix[16];
    interface()->make_current();

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    modelview_matrix();
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    GLdouble modelview[16];
    int i;
    for (i = 0; i < 16; i++) modelview[i] = matrix[i];

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    projection_matrix();
    glGetFloatv(GL_PROJECTION_MATRIX, matrix);
    GLdouble projection[16];
    for (i = 0; i < 16; i++) projection[i] = matrix[i];

    GLint viewport[4];
    viewport[0] = viewport[1] = 0;
    viewport[2] = width();
    viewport[3] = height();

    Natto::node_iter vi  = natto()->nodes_begin();
    Natto::node_iter end = natto()->nodes_end();
    for (; vi != end; vi++) {
	if (!(*vi)->test_flags(Natto::drawn_bit)) continue;
	GLdouble x = 0, y = 0, z = 0;
	GLint s = gluProject((*vi)->x, (*vi)->y, (*vi)->z,
			     modelview, projection, viewport, &x, &y, &z);
	if (s && 0 <= x && x <= width() && 0 <= y && y <= height())
	    vnodes.push_back(*vi);
    }
    return;
}
