/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef NATTOVIEWGL_H
#define NATTOVIEWGL_H
#ifdef __GNUG__
#pragma interface
#endif

//////////////////////////////////////////////////////////////////////////////
//  NattoViewGL.h -- OpenGL specific display routines
//

#include <GL/gl.h>
#include "NattoView.h"


class NodeShapeGL: public NodeShape
{
 public:
    NodeShapeGL(): icon_list(0), label_list(0),
        tex_width(0), tex_height(0), tex_image(0) {}
    
    unsigned icon_list;   // display list of node icon
    unsigned label_list;  // display list of node label

    int   tex_width;
    int   tex_height;
    void* tex_image;

 private:
    NodeShapeGL(const NodeShapeGL&);
    NodeShapeGL& operator=(const NodeShapeGL&);
    DBG_NEW
};


class LinkShapeGL: public LinkShape
{
 public:
    LinkShapeGL() {}
    DBG_NEW
};



//////////////////////////////////////////////////////////////////////////////
//  NattoViewGL --
//
class NattoViewGL: public NattoView
{
 public:
    // primitive functions
    NattoViewGL(const char* name, GuiObject* parent,
		bool install_cmap, bool double_buffer, MainFrame* frame);
    NattoViewGL(const char* name, GuiObject* parent,
		NattoViewGL* viewgl, int x, int y, int width, int height);
    
    // picking objects
    virtual void pick(int x, int y, vector<int>& sysobjs,
		      vector<Natto::node*>& nodes,
                      vector<Natto::link*>& links);
    virtual void viewable_nodes(vector<Natto::node*>&);
    struct Hit { GLuint z, name; };

    // OpenGL buffer operations
    void get_raster_pos(Natto::node* vp, float& x, float& y);

    // initialization of the GL state machine
    void init_graphics();
    
 protected:
    // utility routines
    virtual void resize(int x, int y, int width, int height);
    void         projection_matrix();
    void         modelview_matrix();
    
    // OpenGL buffer operations
    virtual void* string_bitmap(string str, int& w, int& h, int& base) {
	return interface()->string_bitmap(str, w, h, base);
    }
    
    // subroutines of update()
    virtual void update_prefix();
    
    // subroutines of draw() (overrided)
    virtual void draw_prefix();
    virtual void draw_suffix();
    virtual void draw_nodes_prefix();
    virtual void draw_nodes_suffix();
    virtual void draw_a_node(Natto::node* vp);
    virtual void draw_links_prefix();
    virtual void draw_links_suffix();
    virtual void draw_a_link(Natto::link* ep);
    virtual void draw_board();
    virtual void draw_cursor();

    //
    virtual void draw_a_node_icon (Natto::node* vp);
    virtual void draw_a_node_label(Natto::node* vp);
    virtual void draw_a_link_line (Natto::link* ep);
    
    enum { SEL_BUF_SIZE = 2048 };

    enum name_t {
	node_bit   = (1 << 30),
	link_bit   = (1 << 29),
	sysobj_bit = (1 << 28),
	cursor     = (sysobj_bit | 1)
    };

 private:
    GLubyte* fill_bitmap_;  // for background of name plates 
    int      fill_bytes_;   // size of fill_bitmap_ in bytes
    vector<Natto::node*> node_table_;  // GLname-onto-node table
    vector<Natto::link*> link_table_;  // GLname-onto-link table

    NattoViewGL(const NattoViewGL&);
    NattoViewGL& operator=(const NattoViewGL&);
    DBG_NEW
};


inline bool
operator<(
    const NattoViewGL::Hit& h1,
    const NattoViewGL::Hit& h2)
{
    return h1.z < h2.z;
}

#endif /* NATTOVIEWGL_H */

