/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  NetObj

#ifdef __GNUG__
#pragma implementation
int dummy_NetObject_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "NetObject.h"

#include <algorithm>
#include "NetManager.h"
#include "LinkExtractor.h"


NetObject::netobjs_type NetObject::netobjs_;


NetAnchor::NetAnchor()
{}


NetAnchor::NetAnchor(
    string      href,
    link_type_t ltype,
    string      anchor,
    string      title,
    string      mtype,
    link_type_t rev):
    href_(href), link_type_(ltype),
    anchor_(anchor), title_(title), media_type_(mtype), rev_(rev)
{
    DBG("anc", href_ << " creating");
}


NetAnchor::~NetAnchor()
{}


NetObject::NetObject(
    URL url):
    url_(url), status_(0), location_(NULL), headers_(NULL),
    content_length_(0), date_(0), expires_(0), last_modified_(0)
{
    url_.frag("");
    DBG("obj", url_ << " creating");
    const pair<const string, NetObject*> elem(url.abs(), this);
    netobjs_.insert(elem);;
}


NetObject::~NetObject()
{
    DBG("obj", url_ << " destroying");
    netobjs_type::iterator i = netobjs_.find(this->url().abs());
    netobjs_.erase(i);
    delete headers_;
}


void
NetObject::download(
    bool notifying)
{
    DBG("obj", url_ << " downloading");

    NetRequest  req(this->url());
    NetResponse res;
    ::netmanager->request(req, res);
    status_ = res.status();

    if (res.message()) {
        res.message()->freeze_headers();
        headers_ = res.message()->headers();
        this->cache_headers();

        memobject* content = res.message()->content();
        if (content && content->area()) {
            DBG("obj", url_ << " parsing HTML");
            LinkExtractor html((char*) content->area(), content->size(), this);
            html.parse();
        }
    }
    this->changed(true);
    this->notify();
}


void
NetObject::cache_headers()
{
    content_type_   = "";
    content_length_ = 0;
    date_           = 0;
    expires_        = 0;
    last_modified_  = 0;

    string str;
    size_t siz = 0;
    time_t tim = 0;
    if (headers_->get_Content_Type(str))   content_type_   = str;
    if (headers_->get_Content_Length(siz)) content_length_ = siz;
    if (headers_->get_Date(tim))           date_           = tim;
    if (headers_->get_Expires(tim))        expires_        = tim;

    URL url;
    if (headers_->get_Location(url)) {
        location_ = find_by_url(url);
        if (!location_) location_ = new NetObject(url);
    }
    this->changed(true);
}


void
NetObject::base(
    URL url)
{
    base_ = url;
    this->changed(true);
}


void
NetObject::title(
    string str)
{
    title_ = str;
    this->changed(true);
}


void
NetObject::hyperlink(
    const NetAnchor& anchor)
{
    anchors_.push_back(anchor);
    this->changed(true);
}


NetObject*
NetObject::find_by_url(
    URL url)
{
    url.frag("");
    DBG("obj", "finding url=" << url);
    netobjs_type::iterator i = netobjs_.find(url.abs());
    return i == netobjs_.end() ? (NetObject*) 0: (*i).second;
}


void
NetObject::clear_database()
{
    for (netobjs_type::iterator i = netobjs_.begin(); i != netobjs_.end(); i++)
        delete (*i).second;
}
