/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  GUI

#ifdef __GNUG__
#pragma implementation
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "PortKit.h"

#ifndef NULL
#define NULL 0
#endif


PortKit::~PortKit()
{}


GuiConfig::~GuiConfig()
{}


GuiObject::GuiObject(
    const char* name,
    GuiObject*  parent):
    imp_(NULL), parent_(parent)
{
    DBG("obj", name << " creating");
    name_ = new char[strlen(name) + 1];
    strcpy(name_, name);
}


GuiObject::~GuiObject()
{
    DBG("obj", name_ << " destroying");
    delete imp_;
    delete[] name_;
}


GuiObject_imp*
GuiObject::imp()
{
    return imp_;
}


GuiObject*
GuiObject::parent()
{
    return parent_;
}


PortKit*
GuiObject::kit()
{
    if (imp_)    return imp_->kit();
    if (parent_) return parent_->kit();
    /* else */   return ::portkit;
}


void
GuiObject::create(
    GuiObject_imp* (PortKit::* func)(GuiObject*),
    GuiConfig* config)
{
    if (!imp_) imp_ = (kit()->*func)(this);
    imp_->init(config);
}


void
GuiObject::create(
    GuiObject_imp* imp)
{
    if (!imp_) {
	imp_ = imp;
	imp_ ->obj_ = this;
    }
}


char*
GuiObject::name()
{
    return name_;
}


void
GuiObject::manage()
{
    if (imp_) imp_->manage();
}


void
GuiObject::show()
{
    if (imp_) imp_->show();
}


GuiObject_if::~GuiObject_if()
{}


GuiObject_imp::GuiObject_imp(
    GuiObject* obj):
    obj_(obj)
{
    DBG("imp", obj_->name() << " creating");
}


GuiObject_imp::~GuiObject_imp()
{
    DBG("imp", obj_->name() << " destroying");
}


void
GuiObject_imp::init(GuiConfig*)
{}


GuiObject*
GuiObject_imp::obj()
{
    return obj_;
}



LayoutForm::LayoutForm(
    const char* name,
    GuiObject*  parent):
    GuiObject(name, parent)
{}


void
LayoutForm::create(
    GuiConfig* config)
{
    GuiObject::create(&PortKit::create_LayoutForm, config);
}


LayoutForm_if::~LayoutForm_if()
{}
