/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef PORTKIT_H
#define PORTKIT_H
#ifdef __GNUG__
#pragma interface
#endif

class GuiConfig;
class GuiObject;        class GuiObject_imp;

class LayoutForm;       class LayoutForm_if;
class MainFrame;        class MainFrame_if;
class NattoView;        class NattoView_if;
class ToolBar;          class ToolBar_if;
class SiteBar;          class SiteBar_if;
class ViewPalette;      class ViewPalette_if;
class StatusBar;        class StatusBar_if;
class AnchorBox;        class AnchorBox_if;
class TourBox;          class TourBox_if;
class GenericMapperBox; class GenericMapperBox_if;


class PortKit
{
 public:
    virtual ~PortKit();

    //virtual EventManager_imp* create_EventManager() = 0;
    virtual GuiObject_imp*    create_LayoutForm(GuiObject*) = 0;
    virtual GuiObject_imp*    create_MainFrame(GuiObject*) = 0;
    virtual GuiObject_imp*    create_NattoView(GuiObject*) = 0;
    virtual GuiObject_imp*    create_ToolBar(GuiObject*) = 0;
    virtual GuiObject_imp*    create_SiteBar(GuiObject*) = 0;
    virtual GuiObject_imp*    create_ViewPalette(GuiObject*) = 0;
    virtual GuiObject_imp*    create_StatusBar(GuiObject*) = 0;
    virtual GuiObject_imp*    create_AnchorBox(GuiObject*) = 0;
    virtual GuiObject_imp*    create_TourBox(GuiObject*) = 0;
    virtual GuiObject_imp*    create_GenericMapperBox(GuiObject*) = 0;

    DBG_NEW
};

EXTERN PortKit* portkit;



class GuiObject
{
 public:
    GuiObject(const char* name, GuiObject* parent);
    virtual ~GuiObject();

    virtual GuiObject_imp* imp();
    virtual char*          name();
    virtual GuiObject*     parent();
    virtual PortKit*       kit();
    
    virtual void create(GuiConfig*) = 0;
    virtual void manage();
    virtual void show();

 protected:
    virtual void create(GuiObject_imp* (PortKit::*)(GuiObject*), GuiConfig*);
    virtual void create(GuiObject_imp*);

 private:
    GuiObject_imp* imp_;
    char*          name_;
    GuiObject*     parent_;

    GuiObject(const GuiObject&);
    GuiObject& operator=(const GuiObject&);
    DBG_NEW
};


class GuiObject_if
{
 public:
    virtual ~GuiObject_if();
    DBG_NEW
};


class GuiObject_imp
{
    friend GuiObject;

 public:
    explicit GuiObject_imp(GuiObject*);
    virtual ~GuiObject_imp();
    virtual void init(GuiConfig*);

    virtual GuiObject*    obj();
    virtual GuiObject_if* interface() = 0;
    virtual PortKit*      kit() = 0;

    virtual void manage() = 0;
    virtual void show() = 0;

 private:
    GuiObject* obj_;

    GuiObject_imp(const GuiObject_imp&);
    GuiObject_imp& operator=(const GuiObject_imp&);
    DBG_NEW
};



class GuiConfig
{
 public:
    virtual ~GuiConfig();
    DBG_NEW
};


class LayoutForm: public GuiObject
{
 public:
    LayoutForm(const char* name, GuiObject* parent);

    virtual void create(GuiConfig*);
    DBG_NEW
};


class LayoutForm_if: public GuiObject_if
{
 public:
    virtual ~LayoutForm_if();
    DBG_NEW
};

#endif /* PORTKIT_H */
