/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_StatusBar_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "StatusBar.h"

#include "MainFrame.h"
#include "util.h"


StatusBar::StatusBar(
    const char* name,
    GuiObject*  parent):
    GuiObject(name, parent), old_rdbuf_(NULL)
{}


StatusBar::~StatusBar()
{
    delete msgout.rdbuf(old_rdbuf_);
    old_rdbuf_ = NULL;
}


StatusBar_if*
StatusBar::interface()
{
    return DCAST(StatusBar_if*, imp()->interface());
}


void
StatusBar::create(
    GuiConfig* config)
{
    GuiObject::create(&PortKit::create_StatusBar, config);
    old_rdbuf_ = msgout.rdbuf(new StatusBarBuf(this));
}


void
StatusBar::message(
    string msg)
{
    interface()->message(msg);
}


void
StatusBar::natto_size(
    Natto* natto)
{
    interface()->natto_size(natto);
}


StatusBar_if::~StatusBar_if()
{}


StatusBarBuf::StatusBarBuf(
    StatusBar* statbar):
    statbar_(statbar), tempbuf_(new strstream)
{}


StatusBarBuf::~StatusBarBuf()
{
    delete tempbuf_;
}


int
StatusBarBuf::sync()
{
    streamsize n = this->pptr() - this->pbase();
    tempbuf_->write(this->pbase(),  n);
    tempbuf_->put('\0');
    statbar_->message(tempbuf_->str());
    tempbuf_->freeze(false);
    delete tempbuf_;
    tempbuf_ = new strstream;
    return 0;
}


int
StatusBarBuf::overflow(
    int ch)
{
    streamsize n = this->pptr() - this->pbase();
    if (n && this->sync() != 0) return EOF;
    if (ch != EOF) tempbuf_->put(ch);
    this->pbump(-n);
    return 0;
}


streamsize
StatusBarBuf::xsputn(
    char*      text,
    streamsize n)
{
    if (this->sync() == EOF) return 0;
    tempbuf_->write(text, n);
    return tempbuf_->pcount();
}
