/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_TourBox_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "TourBox.h"

#include "NattoView.h"
#include "SiteBar.h"


TourBox::TourBox(
    const char* name,
    GuiObject*  parent,
    NattoView*  view):

    GuiObject(name, parent), view_(view),
    form_(new LayoutForm("tourbox_form", this))
{}


TourBox::~TourBox()
{
    delete form_;
}


TourBox_if*
TourBox::interface()
{
    return DCAST(TourBox_if*, imp()->interface());
}


void
TourBox::create(
    GuiConfig* config)
{
    GuiObject::create(&PortKit::create_TourBox, config);
}


void
TourBox::activate(
    Natto::node* vp)
{
    this->select(vp);
    view_->frame()->browse(vp);
}


void
TourBox::select(
    Natto::node* vp)
{
    view_->frame()->focus_node(vp);
}


void
TourBox::scan(
    vector<Natto::node*>& nodes)
{
    vector<Natto::node*>::iterator vi;
    int i;
    for (vi = nodes.begin(), i = 0; vi != nodes.end(); vi++, i++) {
	interface()->unhighlight();
	interface()->highlight(i);
	view_->frame()->scan(*vi);
    }
}


void
TourBox::browse(
    Natto::node* vp)
{
    view_->frame()->scan(vp);
    view_->frame()->browse(vp);
}


void
TourBox::update(
    vector<Natto::node*>& items,
    int i)
{
    this->clear();
    view_->viewable_nodes(items);
    interface()->make_listing(i);
}


TourBox_if::~TourBox_if()
{}
