/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef TOURBOX_H
#define TOURBOX_H
#ifdef __GNUG__
#pragma interface
#endif

#include <vector>
#include "PortKit.h"
#include "Natto.h"


class TourBox_if: public GuiObject_if
{
 public:
    virtual ~TourBox_if();

    virtual void popup()              = 0;
    virtual void popdown()            = 0;
    virtual void clear()              = 0;
    virtual void highlight(int)       = 0;
    virtual void unhighlight()        = 0;
    virtual void set_title(string)    = 0;
    virtual void make_listing(int)    = 0;
    DBG_NEW
};


class TourBox: public GuiObject
{
 public:
    TourBox(const char* name, GuiObject* parent, NattoView* view);
    virtual ~TourBox();

    // as GuiObject
    virtual TourBox_if* interface();
    virtual void        create(GuiConfig*);

    // operations
    virtual void popup()   { interface()->popup(); }
    virtual void popdown() { interface()->popdown(); }
    virtual void clear()   { interface()->clear(); }

    // callbacks
    virtual void activate(Natto::node*);
    virtual void select(Natto::node*);
    virtual void scan(vector<Natto::node*>&);
    virtual void browse(Natto::node*);
    virtual void update(vector<Natto::node*>&, int);

    // member access
    NattoView*  view() { return view_; }
    LayoutForm* form() { return form_; }

 private:
    NattoView*  view_;
    LayoutForm* form_;

    TourBox(const TourBox&);
    TourBox& operator=(const TourBox&);
    DBG_NEW
};

#endif /* TOURBOX_H */
