/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_ViewPalette_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "ViewPalette.h"

#include "NattoView.h"


ViewPalette::ViewPalette(
    const char* name,
    GuiObject*  parent):
    GuiObject(name, parent), view_(NULL)
{}


ViewPalette_if*
ViewPalette::interface()
{
    return DCAST(ViewPalette_if*, imp()->interface());
}


void
ViewPalette::create(
    GuiConfig* config)
{
    GuiObject::create(&PortKit::create_ViewPalette, config);
}


void
ViewPalette::view(
    NattoView* view)
{
    view_ = view;
    interface()->set_render_mode();
}


void
ViewPalette::lift_up()
{
    view_->lift_up();
    view_->update();
}


void
ViewPalette::lift_down()
{
    view_->lift_down();
    view_->update();
}


void
ViewPalette::lift_down_all()
{
    view_->lift_down_all();
    view_->update();
}


void
ViewPalette::home_view()
{
    view_->home_view();
    view_->update();
}


void
ViewPalette::top_view()
{
    view_->top_view();
    view_->update();
}


void
ViewPalette::top_view_2d()
{
    view_->top_view_2d();
    view_->update();
}


void
ViewPalette::zoom_in()
{
    view_->zoom_in();
    view_->update();
}


void
ViewPalette::zoom_out()
{
    view_->zoom_out();
    view_->update();
}


void
ViewPalette::wireframe(
    bool b)
{
    if (b) view_->render_mode(NattoView::wireframe_bit, 0);
    else   view_->render_mode(0, NattoView::wireframe_bit);
    view_->update();
}


bool
ViewPalette::wireframe()
{
    return view_->render_mode() & NattoView::wireframe_bit;
}


void
ViewPalette::draw_icon(
    bool b)
{
    if (b) view_->render_mode(NattoView::draw_icon_bit, 0);
    else   view_->render_mode(0, NattoView::draw_icon_bit);
    view_->update();
}


bool
ViewPalette::draw_icon()
{
    return view_->render_mode() & NattoView::draw_icon_bit;
}


void
ViewPalette::draw_label(
    bool b)
{
    if (b) view_->render_mode(NattoView::draw_label_bit, 0);
    else   view_->render_mode(0, NattoView::draw_label_bit);
    view_->update();
}


bool
ViewPalette::draw_label()
{
    return view_->render_mode() & NattoView::draw_label_bit;
}


ViewPalette_if::~ViewPalette_if()
{}
