/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef DFS_H
#define DFS_H
#ifdef __GNUG__
#pragma interface
#endif

//////////////////////////////////////////////////////////////////////////////
//  dfs.h -- depth first search for graph
//

#include <limits.h>
#include "graph.h"


enum dfs_return { dfs_next, dfs_found, dfs_visited };


template <class Vertex, class Finder, class Iterator>
Vertex*
dfs_forward_aux(
    Vertex*  vp,
    Finder&  finder,
    int      depth,
    int      maxdepth,
    Iterator iter)
{
    if (depth >= maxdepth) return NULL;

    for (Iterator i = vp->out_begin(); i != vp->out_end(); i++) {
	Vertex* p = (*i)->to();
	switch (finder(p, *i, depth)) {
	 case dfs_next:
	    p = dfs_forward_aux(p, finder, depth + 1, maxdepth, iter);
	    if (p) return p;
	    else   break;
	 case dfs_found:
	    return vp;
	 case dfs_visited:
	    break;
	}
    }
    return NULL;
}


template <class Vertex, class Finder>
inline Vertex*
dfs_forward(
    Vertex* vp,
    Finder& finder,
    int     maxdepth = INT_MAX)
{
    switch (finder(vp, NULL, 0)) {
     case dfs_next:
	return dfs_forward_aux(vp, finder, 1, maxdepth, vp->in_begin());
     case dfs_found:
	return vp;
     case dfs_visited:
	return NULL;
    }
    return NULL;
}

#endif /* DFS_H */
