/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef KANJI_H
#define KANJI_H

#include <string>
#include <iostream.h>
#include <stddef.h>


enum {
    KANJI_ASCII = 0, KANJI_JIS = 1,
    KANJI_SJIS  = 2, KANJI_EUC = 4, KANJI_ALL = 7
};

int guess_kanji_code(char* str, size_t len, int mask);


class converterbuf: public streambuf
{
 public:
    explicit converterbuf(ostream* out): out_(out) {}
    int sync();
    int overflow(int ch);

 protected:
    virtual int putch(unsigned char ch) { *out_ << ch; return ch; }

    ostream* out_;

 private:
    converterbuf(const converterbuf&);
    converterbuf& operator=(const converterbuf&);
    DBG_NEW
};


class s2ebuf: public converterbuf
{
 public:
    explicit s2ebuf(ostream* out):
        converterbuf(out), state_(first), prev_('\0') {}

 private:
    int  putch(unsigned char);
    void convert(unsigned char&, unsigned char&);

    enum { first, second } state_;
    unsigned char prev_;
    DBG_NEW
};


class j2ebuf: public converterbuf
{
 public:
    explicit j2ebuf(ostream* out):
        converterbuf(out), state_(ascii), prev_('\0') {}

 private:
    int putch(unsigned char);

    enum {
	ascii, in_esc, in_esc_dollar, first, second, out_esc, out_esc_paren
    } state_;
    unsigned char prev_;
    DBG_NEW
};


class s2estream: public ostream
{
 public:
    explicit s2estream(ostream* out): ostream(new s2ebuf(out)) {}
    virtual ~s2estream() { delete rdbuf(); }

    DBG_NEW
};


class j2estream: public ostream
{
 public:
    explicit j2estream(ostream* out): ostream(new j2ebuf(out)) {}
    virtual ~j2estream() { delete rdbuf(); }

    DBG_NEW
};


istream& anytoeuc(istream&, ostream&);
string   anytoeuc(string);


#endif /* KANJI */
