/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

// This file contains the code based on GLUT 3.1
// Copyright (c) Mark J. Kilgard, 1994.
//
// and OpenGL samples by SGI
// (c) Copyright 1993, Silicon Graphics, Inc.


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "utilgl.h"

#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include <algorithm>
#include <GL/glu.h>
#include <GL/glx.h>


void
GetUnpackingMode(
    PackingMode& pm)
{
    glGetIntegerv(GL_UNPACK_SWAP_BYTES,  &pm.swap_bytes);
    glGetIntegerv(GL_UNPACK_LSB_FIRST,   &pm.lsb_first);
    glGetIntegerv(GL_UNPACK_ROW_LENGTH,  &pm.row_length);
    glGetIntegerv(GL_UNPACK_SKIP_ROWS,   &pm.skip_rows);
    glGetIntegerv(GL_UNPACK_SKIP_PIXELS, &pm.skip_pixels);
    glGetIntegerv(GL_UNPACK_ALIGNMENT,   &pm.alignment);
}


void
SetUnpackingMode(
    PackingMode& pm)
{
    glPixelStorei(GL_UNPACK_SWAP_BYTES,  pm.swap_bytes);
    glPixelStorei(GL_UNPACK_LSB_FIRST,   pm.lsb_first);
    glPixelStorei(GL_UNPACK_ROW_LENGTH,  pm.row_length);
    glPixelStorei(GL_UNPACK_SKIP_ROWS,   pm.skip_rows);
    glPixelStorei(GL_UNPACK_SKIP_PIXELS, pm.skip_pixels);
    glPixelStorei(GL_UNPACK_ALIGNMENT,   pm.alignment);
}


void
WireSphere(
    GLfloat radius,
    GLint   slices,
    GLint   stacks)
{
    GLUquadricObj* quad_obj = gluNewQuadric();
    gluQuadricDrawStyle(quad_obj, GLU_LINE);
    gluSphere(quad_obj, radius, slices, stacks);
    gluDeleteQuadric(quad_obj);
}


void
SolidSphere(
    GLfloat radius,
    GLint   slices,
    GLint   stacks)
{
    GLUquadricObj* quad_obj = gluNewQuadric();
    gluQuadricDrawStyle(quad_obj, GLU_FILL);
    gluQuadricNormals(quad_obj, GLU_SMOOTH);
    gluSphere(quad_obj, radius, slices, stacks);
    gluDeleteQuadric(quad_obj);
}


static void
draw_box(
    GLfloat size,
    GLenum type)
{
    static GLfloat n[6][3] = {
	{-1.0,  0.0,  0.0},
	{ 0.0,  1.0,  0.0},
	{ 1.0,  0.0,  0.0},
	{ 0.0, -1.0,  0.0},
	{ 0.0,  0.0,  1.0},
	{ 0.0,  0.0, -1.0}
    };
    static GLint faces[6][4] = {
	{0, 1, 2, 3},
	{3, 2, 6, 7},
	{7, 6, 5, 4},
	{4, 5, 1, 0},
	{5, 6, 2, 1},
	{7, 4, 0, 3}
    };
    GLfloat v[8][3];
    GLint i;

    v[0][0] = v[1][0] = v[2][0] = v[3][0] = -size / 2;
    v[4][0] = v[5][0] = v[6][0] = v[7][0] =  size / 2;
    v[0][1] = v[1][1] = v[4][1] = v[5][1] = -size / 2;
    v[2][1] = v[3][1] = v[6][1] = v[7][1] =  size / 2;
    v[0][2] = v[3][2] = v[4][2] = v[7][2] = -size / 2;
    v[1][2] = v[2][2] = v[5][2] = v[6][2] =  size / 2;
    
    for (i = 0; i < 6; i++) {
	glBegin(type);
	n3fv(&n[i][0]);
	glTexCoord2f(0.0, 0.0);  v3fv(&v[faces[i][0]][0]);
	glTexCoord2f(1.0, 0.0);	 v3fv(&v[faces[i][1]][0]);
	glTexCoord2f(1.0, 1.0);  v3fv(&v[faces[i][2]][0]);
	glTexCoord2f(0.0, 1.0);  v3fv(&v[faces[i][3]][0]);
	glEnd();
    }
}


void
WireCube(
    GLfloat size)
{
    draw_box(size, GL_LINE_LOOP);
}


void
SolidCube(
    GLfloat size)
{
    draw_box(size, GL_QUADS);
}


void
StrokeText(
    GLfloat x,
    GLfloat y,
    GLfloat z,
    GLfloat angle,
    GLfloat zoom,
    string  str,
    size_t  len)
{
    if (len == 0) len = str.length();
    else          len = min(str.length(), len);

    glPushMatrix();
    glTranslatef(x, y, z);
    glRotatef(angle, 0.0, 0.0, 1.0);
    glScalef(zoom, zoom, zoom);
    for (size_t i = 0; i < len; i++)
	StrokeCharacter(&StrokeRoman, str[i]);
    glPopMatrix();
}


//////////////////////////////////////////////////////////////////////////////
//  The following function is based on glut_stroke.c in GLUT 3.1
//
///* Copyright (c) Mark J. Kilgard, 1994. */
//
///* This program is freely distributable without licensing fees
//   and is provided without guarantee or warrantee expressed or
//   implied. This program is -not- in the public domain. */

void
StrokeCharacter(
    StrokeFontPtr fontinfo,
    int           c)
{
    StrokeCharPtr ch;
    if (c < 0 || c >= fontinfo->num_chars || !(ch = &fontinfo->ch[c]))
	return;
    
    int i = ch->num_strokes;
    StrokePtr stroke = ch->stroke;
    for (; i > 0; i--, stroke++) {
	glBegin(GL_LINE_STRIP);
	int j = stroke->num_coords;
	CoordPtr coord = stroke->coord;
	for (; j > 0; j--, coord++)
	    glVertex2f(coord->x, coord->y);
	glEnd();
    }
    glTranslatef(ch->right, 0.0, 0.0);
}


//////////////////////////////////////////////////////////////////////////////
//  The following code is based on glut_shape.c in GLUT 3.1.
//
///* Copyright (c) Mark J. Kilgard, 1994. */
//
///* This program is freely distributable without licensing fees
//   and is provided without guarantee or warrantee expressed or
//   implied. This program is -not- in the public domain. */
//
//  Again, glut_shapes.c is based on Open GL samples by SGI.
//
/**
(c) Copyright 1993, Silicon Graphics, Inc.

ALL RIGHTS RESERVED

Permission to use, copy, modify, and distribute this software
for any purpose and without fee is hereby granted, provided
that the above copyright notice appear in all copies and that
both the copyright notice and this permission notice appear in
supporting documentation, and that the name of Silicon
Graphics, Inc. not be used in advertising or publicity
pertaining to distribution of the software without specific,
written prior permission.

THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU
"AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR
OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  IN NO
EVENT SHALL SILICON GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE
ELSE FOR ANY DIRECT, SPECIAL, INCIDENTAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER,
INCLUDING WITHOUT LIMITATION, LOSS OF PROFIT, LOSS OF USE,
SAVINGS OR REVENUE, OR THE CLAIMS OF THIRD PARTIES, WHETHER OR
NOT SILICON GRAPHICS, INC.  HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH LOSS, HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
ARISING OUT OF OR IN CONNECTION WITH THE POSSESSION, USE OR
PERFORMANCE OF THIS SOFTWARE.

US Government Users Restricted Rights

Use, duplication, or disclosure by the Government is subject to
restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
(c)(1)(ii) of the Rights in Technical Data and Computer
Software clause at DFARS 252.227-7013 and/or in similar or
successor clauses in the FAR or the DOD or NASA FAR
Supplement.  Unpublished-- rights reserved under the copyright
laws of the United States.  Contractor/manufacturer is Silicon
Graphics, Inc., 2011 N.  Shoreline Blvd., Mountain View, CA
94039-7311.

OpenGL(TM) is a trademark of Silicon Graphics, Inc.
*/


static inline void
normalize(
    GLfloat v[3])
{
    GLfloat d = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
    if (d == 0.0) v[0] = d = 1.0;
    d = 1 / d;
    v[0] *= d;
    v[1] *= d;
    v[2] *= d;
}


#define X .525731112119133606
#define Z .850650808352039932

static GLfloat vdata[12][3] = {
    { -X,  0,  Z }, {  X,  0,  Z }, { -X,  0, -Z }, {  X,  0, -Z },
    {  0,  Z,  X }, {  0,  Z, -X }, {  0, -Z,  X }, {  0, -Z, -X },
    {  Z,  X,  0 }, { -Z,  X,  0 }, {  Z, -X,  0 }, { -Z, -X,  0 },
};

static int tindices[20][3] = {
    {  0,  4,  1 }, {  0,  9,  4 }, {  9,  5,  4 }, {  4,  5,  8 },
    {  4,  8,  1 }, {  8, 10,  1 }, {  8,  3, 10 }, {  5,  3,  8 },
    {  5,  2,  3 }, {  2,  7,  3 }, {  7, 10,  3 }, {  7,  6, 10 },
    {  7, 11,  6 }, { 11,  0,  6 }, {  0,  1,  6 }, {  6,  1, 10 },
    {  9,  0, 11 }, {  9, 11,  2 }, {  9,  2,  5 }, {  7,  2, 11 },
};


static void
subdivide(
    GLfloat n1[3], GLfloat n2[3], GLfloat n3[3],
    GLfloat r,
    int     depth,
    GLenum  shade_type)
{
    if (--depth < 0) {
	glBegin(shade_type);
	n3fv(n3);  v3f(r * n3[0], r * n3[1], r * n3[2]);
	n3fv(n2);  v3f(r * n2[0], r * n2[1], r * n2[2]);
	n3fv(n1);  v3f(r * n1[0], r * n1[1], r * n1[2]);
	glEnd();
	return;
    }
    
    GLfloat n12[3], n23[3], n31[3];
    for (int i = 0; i < 3; i++) {
	n12[i] = n1[i] + n2[i];
	n23[i] = n2[i] + n3[i];
	n31[i] = n3[i] + n1[i];
    }
    normalize(n12);
    normalize(n23);
    normalize(n31);
    
    subdivide(n1,  n12, n31, r, depth, shade_type);
    subdivide(n2,  n23, n12, r, depth, shade_type);
    subdivide(n3,  n31, n23, r, depth, shade_type);
    subdivide(n12, n23, n31, r, depth, shade_type);
}


void
WireSphere_icosahedron(
    GLfloat r,
    int     depth)
{
    for (int i = 0; i < 20; i++) {
	subdivide(&vdata[tindices[i][0]][0],
		  &vdata[tindices[i][1]][0],
		  &vdata[tindices[i][2]][0], r, depth, GL_LINE_LOOP);
    }
}


void
SolidSphere_icosahedron(
    GLfloat r,
    int     depth)
{
    for (int i = 0; i < 20; i++) {
	subdivide(&vdata[tindices[i][0]][0],
		  &vdata[tindices[i][1]][0],
		  &vdata[tindices[i][2]][0], r, depth, GL_TRIANGLES);
    }
}


///////////////////////////////////////////////////////////////////////////////
/*
 * (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * US Government Users Restricted Rights
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */

#include <fstream.h>
#include <stdlib.h> 

typedef struct _rawImageRec {
    unsigned short imagic;
    unsigned short type;
    unsigned short dim;
    unsigned short sizeX, sizeY, sizeZ;
    unsigned long min, max;
    unsigned long wasteBytes;
    char name[80];
    unsigned long colorMap;
    unsigned char *tmp, *tmpR, *tmpG, *tmpB;
    unsigned long rleEnd;
    GLuint *rowStart;
    GLint *rowSize;
    int read;
} rawImageRec;


static void ConvertShort(unsigned short *array, long length)
{
    unsigned long b1, b2;
    unsigned char *ptr;

    ptr = (unsigned char *)array;
    while (length--) {
	b1 = *ptr++;
	b2 = *ptr++;
	*array++ = (b1 << 8) | (b2);
    }
}

static void ConvertLong(GLuint *array, long length)
{
    unsigned long b1, b2, b3, b4;
    unsigned char *ptr;

    ptr = (unsigned char *)array;
    while (length--) {
	b1 = *ptr++;
	b2 = *ptr++;
	b3 = *ptr++;
	b4 = *ptr++;
	*array++ = (b1 << 24) | (b2 << 16) | (b3 << 8) | (b4);
    }
}


static rawImageRec*
RawImageOpen(
    istream& in)
{
    union {
	int testWord;
	char testByte[4];
    } endianTest;
    rawImageRec *raw;
    GLenum swapFlag;
    int x;
    int want, done;

    endianTest.testWord = 1;
    if (endianTest.testByte[0] == 1) {
	swapFlag = GL_TRUE;
    } else {
	swapFlag = GL_FALSE;
    }

    raw = new rawImageRec;
    if (!raw) return NULL;

    want = 12;
    done =  0;
    do {
	in.read((char*) raw + done, want);
	done = in.gcount();
    } while ((want -= done));
    raw->read = 12;

    if (swapFlag) {
	ConvertShort(&raw->imagic, 6);
    }

    raw->tmp  = new unsigned char[raw->sizeX*256];
    raw->tmpR = new unsigned char[raw->sizeX*256];
    raw->tmpG = new unsigned char[raw->sizeX*256];
    raw->tmpB = new unsigned char[raw->sizeX*256];
    if (!raw->tmp || !raw->tmpR|| !raw->tmpG || !raw->tmpB) {
	delete[] raw->tmp;
	delete[] raw->tmpR;
	delete[] raw->tmpG;
	delete[] raw->tmpB;
	delete raw;
	return NULL;
    }

    if ((raw->type & 0xFF00) == 0x0100) {
	x = raw->sizeY * raw->sizeZ;
	raw->rowStart = new GLuint[x];
	raw->rowSize  = new GLint [x];
	if (raw->rowStart == NULL || raw->rowSize == NULL) {
	    delete[] raw->tmp;
	    delete[] raw->tmpR;
	    delete[] raw->tmpG;
	    delete[] raw->tmpB;
	    delete raw;
	    return NULL;
	}
	raw->rleEnd = 512 + (2 * x * sizeof(GLuint));

	// in.seekg(512);
	want = 512 - raw->read;
	done =  0;
	do {
	    in.read(raw->tmp + done, want);
	    done = in.gcount();
	} while ((want -= done));
	raw->read = 512;

	want = x * sizeof(GLuint);
	done = 0;
	do {
	    in.read(raw->rowStart + done, want);
	    done = in.gcount();
	} while ((want -= done));
	raw->read += x * sizeof(GLuint);

	want = x * sizeof(GLuint);
	done = 0;
	do {
	    in.read(raw->rowSize + done, want);
	    done = in.gcount();
	} while ((want -= done));
	raw->read += x * sizeof(GLuint);

	if (swapFlag) {
	    ConvertLong(raw->rowStart, x);
	    ConvertLong((GLuint *)raw->rowSize, x);
	}
    }
    return raw;
}


static void
RawImageClose(
    rawImageRec* raw)
{
    delete[] raw->tmp;
    delete[] raw->tmpR;
    delete[] raw->tmpG;
    delete[] raw->tmpB;
    delete raw;
}


static void
RawImageGetRow(
    istream&       in,
    rawImageRec*   raw,
    unsigned char* buf,
    int y,
    int z)
{
    unsigned char *iPtr, *oPtr, pixel;
    int count;
    int want, done;

    if ((raw->type & 0xFF00) == 0x0100) {
	// in.seekg(raw->rowStart[y + z * raw->sizeY]);
	want = raw->rowStart[y + z * raw->sizeY] - raw->read;
	done =  0;
	do {
	    in.read(raw->tmp + done, want);
	    done = in.gcount();
	} while ((want -= done));
	raw->read = raw->rowStart[y + z * raw->sizeY];

	want = raw->rowSize[y + z * raw->sizeY];
	done = 0;
	do {
	    in.read(raw->tmp + done, want);
	    done = in.gcount();
	} while ((want -= done));
	raw->read += raw->rowSize[y + z * raw->sizeY];

	iPtr = raw->tmp;
	oPtr = buf;
	while (1) {
	    pixel = *iPtr++;
	    count = (int)(pixel & 0x7F);
	    if (!count) return;
	    if (pixel & 0x80) {
		while (count--) *oPtr++ = *iPtr++;
	    } else {
		pixel = *iPtr++;
		while (count--) *oPtr++ = pixel;
	    }
	}
    } else {
	// in.seekg(512 + (y * raw->sizeX) + (z * raw->sizeX * raw->sizeY));
	want = 512 + (y * raw->sizeX) + (z * raw->sizeX * raw->sizeY)
	    - raw->read;
	done =  0;
	do {
	    in.read(raw->tmp + done, want);
	    done = in.gcount();
	} while ((want -= done));
	raw->read = 512 + (y * raw->sizeX) + (z * raw->sizeX * raw->sizeY);

	want = raw->sizeX;
	done = 0;
	do {
	    in.read(buf + done, want);
	} while ((want -= done));
	raw->read += raw->sizeX;
    }
}


static void*
RawImageGetData(
    istream&     in,
    rawImageRec* raw,
    GLint        width,
    GLint        height)
{
    GLubyte* data = new GLubyte[(width + 1) * (height + 1) * 4];
    if (!data) return NULL;

    GLubyte* ptr = data;
    for (GLint i = 0; i < height; i++) {
	RawImageGetRow(in, raw, raw->tmpR, i, 0);
	RawImageGetRow(in, raw, raw->tmpG, i, 1);
	RawImageGetRow(in, raw, raw->tmpB, i, 2);
	for (GLint j = 0; j < width; j++) {
	    *ptr++ = *(raw->tmpR + j);
	    *ptr++ = *(raw->tmpG + j);
	    *ptr++ = *(raw->tmpB + j);
	}
    }
    return data;
}


void*
LoadSGIImage(
    istream& in,
    GLint&   width,
    GLint&   height)
{
    width = height = 0;
    rawImageRec* raw = RawImageOpen(in);
    if (!raw) return NULL;
    width  = raw->sizeX;
    height = raw->sizeY;
    void* bitmap = RawImageGetData(in, raw, width, height);
    RawImageClose(raw);
    return bitmap;
}
