/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "HTML.h"

#include "util.h"


bool LinkExtractor::class_initialized_ = false;
LinkExtractor::start_handlers_type LinkExtractor::start_handlers_;//(32);
LinkExtractor::end_handlers_type   LinkExtractor::end_handlers_;//(32);


LinkExtractor::LinkExtractor(
    const char* text,
    size_t      len,
    NetObject*  netobj):
    SGML(text, len), netobj_(netobj)
{
    if (!class_initialized_) class_init();
}


void
LinkExtractor::class_init()
{
    class_initialized_ = true;
    start_handlers_["HEAD"] = &start_head;
    start_handlers_["TITLE"] = &start_title;
    start_handlers_["LINK"] = &start_link;
    start_handlers_["BASE"] = &start_base;
    start_handlers_["BODY"] = &start_body;
    start_handlers_["H1"] = &start_hx;
    start_handlers_["H2"] = &start_hx;
    start_handlers_["H3"] = &start_hx;
    start_handlers_["H4"] = &start_hx;
    start_handlers_["H5"] = &start_hx;
    start_handlers_["H6"] = &start_hx;
    start_handlers_["A"] = &start_a;
    start_handlers_["IMG"] = &start_img;
    start_handlers_["AREA"] = &start_area;
    start_handlers_["FORM"] = &start_form;
    start_handlers_["INPUT"] = &start_input;
    start_handlers_["FRAME"] = &start_frame;
    start_handlers_["IFRAME"] = &start_iframe;
    start_handlers_["SCRIPT"] = &start_script;

    end_handlers_["TITLE"] = &end_title;
    end_handlers_["H1"] = &end_hx;
    end_handlers_["H2"] = &end_hx;
    end_handlers_["H3"] = &end_hx;
    end_handlers_["H4"] = &end_hx;
    end_handlers_["H5"] = &end_hx;
    end_handlers_["H6"] = &end_hx;
    end_handlers_["A"] = &end_a;
    end_handlers_["FORM"] = &end_form;
}


void
LinkExtractor::text_block(
    const char* p,
    const char* q)
{
    if (state_ & (hx_bit | title_bit))
        heading_ += string(p, q - p);
    if (state_ & a_bit)
        anchor_  += string(p, q - p);
}


void
LinkExtractor::start_element(
    string      elem,
    attrs_type& attrs)
{
    start_handlers_type::iterator i = start_handlers_.find(elem);
    if (i != start_handlers_.end()) (this->*(*i).second)(elem, attrs);
}


void
LinkExtractor::end_element(
    string elem)
{
    end_handlers_type::iterator i = end_handlers_.find(elem);
    if (i != end_handlers_.end()) (this->*(*i).second)(elem);
}
