/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "NetFTP.h"

#include <strstream.h>
#include "util.h"
#include "netstream.h"


NetFTP::NetFTP(
    NetManager* manager):
    NetProtocol(manager)
{}


bool
NetFTP::get(
    NetRequest&  req,
    NetResponse& res)
{
    return this->do_request(req, res, "GET");
}


bool
NetFTP::head(
    NetRequest&  req,
    NetResponse& res)
{
    URL url = req.url();
    netstream* www = this->get_stream(url);
    
    RequestLine reqln("HEAD", url.reqpath(), "FTP/1.1");
    req.headers()->set_Host(url.host());
    *www << reqln << *req.message();

    ResponseLine resln;
    NetHeaders* headers = new NetHeaders;
    *www >> resln >> *headers;
    res.message(new NetMessage(headers));

    this->release_stream(www);
    return true;
}


bool
NetFTP::post(
    NetRequest&  req,
    NetResponse& res)
{
    return this->do_request(req, res, "POST");
}


bool
NetFTP::do_request(
    NetRequest&  req,
    NetResponse& res,
    const char*  cmd)
{
    URL url = req.url();
    netstream* www = this->get_stream(url);

    int state;

    this->release_stream(www);
    return true;
}


ostream&
operator<<(
    ostream& out,
    const NetFTP::RequestLine& req)
{
    out << req.method << ' ' << req.path << ' ' << req.version << "\r\n";
    return out;
}


istream&
operator>>(
    istream& in,
    NetFTP::ResponseLine& res)
{
    strstream buf;
    if (copy_line(in, buf)) {
        buf >> res.version >> ws >> res.status >> ws;
        char desc[512];
        buf.getline(desc, 511);
        desc[511] = '\0';
        res.description = desc;
    }
    else {
        res.version = "FTP/unknown";
        res.status = -1;
        res.description = "Response line not present.";
    }
    return in;
}


ostream&
operator<<(
    ostream& out,
    const NetFTP::ResponseLine& res)
{
    out << res.version << ' ' << res.status << ' ' << res.description <<"\r\n";
    return out;
}
