/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_NetLocal_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "NetLocal.h"

#include <dirent.h>
#include <sys/stat.h>
#include <vector>
#include <algorithm>
#include <fstream.h>
#include <strstream.h>

#if defined __GNUG__ && (defined EGCS || __GNUC__ >= 2 && __GNUC_MINOR__ >= 8)
#define remove erase
#endif

#define MODE 0666


NetLocal::NetLocal(
    NetManager* manager):
    NetProtocol(manager)
{}


void
NetLocal::get(
    NetRequest&  req,
    NetResponse& res)
{
    string path = req.url().path();
    if (path[path.length() - 1] == '/')
        this->get_dir(req, res);
    else 
        this->get_file(req, res);
}


void
NetLocal::get_file(
    NetRequest&  req,
    NetResponse& res)
{
    URL url = req.url();
    NetHeaders* headers = new NetHeaders;
    memobject*  content = NULL;
    
    struct stat sbuf;
    stat(url.path().c_str(), &sbuf);  // lstat?
    if (S_ISREG(sbuf.st_mode)) {
        // real headers
        this->set_stat(*headers, sbuf);
        headers->set_Content_Length(sbuf.st_size);
        // .meta headers
        string meta = req.url() + ".meta";
        ifstream hin(meta.c_str(), ios::in);
        if (hin) hin >> *headers;
        hin.close();
        // content
        content =
            new mapped_file(url.path(), sbuf.st_size, mapped_file::in, MODE);
        res.status(200);
    }
    else if (S_ISDIR(sbuf.st_mode)) {
        url.path(url.path() + '/');
        headers->set_Location(url);
        res.status(304);
    }
    else {
        res.status(404);
    }
    res.message(new NetMessage(headers, content));
}


void
NetLocal::head_file(
    NetRequest&  req,
    NetResponse& res)
{
    URL url = req.url();
    NetHeaders* headers = new NetHeaders;
    memobject*  content = NULL;
    
    struct stat sbuf;
    stat(url.path().c_str(), &sbuf);  // lstat?
    if (S_ISREG(sbuf.st_mode)) {
        // real headers
        this->set_stat(*headers, sbuf);
        headers->set_Content_Length(sbuf.st_size);
        // .meta headers
        string meta = req.url() + ".meta";
        ifstream hin(meta.c_str(), ios::in);
        if (hin) hin >> *headers;
        res.status(200);
    }
    else if (S_ISDIR(sbuf.st_mode)) {
        url.path(url.path() + '/');
        headers->set_Location(url);
        res.status(304);
    }
    else {
        res.status(404);
    }
    res.message(new NetMessage(headers, content));
}


void
NetLocal::get_dir(
    NetRequest&  req,
    NetResponse& res)
{
    URL url = req.url();
    struct stat sbuf;
    stat(url.path().c_str(), &sbuf);  // lstat? and readdir_r?

    NetHeaders* headers = new NetHeaders;
    memobject*  content = NULL;
    
    if (S_ISDIR(sbuf.st_mode)) {
        vector<string> v;
        size_t len = 0;
        DIR* dp = opendir(url.path().c_str());
        for (struct dirent* ent = NULL; (ent = readdir(dp));) {
            v.push_back(ent->d_name);
            len += strlen(ent->d_name) + 1;
        }
        closedir(dp);
        sort(v.begin(), v.end());

        this->set_stat(*headers, sbuf);
        headers->set_Content_Length(len);
        headers->set_Content_Type("text/x-directory");

        content = new heap_memory(len);
        strstream buf((char*) content->area(), content->size());
        for (vector<string>::iterator i = v.begin(); i != v.end(); i++)
            buf << (*i) << "\n";
        res.status(200);
    }
    else if (S_ISREG(sbuf.st_mode)) {
        url.path(url.path().remove(url.path().length() - 1));
        headers->set_Location(url);
        res.status(304);
    }
    else {
        res.status(404);
    }
    res.message(new NetMessage(headers, content));
}


void
NetLocal::head_dir(
    NetRequest&  req,
    NetResponse& res)
{
    URL url = req.url();
    struct stat sbuf;
    stat(url.path().c_str(), &sbuf);  // lstat?

    NetHeaders* headers = new NetHeaders;
    memobject*  content = NULL;
    
    if (S_ISDIR(sbuf.st_mode)) {
        size_t len = 0;
        DIR* dp = opendir(url.path().c_str());
        for (struct dirent* ent = NULL; (ent = readdir(dp));)
            len += strlen(ent->d_name) + 1;
        closedir(dp);
        
        this->set_stat(*headers, sbuf);
        headers->set_Content_Length(len);
        headers->set_Content_Type("text/x-directory");
        res.status(200);
    }
    else if (S_ISREG(sbuf.st_mode)) {
        url.path(url.path().remove(url.path().length() - 1));
        headers->set_Location(url);
        res.status(304);
    }
    else {
        res.status(404);
    }
    res.message(new NetMessage(headers, content));
}


void
NetLocal::set_stat(
    NetHeaders& headers,
    const struct stat& stbuf)
{
    headers.set_Last_Modified(stbuf.st_mtime);
}
