/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_NetMessage_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "NetMessage.h"

#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <strstream.h>
#include "timestr.h"


bool
NetHeaders::get(
    string  key,
    string& buf)
const {
    fields_type::const_iterator i = fields_.find(capitalize2(key));
    bool found = i != fields_.end();
    buf = found ? (*i).second: string();
    return found;
}


void
NetHeaders::set(
    string key,
    string val)
{
    fields_[capitalize2(key)] = val;
}


bool
NetHeaders::get(
    string  key,
    size_t& buf)
const {
    fields_type::const_iterator i = fields_.find(capitalize2(key));
    bool found = i != fields_.end();
    buf = found ? strtoul((*i).second.c_str(), NULL, 10): 0;
    return found;
}


void
NetHeaders::set(
    string key,
    size_t val)
{
    key = capitalize2(key);
    strstream buf;
    buf << val << ends;
    fields_[key] = buf.str();
    buf.freeze(false);
}


bool
NetHeaders::get(
    string  key,
    URL&    buf)
const {
    fields_type::const_iterator i = fields_.find(capitalize2(key));
    bool found = i != fields_.end();
    buf = found ? URL((*i).second): URL();
    return found;
}


void
NetHeaders::set(
    string key,
    URL    val)
{
    fields_[capitalize2(key)] = val.abs();
}


bool
NetHeaders::get_time(
    string  key,
    time_t& buf)
const {
    fields_type::const_iterator i = fields_.find(capitalize2(key));
    bool found = i != fields_.end();
    buf = found ? strtotime((*i).second.c_str()): 0;
    return found;
}


void
NetHeaders::set_time(
    string key,
    time_t val)
{
    char buf[64];
    fields_[capitalize2(key)] = strf_rfc1128time(&val, buf);
}


istream&
operator>>(
    istream& in,
    NetHeaders& h)
{
    char field[128];
    char value[512];
    for (;;) {
        strstream buf;
        copy_line(in, buf);  buf << ends;

        size_t len = sizeof(field);
        buf.getline(field, len - 1, ':');
        field[len - 1] = '\0';

        if (!field[0]) break;
        buf >> ws;

        len = sizeof(value);
        buf.getline(value, len - 1);
        value[len - 1] = '\0';
        
        h.fields_[capitalize2(field)] = value;
    }
    return in;
}


ostream&
operator<<(
    ostream& out,
    const NetHeaders& h)
{
    NetHeaders::fields_type::const_iterator i;
    for (i = h.fields_.begin(); i != h.fields_.end(); i++)
        out << (*i).first << ": " << (*i).second << "\r\n";
    return out;
}


NetMessage::NetMessage(
    NetHeaders* headers,
    memobject*  content,
    URL         id):
    id_(id), headers_(headers), content_(content),
    headers_frozen_(false), content_frozen_(false)
{}


NetMessage::~NetMessage()
{
    if (!headers_frozen_) delete headers_;
    if (!content_frozen_) delete content_;
}


#if 0
void
NetMessage::content(
    memobject* content)
{
    content_ = content;
    if (!content) return;
    headers_->set_Content_Length(content->size());
}
#endif


ostream&
operator<<(
    ostream&          out,
    const NetMessage& msg)
{
    if (msg.headers())
        out << *msg.headers() << "\r\n";
    if (msg.content())
        out.write(msg.content()->area(), msg.content()->size());
    return out;
}


istream&
operator>>(
    istream&    in,
    NetMessage& msg)
{
    delete msg.headers_;
    delete msg.content_;
    
    NetHeaders* headers = new NetHeaders;
    in >> *headers;

    heap_memory* content = NULL;
    size_t len = 0;
    if (headers->get_Content_Length(len)) {
        content = new heap_memory(len);
        char* area = (char*) content->area();
        size_t n = 0, nread = 0;
        do {
            in.read(area + nread, len - nread);
            nread += (n = in.gcount());
        } while (in && n && nread < len);
        len = nread;
    }
    else {
        strstream str;
        size_t nread = 0;
        for (nread = 0; in; nread++) {
            unsigned char ch;
            in.get(ch);
            if (!in) break;
            str.put(ch);
        };
        len = nread;
        str.freeze();
        content = new heap_memory(str.str(), len);
    }
    msg.headers_ = headers;
    msg.content_ = content;
    return in;
}
