/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef NETMESSAGE_H
#define NETMESSAGE_H
#ifdef __GNUG__
#pragma interface
#endif

#include <time.h>
#include <istream.h>
#include <ostream.h>
#include "util.h"
#include "utilstl.h"
#include "memobject.h"
#include "URL.h"


class NetHeaders
{
    friend istream& operator>>(istream&, NetHeaders&);
    friend ostream& operator<<(ostream&, const NetHeaders&);
 private:
    typedef smap<string, string> fields_type;
    fields_type fields_;

 public:
    bool get(string, string&) const;
    void set(string, string);

    bool get(string, size_t&) const;
    void set(string, size_t);

    bool get(string, URL&) const;
    void set(string, URL);

    bool get_time(string, time_t&) const;
    void set_time(string, time_t);

 public:

#define HEADER(name, type, field)  \
    bool get_ ## name(type& x) const  { return get(field, x); } \
    void set_ ## name(const type&  x) {        set(field, x); }
    
#define HDTIME(name, type, field)  \
    bool get_ ## name(type& x) const  { return get_time(field, x); } \
    void set_ ## name(const type&  x) {        set_time(field, x); }
    
    HEADER(Cache_Control,       string, "Cache-Control")
    HEADER(Connection,          string, "Connection")
    HDTIME(Date,                time_t, "Date")
    HEADER(Pragma,              string, "Pragma")
    // Transfer_Encodeing Upgrate Via

    HEADER(From,                string, "From")
    HEADER(Host,                string, "Host")
    HDTIME(If_Modified_Since,   time_t, "If-Modified-Since")
    HDTIME(If_Unmodified_Since, time_t, "If-Unmodified-Since")
    HEADER(Referer,             URL,    "Referer")
    HEADER(User_Agent,          string, "User-Agent")
    // Accept Accept_Charset Accept_Encoding Accept_Language
    // Authorization
    // If_Match If_None_Match If_Range
    // Max_Forwards Proxy_Authorization Range

    HEADER(Location,            URL,    "Location")
    HEADER(Server,              string, "Server")
    // Age
    // Proxy_Authenticate Public Retry_After
    // Vary Warning WWW_Authenticate

    HEADER(Content_Length,      size_t, "Content-Length")
    HEADER(Content_Type,        string, "Content-Type")
    HDTIME(Expires,             time_t, "Expires")
    HDTIME(Last_Modified,       time_t, "Last-Modified")
    // Allow Content_Base Content_Encoding Content_Language
    // Content_Location Content_MD5 Content_Range
    // ETag

    // Alternates Content_Version Derived_From Link URI
DBG_NEW
};


class NetMessage
{
    friend istream& operator>>(istream&, NetMessage&);
 public:
    NetMessage(NetHeaders* = NULL, memobject* = NULL, URL = URL());
    virtual ~NetMessage();

    URL id() const { return id_; }

    NetHeaders*       headers()       { return headers_; }
    const NetHeaders* headers() const { return headers_; }
    memobject*        content()       { return content_; }
    const memobject*  content() const { return content_; }

    void freeze_headers() { headers_frozen_ = true; }
    void freeze_content() { content_frozen_ = true; }

    void id(URL);
    // void content(memobject*);

 private:
    URL         id_;
    NetHeaders* headers_;
    memobject*  content_;
    bool        headers_frozen_;
    bool        content_frozen_;

    NetMessage(const NetMessage&);
    NetMessage& operator=(const NetMessage&);
    DBG_NEW
};

ostream& operator<<(ostream&, const NetMessage&);
istream& operator>>(istream&, NetMessage&);

#endif /* NETMESSAGE_H */





