/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_SGML_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "SGML.h"

#include <ctype.h>
#include "util.h"


#define SKIP_SPACE    if (!skip_space())   return
#define SKIP_COMMENT  if (!skip_comment()) return


SGML::SGML():
    text_(NULL), end_(NULL), ptr_(NULL)
{}


SGML::SGML(textpos t, size_t l):
    text_(t), end_(t + l), ptr_(t)
{}


SGML::~SGML()
{}


void
SGML::text(
    textpos t,
    size_t  l)
{
    text_ = ptr_ = t;
    end_ = t + l;
}


inline bool
SGML::skip_space()
{
    register textpos p = ptr_;
    while (p < end_ && isspace((unsigned char) *p)) p++;
    return (ptr_ = p) < end_;
}


inline string
SGML::token(
    const char* delims)
{
    register textpos p = ptr_;
    while (p < end_ && (isalnum((unsigned char) *p) ||
                        (!isspace((unsigned char) *p) && !strchr(delims, *p))))
        p++;
    string s(ptr_, p - ptr_);
    ptr_ = p;
    return s;
}


string
SGML::tokenq(
    const char* delims)
{
    register textpos p = ptr_;
    if (*p == '\"' || *p == '\'') {  //"
        p = (textpos) memchr(p + 1, *p, end_ - p);
        if (p) {
            string s(ptr_ + 1, p - (ptr_ + 1));
            ptr_ = ++p;
            return s;
        }
        else {
            string s(ptr_ + 1, end_ - (ptr_ + 1));
            ptr_ = p;
            return s;
        }
    }
    /* else */
    return this->token(delims);
}


void
SGML::parse(
    textpos ptr)
{
    if (ptr) ptr_ = ptr;
    else     ptr_ = text_;

    this->start_document();
    for (;;) {
        textpos start = ptr_;
        ptr_ = (textpos) memchr(ptr_, '<', end_ - ptr_);
        if (!ptr_) ptr_ = end_;
        if (ptr_ != start) this->text_block(start, ptr_);
        ptr_++;  // skip '<'
        if (!skip_space()) break;
        this->parse_tag();
    }
    this->end_document();
}


void
SGML::parse_tag()
{
    switch (*ptr_) {
     case '!':
        ptr_++;
        SKIP_SPACE;
        this->parse_decl();
        break;
     case '/':
        ptr_++;
        SKIP_SPACE;
        this->parse_endtag();
        break;
     case '?':
        ptr_++;
        SKIP_SPACE;
        if (*ptr_ == '/') {
            ptr_++;
            SKIP_SPACE;
            this->parse_endtag();
        }
        else {
            this->parse_starttag();
        }
        break;
     default:
        this->parse_starttag();
        break;
    }
}


void
SGML::parse_starttag()
{
    attrs_type attrs;

    string elem = uc(this->token("<>/"));
    SKIP_SPACE;

    switch (*ptr_) {
     case '>':
        ptr_++;
        this->start_element(elem, attrs);
        return;
     case '<':
        this->start_element(elem, attrs);
        return;
     case '/':
        ptr_++;
        this->start_element(elem, attrs);
        this->parse_slashtag();
        return;
    }

    for (;;) {
        string attr = lc(this->token("<>="));
        attrs[attr] = "";
        SKIP_SPACE;

        if (*ptr_ == '=') {
            ptr_++;
            SKIP_SPACE;
            attrs[attr] = this->tokenq("<>");
            SKIP_SPACE;
        }
        switch (*ptr_) {
         case '>':
            ptr_++;
            this->start_element(elem, attrs);
            return;
         case '<':
            this->start_element(elem, attrs);
            return;
        }
    }
}


void
SGML::parse_slashtag()
{
    for (register textpos p = ptr_; p < end_; p++) {
        switch (*p) {
         case '/':
         case '>':
            this->text_block(ptr_, p);
            ptr_ = ++p;
            this->end_element("");
            return;
         case '<':
            this->text_block(ptr_, p);
            ptr_ = p;
            this->end_element("");
            return;
        }
    }
    ptr_ = end_;
}


void
SGML::parse_endtag()
{
    string elem = uc(this->token("<>"));
    SKIP_SPACE;

    for (register textpos p = ptr_; p < end_; p++) {
        switch (*p) {
         case '>':
            ptr_ = ++p;
            this->end_element(elem);
            return;
         case '<':
            ptr_ = p;
            this->end_element(elem);
            return;
        }
    }
    ptr_ = end_;
}


void
SGML::parse_decl()
{
    switch (*ptr_) {
     case '>':
        ptr_++;
        return;
     case '<':
        return;
     case '[':
        ptr_++;
        SKIP_COMMENT;
        this->parse_ifdef();
        SKIP_COMMENT;
        if (*ptr_ != ']') return;
        ptr_++;
        SKIP_COMMENT;
        if (*ptr_ != '>') return;
        ptr_++;
        return;
    }

    string decl = uc(this->token("<>"));
    SKIP_COMMENT;
    
    args_type args;
    for (;;) {
        switch (*ptr_) {
         case '>':
            ptr_++;
            this->sgml_decl(decl, args);
            return;
         case '<':
            this->sgml_decl(decl, args);
            return;
        }
        args.push_back(this->tokenq("<>"));
        SKIP_COMMENT;
    }
}


bool
SGML::skip_comment()
{
    while (ptr_ < end_) {
        if (!this->skip_space()) return false;
        if (ptr_ < end_ - 2 && memcmp(ptr_, "--", 2) != 0) return true;

        register textpos p = (ptr_ += 2);
        while (p < end_ && (p = (textpos) memchr(p, '-', end_ - p))) {
            if (++p < end_ && *p == '-') {
                ptr_ = ++p;
                break;
            }
        }
    }
    ptr_ = (textpos) memchr(ptr_, '>', end_ - ptr_);
    if (!ptr_) ptr_ = end_;
    return ptr_ < end_;
}


void
SGML::parse_ifdef()
{
    string symbol = this->token("<>[]");
    SKIP_COMMENT;
    if (*ptr_ != '[') return;
    ptr_++;

    this->start_ifdef(symbol);
    for (register textpos p = ptr_; p < end_; p++) {
        switch (*p) {
         case ']':
            this->text_block(ptr_, p);
            ptr_ = ++p;
            this->end_ifdef(symbol);
            return;
         case '<':
            this->text_block(ptr_, p);
            ptr_ = ++p;
            SKIP_SPACE;
            this->parse_tag();
            p = ptr_;
        }
    }
}



void
SGMLTracer::text_block(
    textpos p,
    textpos q)
{
#if 0
    cout << setw(5) << setfill('.') << p - text() << '.'
         << setw(5) << setfill('.') << q - text() << ' ';
#endif
}


void
SGMLTracer::start_element(
    string      elem,
    attrs_type& attrs)
{
    cout << '<' << elem << '>' << endl;
    if (attrs.size()) {
        for (attrs_type::iterator i = attrs.begin(); i != attrs.end(); i++)
            cout << "            " << (*i).first << '=' << (*i).second << endl;
    }
}


void
SGMLTracer::end_element(
    string elem)
{
    cout << "</" << elem << '>' << endl;
}


void
SGMLTracer::sgml_decl(
    string     decl,
    args_type& args)
{
    cout << "<!" << decl << endl;
    if (args.size()) {
        for (args_type::size_type i = 0; i < args.size(); i++)
            cout << "            " << args[i] << endl;
    }
    else {
        cout << endl;
    }
}


void
SGMLTracer::start_ifdef(
    string symbol)
{
    cout << "ifdef " << symbol << endl;
}


void
SGMLTracer::end_ifdef(
    string symbol)
{
    cout << "endif " << symbol << endl;
}
