/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef SGML_H
#define SGML_H
#ifdef __GNUG__
#pragma interface
#endif

#include <vector>
#include <string>
#include "utilstl.h"


class SGML
{
 public:
    typedef const char* textpos;
    
    SGML();
    SGML(textpos, size_t);
    virtual ~SGML();

    void    text(textpos t, size_t l);
    textpos text() { return text_; }
    void    parse(textpos = NULL);

 protected:
    typedef smap<string, string> attrs_type;
    typedef vector<string>       args_type;
    
    virtual void start_document() {}
    virtual void end_document() {}
    virtual void text_block(textpos, textpos) {}
    virtual void start_element(string, attrs_type&) {}
    virtual void end_element(string) {}
    virtual void sgml_decl(string, args_type&) {}
    virtual void start_ifdef(string) {}
    virtual void end_ifdef(string) {}

 private:
    void parse_tag();
    void parse_starttag();
    void parse_slashtag();
    void parse_endtag();
    void parse_decl();
    void parse_comment();
    void parse_ifdef();

    bool   skip_space();
    bool   skip_comment();
    string token(const char* delims);
    string tokenq(const char* delims);

    textpos text_;
    textpos end_;
    textpos ptr_;

    SGML(const SGML&);
    SGML& operator=(const SGML&);
    DBG_NEW
};


class SGMLTracer: public SGML
{
 public:
    SGMLTracer() {}
    SGMLTracer(textpos t, size_t l): SGML(t,l) {}

 protected:
    virtual void text_block(textpos, textpos);
    virtual void start_element(string, attrs_type&);
    virtual void end_element(string);
    virtual void sgml_decl(string, args_type&);
    virtual void start_ifdef(string);
    virtual void end_ifdef(string);
    DBG_NEW
};


#if 0

class SGMLNest: public SGML
{
 protected:
    virtual void text_block(textpos p, textpos q)
        { parent_->text_block(p, q); }
    virtual void start_element(string elem, attrs_type& attrs)
        { parent_->start_element(elem, attrs); }
    virtual void end_element(string elem)
        { parent_->end_element(elem); }
    virtual void sgml_decl(string decl, args_type& args)
        { parent_->sgml_decl(decl, args); }
    virtual void start_ifdef(string symbol)
        { parent_->start_ifdef(symbol); }
    virtual void end_ifdef(string symbol)
        { parent_->end_element(symbol); }
    
 private:
    SGML* parent;

    SGMLNest(const SGMLNest&);
    SGMLNest& operator=(const SGMLNest&);
    DBG_NEW
};

#endif

#endif /* SGML_H */
