/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  URL

#ifdef __GNUG__
#pragma implementation
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "URL_gen.h"

#include <strstream.h>
#include "util.h"

#if defined __GNUG__ && (defined EGCS || __GNUC__ >= 2 && __GNUC_MINOR__ >= 8)
#define remove erase
#endif


URL_gen::URL_gen(
    string url,
    URL    base):
    port_(0)
{
    DBG("gen", url << " (base=" << base << ") creating");

    string scheme, netloc, epath, efrag;
    string euser, epassword, ehost;
    URL::crack_url(url, scheme, netloc, epath, equery_, eparams_, efrag);
    URL::crack_netloc(netloc, euser, epassword, ehost, port_);
    path_ = simplify_path(URL::unescape(epath));
    frag_ = URL::unescape(efrag);
    host_ = URL::unescape(ehost);
    user_ = URL::unescape(euser);
    password_ = URL::unescape(epassword);

    DBG("gen", "cracked=[" << scheme << ',' <<
        user_ << ',' << password_ << ',' << host_    << ',' << port_ << ',' <<
        path_ << ',' << equery_   << ',' << eparams_ << ',' << frag_ << ']');

    if (!base) {
        if (!path_.length()) path_ = "/";
        return;
    }

    if (netloc.length()) {
        if (!path_.length()) path_ = "/";
        return;
    }
    /* else */
    user_     = base.user();
    password_ = base.password();
    host_     = base.host();
    port_     = base.port();

    if (!path_.length()) {
        path_ = base.path();
        if (frag_.length()) {
            if (!equery_.length())  equery_  = base.equery();
            if (!eparams_.length()) eparams_ = base.eparams();
        }
    } else if (path_ != base.path()) {
        path_ = absolute_path(path_, base.path());
    }
}


string
URL_gen::netloc() const
{
    strstream buf;
    
    if (user().length()) {
        buf << URL::escape(user(), netloc_bit);
        if (password().length())
            buf << ':' << URL::escape(password(), netloc_bit);
        buf << '@';
    }
    buf << URL::escape(host(), netloc_bit);
    if (port() != default_port()) buf << ':' << port();

    buf << ends;
    string result = buf.str();
    buf.freeze(false);
    return result;
}


string
URL_gen::parse(
    int bits) const
{
    string url;

    if (bits & scheme_bit) url += scheme(), url += ':';
    if (bits & netloc_bit) url += "//", url += netloc();
    if (bits & path_bit)   url += path();
    if (bits & query_bit) {
        if (query().length())  url += '?', url += equery();
        if (params().length()) url += ';', url += eparams();
    }
    if (bits & frag_bit && frag().length())
        url += '#', url += efrag();

    DBG("gen", "parsed=" << url << " (bits=" << bits << ')');
    return url;
}


string
URL_gen::rel(
    URL base,
    int bits) const
{
    if (scheme() != base.scheme() || netloc() != base.netloc())
        return this->parse(bits);

    string url;

    if (bits & path_bit) {
        if (path() != base.path() || !frag().length())
            url += URL::escape(relative_path(path(), base.path()), path_bit);
    }
    if (bits & query_bit) {
        if (query(). length()) url += '?', url += equery();
        if (params().length()) url += ';', url += eparams();
    }
    if (bits & frag_bit && frag().length()) {
        url += '#', url += efrag();
    }

    DBG("gen", "relative=" << url <<
        " (base=" << base << " bits=" << bits << ')');
    return url;
}


bool
URL_gen::is_valid() const
{
    return scheme().length() && path()[0] == '/' &&
        !(password().length() && !user().length());
}
    

string
URL_gen::basename() const
{
    string file = ::basename(this->path());
    if (file == "index.html"   || file == "index.htm" || 
        file == "welcome.html" || file == "Welcome.html" ||
        file == "default.html" || file == "default.htm")
        file = ::basename(dirname(this->path()));

    string host = this->host();
    return file.length() ? file: host.length() ? host: string("/");
}
