/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef URL_GEN_H
#define URL_GEN_H
#ifdef __GNUG__
#pragma interface
#endif

#include "URL.h"


class URL_gen: public URL_rep
{
    friend class URL;

 protected:
    URL_gen(string);
    URL_gen(string, URL);

    virtual string  scheme()   const = 0;
    virtual string  host()     const { return host_; }
    virtual string  user()     const { return user_; }
    virtual string  password() const { return password_; }
    virtual u_short port()     const { return port_ ? port_: default_port(); }
    virtual string  path()     const { return path_; }
    virtual string  query()    const { return URL::unescape(equery_); }
    virtual string  params()   const { return URL::unescape(eparams_); }
    virtual string  frag()     const { return frag_; }
    virtual string  netloc()   const;
    virtual string  equery()   const { return equery_; }
    virtual string  eparams()  const { return eparams_; }
    virtual string  efrag()    const { return URL::escape(frag_, frag_bit); }

    virtual bool host(string s)     { host_     = s;  return true; }
    virtual bool user(string s)     { user_     = s;  return true; }
    virtual bool password(string s) { password_ = s;  return true; }
    virtual bool port(u_short p)    { port_     = p;  return true; }
    virtual bool path(string s)     { path_     = s;  return true; }
    virtual bool query(string s)    { equery_   = URL::escape(s, query_bit);
                                                      return true; }
    virtual bool params(string s)   { eparams_  = URL::escape(s, query_bit);
                                                      return true; }
    virtual bool frag(string s)     { frag_     = s;  return true; }
    virtual bool equery(string s)   { equery_   = s;  return true; }
    virtual bool eparams(string s)  { eparams_  = s;  return true; }
    virtual bool efrag(string s)    { frag_ = URL::unescape(s);  return true; }
    virtual u_short default_port() const = 0;

    virtual string parse(int) const;
    virtual string rel(URL, int) const;
    virtual string basename() const;

    virtual bool is_valid() const;
    
 private:
    string  user_;
    string  password_;
    string  host_;
    u_short port_;
    string  path_;
    string  equery_;
    string  eparams_;
    string  frag_;
    DBG_NEW
};


class URL_http: public URL_gen
{
    friend class URL;

 protected:
    URL_http(string url, URL base): URL_gen(url, base) {}

    virtual URL_http* clone() const { return new URL_http(*this); }

    virtual string  scheme()       const { return "http"; }
    virtual u_short default_port() const { return 80; }
    virtual bool    is_valid()     const { return URL_gen::is_valid()
                                                 && host().length(); }

    static URL_rep* make_rep(string url, URL base) {
        return new URL_http(url, base);
    }
    DBG_NEW
};


class URL_ftp: public URL_gen
{
    friend class URL;

 protected:
    URL_ftp(string url, URL base): URL_gen(url, base) {}

    virtual URL_ftp* clone() const { return new URL_ftp(*this); }

    virtual string  scheme()       const { return "ftp"; }
    virtual u_short default_port() const { return 21; }
    virtual bool    is_valid()     const { return URL_gen::is_valid()
                                                && host().length(); }

    static URL_rep* make_rep(string url, URL base) {
        return new URL_ftp(url, base);
    }
    DBG_NEW
};


class URL_file: public URL_gen
{
    friend class URL;

 protected:
    URL_file(string url, URL base): URL_gen(url, base) {}

    virtual URL_file* clone() const { return new URL_file(*this); }

    virtual string  scheme()       const { return "file"; }
    virtual u_short default_port() const { return 0; }

    static URL_rep* make_rep(string url, URL base) {
        return new URL_file(url, base);
    }
    DBG_NEW
};


class URL_nntp: public URL_gen
{
    friend class URL;

 protected:
    URL_nntp(string url, URL base): URL_gen(url, base) {}

    virtual URL_nntp* clone() const { return new URL_nntp(*this); }

    virtual string  scheme()       const { return "nntp"; }
    virtual u_short default_port() const { return 119; }

    static URL_rep* make_rep(string url, URL base) {
        return new URL_nntp(url, base);
    }
    
 private:
    string group_;
    int    article_;
    DBG_NEW
};

#endif  /* URL_GEN_H */
