/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef URL_NEWS_H
#define URL_NEWS_H
#ifdef __GNUG__
#pragma interface
#endif

#include "URL.h"


class URL_news: public URL_rep
{
    friend class URL;

 protected:
    URL_news(string);
    URL_news(string, URL);

    virtual string scheme()   const { return "news"; }
    virtual string mssageid() const { return path_.find('@') != string::npos ?
                                          path_ : string(); }
    virtual string group()    const { return path_.find('@') == string::npos ?
                                          path_ : string(); }
    virtual string frag()     const { return frag_; }
    virtual string efrag()    const { return URL::escape(frag_, frag_bit); }

    virtual bool messageid(string s) { path_ = s; return true; }
    virtual bool group(string s)     { path_ = s; return true; }
    virtual bool frag(string s)      { frag_ = s; return true; }
    virtual bool efrag(string s)     { frag_ = URL::unescape(s); return true; }

    virtual string parse(int) const;
    virtual string rel(URL, int) const;
    virtual string basename() const;

    virtual bool is_valid() const;
    
 private:
    string path_;
    string frag_;
    DBG_NEW
};


class URL_mail: public URL_news
{
    friend class URL;

 protected:
    URL_mail(string);
    URL_mail(string, URL);

    virtual string scheme() const { return "mail"; }
    virtual string group()  const { return string(); }
    virtual bool group(string) { return false; }

    virtual bool is_valid() const;

    DBG_NEW
};


class URL_mailto: public URL_rep
{
    friend class URL;

 protected:
    URL_mailto(string);
    URL_mailto(string, URL);

    virtual string scheme()   const { return "mailto"; }
    virtual string mssageid() const { return path_.find('@') != string::npos ?
                                          path_ : string(); }
    virtual string group()    const { return path_.find('@') == string::npos ?
                                          path_ : string(); }
    virtual string frag()     const { return frag_; }
    virtual string efrag()    const { return URL::escape(frag_, frag_bit); }

    virtual bool messageid(string s) { path_ = s; return true; }
    virtual bool group(string s)     { path_ = s; return true; }
    virtual bool frag(string s)      { frag_ = s; return true; }
    virtual bool efrag(string s)     { frag_ = URL::unescape(s); return true; }

    virtual string parse(int) const;
    virtual string rel(URL, int) const;
    virtual string basename() const;

    virtual bool is_valid() const;
    
 private:
    string user_;
    string domain_;
    DBG_NEW
};

#endif  /* URL_NEWS_H */
