#! /usr/local/bin/perl -w
#/*
#//  The Natto View
#//
#//  Copyright (c) 1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
#//
#//  The code in this file may be used, copied and redistributed under the
#//  same terms as perl itself (see the files README, Copying and Artistic
#//  in the perl distribution).
#*/

use FileHandle;
use strict;
#use diagnostics;

my $verbose = 1;
my $catalog_file = 'catalog';
open(STDOUT, "|sort");

my %catalog;
my %macros;
my @fixeds;

if ($ARGV[0] eq '-c') {
    $catalog_file = $ARGV[1];
    shift;
    shift;
}

my $fh = new FileHandle "< $catalog_file";
if (defined $fh) {
    while (<$fh>) {
	/PUBLIC\s+\"([^\"]+)\"\s+(\S+)/i;
	$catalog{$1} = $2;
    }
} else {
    die "Can't open catalog file \"$catalog_file\".\n";
}

warn "Reading dtd...\n" if $verbose;
undef $/;
$_ = <>;
remove_comments($_);

sub remove_comments($)
{
    $_[0] =~ s/<!--(.*?)-->//gsx;    # remove comments 
    $_[0] =~ s/--(.*?)--//gsx;       # remove commnets in definitions
    $_[0] =~ s/%\s*/%/gs;            # % name --> %name
    $_[0] =~ s/\s*;/;/gs;            # name ; --> name;
    $_[0] =~ s/\s*([|&,])\s*/$1/gse; # name1 | name2 --> name1|name2
    $_[0] =~ s/\(\s*/\(/gs;          # ( name --> (name
    $_[0] =~ s/\s*\)/\)/gs;          # name ) --> name)
    $_[0];
}

my $phase = 1;
while (/<!ENTITY\s+%/s) {
    warn "Processing macro entities ($phase)...\n" if $verbose;
    while (s/<!ENTITY\s+%(\S+)\s+(.*?)\s*>/macro($1, $2)/sexi) {
	my $key = $1;
	if (s/<!ENTITY\s+%$key\s+(.*?)\s*>//gsxi) {
	    warn "Redef of macro entity \"%$key\" is ignored.\n" if $verbose;
	}
	s/%$key;?/$macros{$key}/gse;
	s/<!\[\s*IGNORE\s*\[(.*?)\]\s*\]>//gsxi;
	s/<!\[\s*INCLUDE\s*\[(.*?)\]\s*\]>/$1/gsxi;
    }
    $phase++;
}
if (/%[\w.-]+;?/) {
    die "\"$&\" can't be expanded.\n";
}

sub macro($, $)
{
    my($key, $value) = @_;
    if ($value =~ s/^PUBLIC//i) {
	if ($value =~ /\s*\"([^\"]+)\"/) {
	    $value = $1;
	}
	my $fh = new FileHandle "< $catalog{$value}";
	if (defined $fh) {
	    warn "Including \"$value\"...\n" if $verbose;
	    my $content = <$fh>;
	    remove_comments($content);
	    $macros{$key} = $content;
	} else {
	    warn "\"$value\" not found!\n";
	}		
    } elsif (!defined $macros{$key}) {
	trim_quotation($value);
	$macros{$key} = $value;
    }
    '';
}    

print "0-BEGIN\n";

warn "Processing character entities...\n" if $verbose;
my %entities;
s/<!ENTITY\s+(\S+)\s+(.*?)\s*>/entity($1, $2)/gsexi;

sub entity($, $)
{
    my($key, $value) = @_;
    if ($value =~ s/^[CS]DATA//i) {
	trim_quotation($value);
	#print "2-ENTITY $key\t$value\n";
	if (!defined $entities{$key}) {
	    if ($value =~ /&#(\d+);?/) {
		$value = "&#$1;";
		#print "2-ENTITY $key\t$value\n";
		$entities{$key} = $value;
	    } elsif ($value =~ /=([0-9a-fA-F][0-9a-fA-F])/) {
		my $value = '&#' . hex($1) . ';';
		#print "2-ENTITY $key\t$value\n";
		$entities{$key} = $value;
	    } else {
		print "ENTITY $key\t$value\n";
		$entities{$key} = $value;
	    }
	} else {
	    warn "Redef. of char entity \"$key\" is ignored.\n" if $verbose;
	}
    }
    '';
}    

my $key;
foreach $key (sort keys %entities) {
    print "2-ENTITY $key\t$entities{$key}\n";
}
print "3-EE\n";

warn "Processing elements...\n" if $verbose;
s/<!ELEMENT\s+(\S+)\s+([-O])\s+([-O])\s+(.*?)\s*>/
    element(lc $1, $2, $3, $4)/gsexi;

sub element($, $, $, $)
{
    my($key, $start, $stop, $content) = @_;
    $key =~ s/\s+//g;
    $key =~ s/^\(([^\)]*)\)$/$1/x;
    my $elem;
    foreach $elem (split(/\|/, $key)) {
	my $ctype =
	    ($content =~ /^CDATA$/i || $content =~ /^EMPTY$/i) ? $content: '-';
	print "5-ELEMENT $elem\t0-new\t$start $stop $ctype\n";
	print "5-ELEMENT $elem\t9-end\n";

	if ($ctype eq '-') {
	    #print "5-ELEMENT $elem\t2-cont\t$content\n";
	    my($base) = $content =~ /^(\S+)/;
	    my @inc   = $content =~ /\s+\+\(([^)(]*)\)/g;
	    my @exc   = $content =~ /\s+\-\(([^)(]*)\)/g;
	    $base = join(' ', $base, @inc);
	    $base =~ s/[^\w\#]+/ /g;
	    $base =~ s/^ +//;
	    $base =~ s/ +$//;
	    my $e;
	    foreach $e (split(/[^\w\#]+/, "@exc")) {
		$base =~ s/^$e //;
		$base =~ s/ $e$//;
		$base =~ s/ $e / /g;
	    }
	    my %has;
	    foreach $e (split(/ +/, $base)) {
		$has{$e}++;
	    }
	    foreach $e (keys %has) {
		$e = ($e =~ /^\w+$/) ? lc $e : uc $e;
		print "5-ELEMENT $elem\t3-has\t$e\n";
	    }
	}
    }
    '';
}

1 while s/\#FIXED\s*"([^"]*)"/fixed($1)/gsei;
1 while s/\#FIXED\s*'([^']*)'/fixed($1)/gsei;

sub fixed($)
{
    push(@fixeds, $_[0]);
    "#FIXED-$#fixeds";
}

warn "Processing attribute lists...\n" if $verbose;
s/<!ATTLIST\s+(\S+)\s+(.*?)\s*>/attlist(lc $1, $2)/gsexi;

sub attlist($, $)
{
    my($key, $attrs) = @_;
    $key =~ s/^\(\s*//;
    $key =~ s/\s*\)$//;
    while ($attrs =~ s/^\s*(\S+)\s+(\S+)\s*(\S+)//s) {
        my($name, $type, $req) = (lc $1, $2, $3);
	if ($req =~ /^\#FIXED-(\d+)/i) {
	    $req = "#FIXED \"$fixeds[$1]\"";
	} 
	elsif ($req !~ /^\#/) {
	    trim_quotation($req);
	    $req = "#DEFAULT \"$req\"";
	}
	my $elem;
        foreach $elem (split(/\|/, $key)) {
	    print "5-ELEMENT $elem\t1-att\t$name\t$req\n";
	}
    }
    '';
}


sub trim_quotation($)
{
    $_[0] =~ s/^\s*"([^"]*)"\s*$/$1/sx;
    $_[0] =~ s/^\s*'([^']*)'\s*$/$1/sx;
    $_[0] =~ s/^\s*//s;
    $_[0] =~ s/\s*$//s;
    $_[0];
}

