#! /usr/local/bin/perl -w
#/*
#//  The Natto View
#//
#//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
#//
#//  The code in this file may be used, copied and redistributed under the
#//  same terms as perl itself (see the files README, Copying and Artistic
#//  in the perl distribution).
#*/

use FileHandle;
use strict;
#use diagnostics;

my $elem = '';
my @elems = ();

my $eel_file = $ARGV[0];
my @basetemp = reverse split(/\./, $eel_file);
my $basename = join('.', reverse pop(@basetemp));
my $c_file   = $basename . '.c';
my $h_file   = $basename . '.h';

my $fh_eel = new FileHandle "< $eel_file";
if (!defined $fh_eel) { die "Can't open eel file \"$eel_file\".\n"; }
my $fh_c   = new FileHandle "> $c_file";
if (!defined $fh_c)   { die "Can't open .c file \"$c_file\".\n"; }
my $fh_h   = new FileHandle "> $h_file";
if (!defined $fh_h)   { die "Can't open .h file \"$h_file\".\n"; }

my $incname = $basename;
my $defname = uc $incname;
$defname =~ s/-/_/g;
my $dtdname = $basename;
$dtdname =~ s/-(\w)\w*/uc $1/ge;

$fh_c->print(<<END);
/*
//  Don't modify.  This file is a generated file. 
*/\n
#include <stddef.h>
#include <WWWUtil.h>
#include "${incname}.h"\n
END

$fh_h->print(<<END);
/*
//  Don't modify.  This file is a generated file. 
*/\n
#ifndef ${defname}_H
#define ${defname}_H
#include <HTStruct.h>
#include <SGML.h>\n
END

my $nentities = 0;
my $nattrs    = 0;
my $elemtab   = '';

while (<$fh_eel>) {
    if (/^\d-BEGIN/) {
	$fh_c->print("/* Entities */\n");
	$fh_c->print("PRIVATE const char* entities\[\] = {\n");
    }
    elsif (/^\d-ENTITY\s+(\S+)/) {
	$fh_c->print("\t\"$1\",\n");
        $nentities++;
    }
    elsif (/^\d-EE/) {
	$fh_c->print("};\n\n");
	$fh_c->print("/* Element Attributes */\n");
	$fh_h->print("/* Element Attributes */\n");
    }
    elsif (s/^\d-ELEMENT\s+(\S+)\s+(\S+)\s+//) {
	my $elem = lc $1;
	my $op   = $2;
        if ($op =~ /\d-new/) {
	    /([-O])\s+([-O])+(\w+)/;
	    my $type  = $3 eq 'CDATA' ? "SGML_LITERAL" :
                       ($2 eq 'O'     ? "SGML_EMPTY"   : "SGML_MIXED");
	    my $attrs = "${elem}_attr";
	    $elem = uc $elem;
            my $name  = "\"$elem\"";
	    my $nattr = "${dtdname}_${elem}_ATTRIBUTES";
	    $elemtab .= "{ $name,\t$attrs,\t$nattr,\t$type },\n";
            $fh_c->print("PRIVATE attr $attrs\[\] = {\n");
	    $fh_h->print("/* $elem */\n");
	    push(@elems, $elem);
	}
	if ($op =~ /\d-att/) {
	    my($attr) = /(\w+)/;
	    $attr = uc $attr;
	    $fh_c->print("\t{ \"$attr\" },\n");
	    $attr =~ s/-/_/g;
	    $elem = uc $elem;
            $fh_h->print("#define ${dtdname}_${elem}_${attr}\t${nattrs}\n");
	    $nattrs++;
	}
        if ($op =~ /\d-end/) {
	    $fh_c->print("\t{ NULL }\n};\n\n");
	    $elem = uc $elem;
            $fh_h->print(
	        "#define ${dtdname}_${elem}_ATTRIBUTES\t${nattrs}\n\n");
	    $nattrs = 0;
	}
    }
}

$fh_c->print(<<END);
/* Elements */
PRIVATE HTTag tags\[\] = {\n$elemtab};\n
PUBLIC const SGML_dtd ${dtdname}_dtd = {
    tags,     sizeof(tags)/sizeof(tags\[0\]),
    entities, sizeof(entities)/sizeof(entities\[0\])
};
END

$fh_h->print("/* Elements */\ntypedef enum _${dtdname}Element {\n");
foreach $elem (@elems) {
    $fh_h->print("\t${dtdname}_${elem},\n");
}
$fh_h->print("\t${dtdname}_ELEMENTS\n} ${dtdname}Element;\n\n");

$fh_h->print(<<END);
#define ${dtdname}_ENTITIES $nentities\n
extern const SGML_dtd ${dtdname}_dtd;\n
#endif /* ${defname}_H */
END
