/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  Mem

#ifdef __GNUG__
#pragma implementation
int dummy_memobject_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "memobject.h"

#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include "util.h"

#ifndef MAP_FILE
#define MAP_FILE  0
#endif

#include <string.h>
#include <errno.h>

extern "C" int mkdir2(const char* path, mode_t mode);


mapped_file::mapped_file()
{}


mapped_file::mapped_file(
    string fname,
    size_t size,
    int    flags,
    mode_t perm)
{
    this->map(fname, size, flags, perm);
}


mapped_file::~mapped_file()
{
    this->unmap();
}


#define SYSERR(submod, func)  \
do { DBG(submod, func << ": " << strerror(errno)); return; } while (0)
    
void
mapped_file::map(
    string fname,
    size_t size,
    int    flags,
    mode_t perm)
{
    int f = O_CREAT;
    int p = 0;
    switch (flags & (in | out)) {
     case in:
        f |= O_RDONLY;
        p |= PROT_READ;
        break;
     case out:
        f |= O_WRONLY;
        p |= PROT_WRITE;
        break;
     case in | out:
        f |= O_RDWR;
        p |= PROT_READ | PROT_WRITE;
        break;
    }
    if (flags & trunc) f |= O_TRUNC;

    mkdir2(dirname(fname).c_str(), perm | 0111);
    int fd = open(fname.c_str(), f, perm);
    if (fd < 0) SYSERR("mmap", "open");
    
    struct stat buf;
    fstat(fd, &buf);
    if (size == 0) size = buf.st_size;
    if (size > (size_t) buf.st_size &&
        (lseek(fd, size - 1, SEEK_SET) < 0 || write(fd, "", 1) != 1)) {
        close(fd);
        SYSERR("mmap", "lseek/write");
     }
    void* area = mmap(NULL, size, p, MAP_FILE | MAP_SHARED, fd, 0);
    close(fd);
    if (area == (void*) -1) SYSERR("mmap", "mmap");
    this->init(area, size);
}


void
mapped_file::unmap()
{
    munmap(this->area(), this->size());
    this->init(NULL, 0);
}


void
mapped_file::sync()
{
    msync(this->area(), this->size(), MS_SYNC);
}


heap_memory::heap_memory()
{}


heap_memory::heap_memory(
    size_t size)
{
    this->alloc(size);
}


heap_memory::heap_memory(
    char*  area,
    size_t size)
{
    this->init(area, size);
}


heap_memory::~heap_memory()
{
    this->_free();
}


void
heap_memory::alloc(
    size_t size)
{
    char* area = new char[size];
    this->init(area, size);
}


void
heap_memory::_free()
{
    delete [] (char*) area();
}
