/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef MEMOBJECT_H
#define MEMOBJECT_H
#ifdef __GNUG__
#pragma interface
#endif

#include <fcntl.h>
#include <stddef.h>
#include <string>


class memobject
{
 public:
    memobject(): area_(0), size_(0) {}
    memobject(void* area, size_t size) { this->init(area, size); }
    virtual ~memobject() {}

    void*       area()       { return area_; }
    const void* area() const { return area_; }
    size_t      size() const { return size_; }

 protected:
    void   init(void* area, size_t size) { area_ = area;  size_ = size; }

 private:
    void*  area_;
    size_t size_;

    memobject(const memobject&);
    memobject& operator=(const memobject&);
    DBG_NEW
};


class mapped_file: public memobject
{
 public:
    enum { in = 1, out = 2, trunc = 4 };

    mapped_file();
    mapped_file(string fname, size_t size, int open_mode, mode_t perm);
    virtual ~mapped_file();

    void map(string fname, size_t size, int open_mode, mode_t perm);
    void unmap();
    void sync();
    DBG_NEW
};


class heap_memory: public memobject
{
 public:
    heap_memory();
    heap_memory(size_t);
    heap_memory(char*, size_t);  // already alloced buffer
    virtual ~heap_memory();

    void alloc(size_t);
    void _free();
    DBG_NEW
};

#endif /* MEMOBJECT_H */
