/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  Net

#ifdef __GNUG__
#pragma implementation
int dummy_netstream_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "netstream.h"

#include <fstream.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <errno.h>
#include <string.h>


#if !defined(HAVE_GETHOSTNAME) && defined(HAVE_UNAME)
int
gethostname(
    char*  name,
    size_t len)
{
    struct utsname buf;
    int ret = uname(&buf);
    strncpy(name, buf.nodename, len - 1);
    name[len - 1] = '\0';
    return ret;
}
#endif


netstream::dns_cache_type netstream::dns_cache_;


netstream::netstream()
{}


netstream::netstream(
    string  host,
    u_short port)
{
    this->open(host, port);
}


netstream::netstream(
    string host,
    string service)
{
    this->open(host, service);
}


bool
netstream::dns_query(
    string host,
    struct in_addr& addr)
{
    memset(&addr, 0, sizeof(addr));

    if (dns_cache_.find(host) != dns_cache_.end()) {
        addr = dns_cache_[host];
        DBG("dns", host << " cached");
        return true;
    }

    struct hostent* hent = NULL;
    int herr = 0;
#ifdef _REENTRANT
    char hbuf[512];
#endif
    //do {
        DBG("dns", "looking up " << host);
#ifdef _REENTRANT
        hent = gethostbyname_r(host.c_str(), &hent, hbuf, sizeof(hbuf), &herr);
#else
        hent = gethostbyname(host.c_str());
        herr = h_errno;
#endif
    //} while (!hent && herr == TRY_AGAIN);
    if (!hent) {
        if (herr == NETDB_INTERNAL)
            DBG("dns", "gethostbyname: " << strerror(errno));
        else
            DBG("dns", "gethostbyname: " << herr);
        return false;
    }
    if (hent->h_addrtype != AF_INET) {
        DBG("dns", "gethostbyname: address type not AF_INET");
        return false;
    }
    memcpy(&addr, hent->h_addr, (size_t) hent->h_length);

    dns_cache_[hent->h_name] = addr;
    for (char** alias = hent->h_aliases; *alias; alias++)
        dns_cache_[*alias] = addr;
    return true;
}


#define SYSERR(submod, func)  \
do { DBG(submod, func << ": " << strerror(errno)); return; } while (0)


void
netstream::open(
    string  rhost,
    u_short rport)
{
    if (this->is_open()) this->close();

    char lhost[128] = "";
    if (gethostname(lhost, sizeof(lhost)) < 0)
        SYSERR("io", "gethostname");
    lhost[sizeof(lhost) - 1] = '\0';

    struct in_addr laddr;
    if (!this->dns_query(lhost, laddr)) return;
    struct sockaddr_in thisaddr;
    thisaddr.sin_family = AF_INET;
    thisaddr.sin_port   = 0;
    thisaddr.sin_addr   = laddr;

    struct in_addr raddr;
    if (!this->dns_query(rhost, raddr)) return;
    struct sockaddr_in thataddr;
    thataddr.sin_family = AF_INET;
    thataddr.sin_port   = htons(rport);
    thataddr.sin_addr   = raddr;

#ifdef DEBUGGING
    string lntoa = inet_ntoa(laddr);
    string rntoa = inet_ntoa(raddr);
    DBG("io", lhost << " (" << lntoa << ") -> "
        << rhost << ':' << rport << " (" << rntoa << ')');
#endif

    int sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock < 0) SYSERR("io", "socket");

    if (bind(sock, (struct sockaddr*) &thisaddr, sizeof(thisaddr)) < 0) {
        ::close(sock);
        SYSERR("io", "bind");
    }
    if (connect(sock, (struct sockaddr*) &thataddr, sizeof(thataddr)) < 0) {
        ::close(sock);
        SYSERR("io", "connect");
    }
    this->attach(sock);
}


void
netstream::open(
    string host,
    string service)
{
    struct servent* sent = NULL;
#ifdef _REENTRANT
    char sbuf[512];
    sent = getservbyname_r(service.c_str(), "tcp", &sent, sbuf, sizeof(sbuf));
#else
    sent = getservbyname(service.c_str(), "tcp");
#endif
    if (!sent) SYSERR("io", "getservbyname");
    this->open(host.c_str(), sent->s_port);
}


#if 0
void
netstream::listen(
#endif
