/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "pipestream.h"

#include <unistd.h>
#include <sys/socket.h>


inline int
pipe2(
    int fd[2])
{
#if 0
    return pipe(fd);
#else
    return socketpair(AF_UNIX, SOCK_STREAM, 0, fd);
#endif
}


pipestream::pipestream()
{}


pipestream::pipestream(
    int fd):
    fstream(fd)
{}


pipestream::pipestream(
    const char* file,
    char* const argv[])
{
    this->open_execvp(file, argv);
}


void
pipestream::open(
    const char* file,
    char* const argv[])
{
    this->open_execvp(file, argv);
}


void
pipestream::open_execvp(
    const char* file,
    char* const argv[])
{
    if (this->is_open()) this->close();

    int fd[2];
    if (pipe2(fd) < 0) return;

    pid_t pid = fork();
    if (pid == 0) {
	/* child */
	::close(fd[0]);
	dup2(fd[1], 0);
	dup2(fd[1], 1);
	execvp(file, argv);
	exit(0);
    }
    else if (pid < 0) {
	/* error */
	::close(fd[0]);
	::close(fd[1]);
	return;
    }
    /* parent */
    ::close(fd[1]);
    this->attach(fd[0]);
}


void
pipestream::open_execv(
    const char* file,
    char* const argv[])
{
    if (this->is_open()) this->close();

    int fd[2];
    if (pipe2(fd) < 0) return;

    pid_t pid = fork();
    if (pid == 0) {
	/* child */
	::close(fd[0]);
	dup2(fd[1], 0);
	dup2(fd[1], 1);
        execv(file, argv);
	exit(0);
    }
    else if (pid < 0) {
	/* error */
	::close(fd[0]);
	::close(fd[1]);
	return;
    }
    /* parent */
    ::close(fd[1]);
    this->attach(fd[0]);
}
