
#include <stdio.h>
#include <string.h>
#include "timestr.h"


static char* Mon_names[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static const char* Wday_names[7] = {
    "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
};


int
strtomonth(
    const char* name)
{
    int i;
    for (i = 0; i < 12; i++)
        if (strncasecmp(Mon_names[i], name, 3) == 0) return i;
    return -1;
}



time_t
strtotime(
    const char* s)
{
    int year, mon, mday, hour, min, sec;
    char wdayname[16] = "";
    char monname [16] = "";
    struct tm tmbuf;
    time_t t;
    
    year = mon = mday = hour = min = sec = 0;
    
    /* RFC 1128:  Wkd, DD Mon YYYY HH:MM:SS GMT */
    if (sscanf(s, "%3s, %02d %3s %04d %02d:%02d:%02d",
               wdayname, &mday, monname, &year, &hour, &min, &sec) == 7) {
        mon = strtomonth(monname);
        if (mon < 0) mon = 0;  /* ##### */
        /* fprintf(stderr, "RFC1128:"); */
    }
    /* ISO 8601:  YYYY.MM.DDThh:mm:ssZ */
    else if (sscanf(s, "%04d.%02d.%02dT%02d:%02d:%02dZ",
                    &year, &mon, &mday, &hour, &min, &sec) == 6) {
        mon--;
        /* fprintf(stderr, "ISO8601:"); */
    }
    /* RFC 850:  Weekday, DD-Mon-YY HH:MM:SS GMT */
    else if (sscanf(s, "%15[^,], %02d-%3s-%02d %02d:%02d:%02d",
                    wdayname, &mday, monname, &year, &hour, &min, &sec) == 7) {
        year += 1900;
        mon = strtomonth(monname);
        if (mon < 0) mon = 0;  /* ##### */
        /* fprintf(stderr, "RFC850:"); */
    }

    memset(&tmbuf, 0, sizeof(tmbuf));
    tmbuf.tm_year = year - 1900;
    tmbuf.tm_mon  = mon;
    tmbuf.tm_mday = mday;
    tmbuf.tm_hour = hour;
    tmbuf.tm_min  = min;
    tmbuf.tm_sec  = sec;
    t = mktime(&tmbuf);
    if (t == (time_t) -1) return (time_t) -1;
    t -= timezone;  /* ######################################## */
    /* fprintf(stderr, "%s\n", ctime(&t)); */
    return t;
}


char*
strf_rfc1128time(
    const time_t* t,
    char*         buf)
{
    struct tm* gmt;
#ifdef _REENTRANT
    struct tm tmbuf;
    gmt = gmtime_r(t, &tmbuf);
#else
    gmt = gmtime(t);
#endif
    /* Wkd, DD Mon YYYY hh:mm:ss TZD */
    /* The strftime('s month representation) depends on locales,
       so we use sprintf */ 
    sprintf(buf,"%s, %02d %s %4d %02d:%02d:%02d GMT",
            Wday_names[gmt->tm_wday],
            gmt->tm_mday, Mon_names[gmt->tm_mon], gmt->tm_year + 1900,
            gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
    return buf;
}


char*
strf_iso8601time(
    const time_t* t,
    char*         buf)
{
    struct tm* gmt;
#ifdef _REENTRANT
    struct tm tmbuf;
    gmt = gmtime_r(t, &tmbuf);
#else
    gmt = gmtime(t);
#endif
    /* YYYY.MM.DDThh:mm:ssZ */
    sprintf(buf,"%04d.%02d.%02dT%02d:%02d:%02dZ",
            gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday,
            gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
    return buf;
}
