/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef UTIL_H
#define UTIL_H
#ifdef __GNUG__
#pragma interface
#endif

#include <string>
#include <istream.h>
#include <ostream.h>

istream& copy_line(istream&, ostream&);
string   trim_space(string);
string   squeeze_space(string);
string   lc(string);
string   uc(string);
string   capitalize2(string);

string   simplify_path(string);
string   basename(string, string = "");
string   dirname(string);
string   absolute_path(string relpath, string curpath);
string   relative_path(string abspath, string curpath);


class coutbuf: public streambuf
{
 public:
    virtual int        sync();
    virtual int        overflow(int);
    virtual streamsize xsputn(char*, streamsize);
    DBG_NEW
};

class blackholebuf: public streambuf
{
 public:
    virtual int        sync()                      { return 0; }
    virtual int        overflow(int)               { return 0; }
    virtual streamsize xsputn(char*, streamsize n) { return n; }
    DBG_NEW
};

#ifndef NO_MSGOUT
extern ostream msgout;
#endif



template <class OutputIterator>
void
split(
    string         str,
    OutputIterator output,
    string         delim = " ")
{
    string::size_type i = 0;
    string::size_type len  = str.length();
    string::size_type dlen = delim.length();
    while (i <= len) {
        size_t j = str.find(delim, i);
        if (j != string::npos) {
            *output++ = str.substr(i, j - i);
            i = j + dlen;
        }
        else {
            *output++ = str.substr(i);
            break;
        }
    }
}


template <class InputIterator>
string
join(
    InputIterator first,
    InputIterator last,
    string        delim = " ")
{
    string str;
    if (first != last) {
        str += *first++;
        while (first != last) {
            str += delim;
            str += *first++;
        }
    }
    return str;
}    

#endif /* UTIL_H */
