/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef UTILSTL_H
#define UTILSTL_H

template <class Container>
void
clear_container(
    Container& x)
{
    while (x.size()) x.pop_back();
}

#include <map>

template <class Key, class T>
struct rbmap: map<Key, T, less<Key> > {};

#if 0

#include <vector>
#include <algorithm>

template <class Key, class T, class Eq>
struct simple_map
{
    typedef T data_type;
    typedef const Key key_type;
    typedef pair<key_type, data_type> value_type;

    typedef vector<value_type> vec;
    vec rep;

    typedef vec::iterator iterator;
    typedef vec::const_iterator const_iterator;
    typedef vec::pointer pointer;
    typedef vec::const_pointer const_pointer;
    typedef vec::reference reference;
    typedef vec::const_reference const_reference;
    typedef vec::size_type size_type;

    size_type size()  const { return rep.size(); }
    bool      empty() const { return rep.empty(); }
    
    iterator begin() { return rep.begin(); }
    iterator end()   { return rep.end(); }
    const_iterator begin() const { return rep.begin(); }
    const_iterator end()   const { return rep.end(); }

    pair<iterator, bool> insert(const value_type& obj) {
        vec::iterator i = this->find(obj.first);
        if (i != rep.end()) return pair<iterator, bool>(i, false);
        return pair<iterator, bool>(rep.insert(obj), true);
    }

    iterator find(const key_type& key) {
        return ::find(rep.begin(), rep.end(), value_type(key, data_type()));
    }
    const_iterator find(const key_type& key) const {
        return ::find(rep.begin(), rep.end(), value_type(key, data_type()));
    }

    T& operator[](const key_type& key)
    {
        value_type val(key, data_type());
        return rep.insert(val).second;
    }

    size_type erase(const key_type& key) {
        vec::iterator i = this->find(key);
        if (i != rep.end()) rep.erase(i);
        return rep.size();
    }
    void erase(iterator i) { rep.erase(i); }
    void erase(iterator f, iterator l) { rep.erase(f, l); }
    // void clear() { rep.clear(); }
    DBG_NEW
};

template <class Key, class T>
struct smap: simple_map<Key, T, equal_to<Key> > {};

#else

#define smap rbmap

#endif

#ifdef HAVE_HASH_MAP

#include <hash_map.h>
#include <string>

struct hash<string>
{
    size_t operator()(string s) const {
        unsigned long h = 0; 
        for (string::size_type i = 0; i < s.length(); i++)
            h = 5 * h + s[i];
        return size_t(h);
    }
    DBG_NEW
};

template <class Key, class T>
struct hmap<Key, T>: hash_map<Key, T, hash<Key>, equal_to<Key> > {};

#else

#define hmap rbmap

#endif

#endif /* UTILSTL_H */
