/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  Motif

#ifdef __GNUG__
#pragma implementation
int dummy_AnchorBox_Xm_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "AnchorBox_Xm.h"

#include <Xm/XmAll.h>
#include "utilstl.h"
#include "utilx.h"
#include "SiteBar_Xm.h"


AnchorBox_Xm::AnchorBox_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj),
    list_(NULL), btnbar_(NULL), prev_node_(NULL)
{}


void
AnchorBox_Xm::init(
    GuiConfig* config)
{
    if (config) config_ = *DCAST(MotifConfig*, config);
    this->delayed_init();
}


void
AnchorBox_Xm::delayed_init()
{
    MotifConfig conf;
    DBG("xm", obj()->name() << " initialiazing");

    GuiObject_Xm* parent = 
	obj()->parent() ? DCAST(GuiObject_Xm*, obj()->parent()->imp()): 0;
    this->widget(XtCreatePopupShell("anchorbox", topLevelShellWidgetClass,
				    parent ? parent->widget(): 0,
                                    config_.args(), config_.size()));
    AddCallback(widget(), XmNdestroyCallback, shell_destroy);

    obj()->form()->create(NULL);
    obj()->form()->manage();
    Widget form = DCAST(GuiObject_Xm*, obj()->form()->imp())->widget();

    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_FORM)
        .add(XmNbottomAttachment, XmATTACH_NONE)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    obj()->sitebar()->create(&conf);
    obj()->sitebar()->manage();
    Widget sitebar = DCAST(GuiObject_Xm*, obj()->sitebar()->imp())->widget();
    
    conf.clear();
    conf.add(XmNorientation,      XmHORIZONTAL)
        .add(XmNtopAttachment,    XmATTACH_NONE)
        .add(XmNbottomAttachment, XmATTACH_FORM)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    btnbar_ = XmCreateRowColumn(form, "btnbar", conf.args(), conf.size());
    XtManageChild(btnbar_);

    Widget w;
    w = XmCreatePushButton(btnbar_, "focus", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, focus_activate);
    w = XmCreatePushButton(btnbar_, "scan", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, scan_activate);
    w = XmCreatePushButton(btnbar_, "browse", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, browse_activate);
    w = XmCreatePushButton(btnbar_, "tour", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, tour_activate);
    w = XmCreatePushButton(btnbar_, "close", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, close_activate);

    conf.clear();
    conf.add(XmNselectionPolicy, XmBROWSE_SELECT);
    list_ = XmCreateScrolledList(form, "list", conf.args(), conf.size());
    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_WIDGET)
        .add(XmNtopWidget,        sitebar)
        .add(XmNbottomAttachment, XmATTACH_WIDGET)
        .add(XmNbottomWidget,     btnbar_)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    XtSetValues(XtParent(list_), conf.args(), conf.size());
    XtManageChild(list_);
    AddCallback(list_, XmNdefaultActionCallback,   list_action);
    AddCallback(list_, XmNbrowseSelectionCallback, list_selection);
}


AnchorBox*
AnchorBox_Xm::obj()
{
    return DCAST(AnchorBox*, GuiObject_imp::obj());
}


GuiObject_if*
AnchorBox_Xm::interface()
{
    return this;
}


void
AnchorBox_Xm::popup()
{
    DBG("xm", obj()->name() << " poping up");
    XtPopup(widget(), XtGrabNone);
}


void
AnchorBox_Xm::popdown()
{
    DBG("xm", obj()->name() << " poping down");
    XtPopdown(widget());
}


void
AnchorBox_Xm::clear()
{
    this->unhighlight();
    clear_container(items_);
    XmListDeleteAllItems(list_);
}


void
AnchorBox_Xm::highlight(
    int item)
{
    int n = 0;
    XtVaGetValues(list_, XmNvisibleItemCount, &n, NULL);
    if (item >= n) XmListSetBottomPos(list_, item + 1);
    XmListSelectPos(list_, item + 1, False);
    // XSync(XtDisplay(list_), False);
}


void
AnchorBox_Xm::unhighlight()
{
    XmListDeselectAllItems(list_);
    // XSync(XtDisplay(list_), False);
}


void
AnchorBox_Xm::set_title(
    string title)
{
    XtVaSetValues(widget(), XmNtitle, title.c_str(), NULL);
}


void
AnchorBox_Xm::list_action(
    Widget, XmListCallbackStruct* ls)
{
    obj()->activate(items_[ls->item_position - 1]);
}


void
AnchorBox_Xm::list_selection(
    Widget, XmListCallbackStruct* ls)
{
    obj()->select(items_[ls->item_position - 1]);
}


void
AnchorBox_Xm::shell_destroy(
    Widget, XtPointer)
{
    DBG("xm", obj()->name() << " shell destroyed callback");
    this->delayed_init();
    Natto::node* vp = prev_node_;
    prev_node_ = NULL;
    obj()->set_node(vp);
}


void
AnchorBox_Xm::focus_activate(
    Widget, XtPointer)
{
    int pos = ListGetSelectedPosFirst(list_);
    if (pos) obj()->focus(items_[pos - 1]);
}


void
AnchorBox_Xm::scan_activate(
    Widget, XtPointer)
{
    obj()->scan(items_);
}


void
AnchorBox_Xm::browse_activate(
    Widget, XtPointer)
{
    if (items_.empty()) return;

    int pos = ListGetSelectedPosFirst(list_);
    if (!pos) pos = 1;
    obj()->browse(items_[pos - 1]);
}


void
AnchorBox_Xm::tour_activate(
    Widget, XtPointer)
{
    if (items_.empty()) return;

    int pos = ListGetSelectedPosFirst(list_) + 1;
    if (!pos || pos > (int) items_.size()) pos = 1;
    this->unhighlight();
    this->highlight(--pos);
    obj()->browse(items_[pos]);
}


void
AnchorBox_Xm::close_activate(
    Widget, XtPointer)
{
    obj()->popdown();
}


void
AnchorBox_Xm::set_node(
    Natto::node* vp)
{
    if (prev_node_ == vp) return;
    this->clear();

    for (Natto::link_iter ei = vp->out_begin(); ei != vp->out_end(); ei++) {
	string item = (*ei)->label() + " (" + (*ei)->href() + ')';
	XmString xms = XmStringCreate(item.c_str(), XmSTRING_DEFAULT_CHARSET);
	if (!XmListItemExists(list_, xms)) {
	    XmListAddItem(list_, xms, 0);
	    items_.push_back(*ei);
	}
	XmStringFree(xms);
	prev_node_ = vp;
    }
}
