/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//                           Takahiro Souma <souma@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_GenericMapperBox_Xm_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "GenericMapperBox_Xm.h"

#include <math.h>
#include <ctype.h>
#include <string.h>
#include <iomanip.h>
#include <Xm/XmAll.h>
#include "utilx.h"
#include "MotifKit.h"
#include "Natto.h"
#include "Preference.h"


GenericMapperBox_Xm::GenericMapperBox_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj)
{}


GenericMapperBox*
GenericMapperBox_Xm::obj()
{
    return DCAST(GenericMapperBox*, GuiObject_imp::obj());
}


GuiObject_if*
GenericMapperBox_Xm::interface()
{
    return this;
}


void
GenericMapperBox_Xm::init(
    GuiConfig* config)
{
    if (config) config_ = *DCAST(MotifConfig*, config);
}


void
GenericMapperBox_Xm::delayed_init()
{
    MotifConfig conf;

    config_.add(XmNautoUnmanage, FALSE)
        .add(XmNdialogType,   XmDIALOG_TEMPLATE);
    GuiObject_Xm* parent = DCAST(GuiObject_Xm*, obj()->parent()->imp());
    this->widget(XmCreateMessageDialog(parent->widget(), obj()->name(),
                                       config_.args(), config_.size()));
    AddCallback(widget(), XmNdestroyCallback, shell_destroy);

    conf.clear();
    conf.add(XmNorientation, XmVERTICAL);
    Widget rowcol =
        XmCreateRowColumn(widget(), "rowcol", conf.args(), conf.size());
    XtManageChild(rowcol);

    Widget facerc[n_dimensions];
    for (int i = 0; i < n_dimensions; i++) {
	Widget caption = CreateCaption(
	    rowcol, obj()->mapper()->dimension_names[i], NULL, 0);
	XtManageChild(caption);
	facerc[i] = XmCreateRowColumn(caption, "dimrc", NULL, 0);
	XtVaSetValues(facerc[i], XmNorientation, XmHORIZONTAL, NULL);
	XtManageChild(facerc[i]);
	this->create_opmenu(facerc[i], i);
    }

    XtUnmanageChild(XmMessageBoxGetChild(widget(), XmDIALOG_HELP_BUTTON));
    AddCallback(widget(), XmNokCallback,     ok_activate);
    AddCallback(widget(), XmNcancelCallback, cancel_activate);
    Widget apply = XmCreatePushButton(widget(), "apply", NULL, 0);
    XtManageChild(apply);
    AddCallback(apply, XmNactivateCallback, apply_activate);
}


void
GenericMapperBox_Xm::popup(
    Natto* natto)
{
    if (!widget()) this->delayed_init();
    natto_ = natto;

    for (int i = 0; i < n_dimensions; i++) {
	XtVaSetValues(op_menu[i], XmNmenuHistory,
		      op_btn[i][obj()->working_mapping_[i]], NULL);
    }
    XtManageChild(widget());
}


void
GenericMapperBox_Xm::popdown()
{
    XtUnmanageChild(widget());
}


void
GenericMapperBox_Xm::create_opmenu(
    Widget parent,
    int    dim)
{
    Widget menurc = XmCreateRowColumn(parent, "menurc", NULL, 0);
    XtManageChild(menurc);
    Widget pulldown = XmCreatePulldownMenu(menurc, "pulldown", NULL, 0);
    
    for (int i = 0; i < n_parameters; i++) {
	op_btn[dim][i] = XmCreatePushButtonGadget(
	    pulldown, obj()->mapper()->parameter_names[i], NULL, 0);
	XtVaSetValues(op_btn[dim][i],
		      XmNuserData, (XtPointer)((dim << 8) | i), NULL);
	AddCallback(op_btn[dim][i], XmNactivateCallback, opmenu_activate);
	XtManageChild(op_btn[dim][i]);
    }

    op_menu[dim] = XmCreateOptionMenu(parent, "opmenu", NULL, 0);
    XtVaSetValues(op_menu[dim], XmNsubMenuId, pulldown, NULL);
    XtManageChild(op_menu[dim]);
}


void
GenericMapperBox_Xm::shell_destroy(
    Widget, XtPointer)
{
    this->widget(NULL);
    obj()->cancel(natto_);
}


void
GenericMapperBox_Xm::opmenu_activate(
    Widget w,
    XtPointer)
{
    int pack = 0;
    XtVaGetValues(w, XmNuserData, &pack, NULL);
    int dim   = pack >> 8;
    int param = pack & 0xff;
    obj()->working_mapping_[dim] = param;
}


void
GenericMapperBox_Xm::ok_activate(
    Widget, XtPointer)
{
    obj()->ok(natto_);
}


void
GenericMapperBox_Xm::cancel_activate(
    Widget, XtPointer)
{
    obj()->cancel(natto_);
}


void
GenericMapperBox_Xm::apply_activate(
    Widget, XtPointer)
{
    obj()->apply(natto_);
}
