/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_MainFrame_Xm_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "MainFrame_Xm.h"

#include <fstream.h>
#include <Xm/XmAll.h>
#include "util.h"
#include "utilx.h"
#include "SpringMapper.h"
#include "GenericMapper.h"
#include "Preference.h"
#include "PortKit.h"
#include "ToolBar_Xm.h"
#include "SiteBar_Xm.h"
#include "ViewPalette_Xm.h"
#include "StatusBar_Xm.h"
#include "AnchorBox_Xm.h"
#include "NattoView_Xm.h"


MainFrame_Xm::MainFrame_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj),
    cmap_(0)
{
    this->widget(toplevel);
}


void
MainFrame_Xm::init(
    GuiConfig* config)
{
    MotifConfig conf;
    if (config) conf = *DCAST(MotifConfig*, config);

    int dbuf[] = { GLEX_DBLBUF, None };
    int sbuf[] = { GLEX_SGLBUF, None };
    Display* dpy = XtDisplay(widget());

    bool is_dbuf = true;
    XVisualInfo* vi = glXChooseVisual(dpy, DefaultScreen(dpy), dbuf);
    if (!vi) {
	vi = glXChooseVisual(dpy, DefaultScreen(dpy), sbuf);
	if (!vi)
	    XtAppError(app_context(), "No RGB visual with depth buffer.");
	is_dbuf = false;
    }
    if (preference.install_cmap && vi->c_class != TrueColor) {
	cmap_ = XCreateColormap(dpy, RootWindow(dpy, vi->screen), 
			       vi->visual, AllocNone);
	XtVaSetValues(widget(),
		      XmNvisual,   vi->visual,
		      XmNdepth,    vi->depth,
		      XmNcolormap, cmap_, NULL);
    }
    else {
	cmap_ = GetShareableColormap(dpy, vi);
    }	

    // form /////////////////////////////////////
    obj()->form()->create(NULL);
    obj()->form()->manage();
    Widget form_widget =
	DCAST(GuiObject_Xm*, obj()->form()->imp())->widget();

    // menubar //////////////////////////////////

    MENU_BEGIN(menubar_, "menubar", form_widget);

    ITEM_SUB_BEGIN("file");
    ITEM_BTN("load", menubar_load);
    ITEM_BTN("save", menubar_save);
    ITEM_SEP("separator");
    ITEM_BTN("quit", menubar_quit);
    ITEM_SUB_END;

    ITEM_SUB_BEGIN("option");
    ITEM_BTN("import_icon",    menubar_import_icon);
    ITEM_BTN("mapper_generic", menubar_generic_mapper);
    ITEM_BTN("mapper_spring",  menubar_spring_mapper);
    ITEM_SUB_END;

    ITEM_SUB_BEGIN("window");
    ITEM_BTN("anchorbox", menubar_anchorbox);
    ITEM_BTN("tourbox",   menubar_tourbox);
    ITEM_BTN("mapping",   menubar_mapperbox);
    ITEM_SUB_END;

    ITEM_SUB_HELP("help");
    ITEM_BTN("about", menubar_about);
    ITEM_SUB_END;
    
    MENU_END;

    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_FORM)
        .add(XmNbottomAttachment, XmATTACH_NONE)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    XtSetValues(menubar_, conf.args(), conf.size());

    // toolbar //////////////////////////////////
    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_WIDGET)
        .add(XmNtopWidget,        menubar_)
        .add(XmNbottomAttachment, XmATTACH_NONE)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    obj()->toolbar()->create(&conf);
    obj()->toolbar()->manage();
    Widget toolbar_widget =
	DCAST(GuiObject_Xm*, obj()->toolbar()->imp())->widget();

    // sitebar //////////////////////////////////
    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_WIDGET)
        .add(XmNtopWidget,        toolbar_widget)
        .add(XmNbottomAttachment, XmATTACH_NONE)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    obj()->sitebar()->create(&conf);
    obj()->sitebar()->manage();
    Widget sitebar_widget =
	DCAST(GuiObject_Xm*, obj()->sitebar()->imp())->widget();

    // status bar ////////////////////////////////
    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_NONE)
        .add(XmNbottomAttachment, XmATTACH_FORM)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    obj()->statusbar()->create(&conf);
    obj()->statusbar()->manage();
    Widget statbar_widget =
	DCAST(GuiObject_Xm*, obj()->statusbar()->imp())->widget();

    // view palette //////////////////////////////
    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_WIDGET)
        .add(XmNtopWidget,        sitebar_widget)
        .add(XmNbottomAttachment, XmATTACH_WIDGET)
        .add(XmNbottomWidget,     statbar_widget)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_NONE);
    obj()->viewpalette()->create(&conf);
    obj()->viewpalette()->manage();
    Widget viewpalette_widget =
	DCAST(GuiObject_Xm*, obj()->viewpalette()->imp())->widget();

    // glframe ////////////////////////////////////
    conf.clear();
    conf.add(XmNfractionBase,     10000)
        .add(XmNtopAttachment,    XmATTACH_WIDGET)
        .add(XmNtopWidget,        sitebar_widget)
        .add(XmNbottomAttachment, XmATTACH_WIDGET)
        .add(XmNbottomWidget,     statbar_widget)
        .add(XmNleftAttachment,   XmATTACH_WIDGET)
        .add(XmNleftWidget,       viewpalette_widget)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    obj()->glframe()->create(&conf);
    obj()->glframe()->manage();
    
    // root view /////////////////////////////////
    obj()->rootview()->create(NULL);
    obj()->rootview()->manage();
    obj()->viewpalette()->view(obj()->rootview());

    //////////////////////////////////////////////
    savebox_ = XmCreateFileSelectionDialog(widget(), "savebox", NULL, 0);
    XtUnmanageChild(
	XmFileSelectionBoxGetChild(savebox_, XmDIALOG_HELP_BUTTON));
    AddCallback(savebox_, XmNcancelCallback, savebox_cancel);
    AddCallback(savebox_, XmNokCallback,     savebox_ok);

    loadbox_ = XmCreateFileSelectionDialog(widget(), "loadbox", NULL, 0);
    XtUnmanageChild(
	XmFileSelectionBoxGetChild(loadbox_, XmDIALOG_HELP_BUTTON));
    AddCallback(loadbox_, XmNcancelCallback, loadbox_cancel);
    AddCallback(loadbox_, XmNokCallback,     loadbox_ok);

    //////////////////////////////////////////////
    aboutbox_ = XmCreateMessageDialog(widget(), "aboutbox", NULL, 0);
    XtUnmanageChild(
	XmMessageBoxGetChild(aboutbox_, XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(
	XmMessageBoxGetChild(aboutbox_, XmDIALOG_CANCEL_BUTTON));
    XmString str = XmStringCreateLtoR(
	"The Natto View version " NATTOVIEW "\n"
	"Copyright (c) 1995-1998\n"
	"Hidekazu Shiozawa\n"
	"<shiozawa@myo.inst.keio.ac.jp>\n"
	"Bug reports are welcome!!",
	XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(aboutbox_, XmNmessageString, str, NULL);
    XmStringFree(str);

    conf.clear();
    obj()->anchorbox(new AnchorBox("anchorbox", obj(), obj()));
    obj()->anchorbox()->create(NULL);
}


MainFrame*
MainFrame_Xm::obj()
{
    return DCAST(MainFrame*, GuiObject_imp::obj());
}


GuiObject_if*
MainFrame_Xm::interface()
{
    return this;
}


void
MainFrame_Xm::set_title(
    string str)
{
    XtVaSetValues(widget(), XmNtitle, str.c_str(), NULL);
}


void
MainFrame_Xm::menubar_save(Widget, XtPointer)
{
    XtManageChild(savebox_);
}


void
MainFrame_Xm::menubar_load(Widget, XtPointer)
{
    XtManageChild(loadbox_);
}


void
MainFrame_Xm::menubar_quit(Widget, XtPointer)
{
    obj()->quit();
}


void
MainFrame_Xm::menubar_about(Widget, XtPointer)
{
    XtManageChild(aboutbox_);
}


void
MainFrame_Xm::menubar_spring_mapper(Widget, XtPointer)
{
    obj()->mapper_spring();
}


void
MainFrame_Xm::menubar_generic_mapper(Widget, XtPointer)
{
    obj()->mapper_generic();
}


void
MainFrame_Xm::menubar_anchorbox(Widget, XtPointer)
{
    obj()->popup_anchorbox();
}


void
MainFrame_Xm::menubar_tourbox(Widget, XtPointer)
{
    obj()->popup_tourbox();
}


void
MainFrame_Xm::menubar_mapperbox(Widget, XtPointer)
{
    obj()->popup_mapperbox();
}


void
MainFrame_Xm::menubar_import_icon(Widget, XtPointer)
{
    obj()->import_icon();
}


void
MainFrame_Xm::savebox_ok(
    Widget w,
    XmFileSelectionBoxCallbackStruct* fs)
{
    char* fname = NULL;
    XmStringGetLtoR(fs->value, XmSTRING_DEFAULT_CHARSET, &fname);
    obj()->save_file(fname);
    this->savebox_cancel(w, fs);
}


void
MainFrame_Xm::savebox_cancel(
    Widget, XmFileSelectionBoxCallbackStruct*)
{
    XtUnmanageChild(savebox_);
}


void
MainFrame_Xm::loadbox_ok(
    Widget w,
    XmFileSelectionBoxCallbackStruct* fs)
{
    char* fname = NULL;
    XmStringGetLtoR(fs->value, XmSTRING_DEFAULT_CHARSET, &fname);
    obj()->load_file(fname);
    this->loadbox_cancel(w, fs);
}


void
MainFrame_Xm::loadbox_cancel(
    Widget, XmFileSelectionBoxCallbackStruct*)
{
    XtUnmanageChild(loadbox_);
}


// Netscape Navigator Remote Control
#include <setjmp.h>
extern "C" {
int mozilla_remote_commands(Display *dpy, Window window,
			    const char* const* commands);
jmp_buf mozilla_env;
}


void
MainFrame_Xm::mozilla_open(
    URL url)
{
    const char* commands[2];
    
    if (!setjmp(mozilla_env)) {
	string cmd = "openURL(" + url.abs() + ')';
	commands[0] = cmd.c_str();
	commands[1] = NULL;
	::mozilla_remote_commands(XtDisplay(widget()), 0, commands);
	return;
    }
    else {
	// Mozzila is not present in the same screen.
	msgout << "Launching Netscape..." << endl;
	string cmd = "netscape " + url.abs() + " &";
	int ret = system(cmd.c_str());
	if (ret != -1 && ret != 127) return;
    }
    msgout << "ERROR: Netscape not found." << endl;
}
