/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_MotifKit_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "MotifKit.h"

#include <Xm/XmAll.h>
#include <algorithm>
#include <functional>
#include "utilstl.h"
#include "MainFrame_Xm.h"
#include "NattoView_Xm.h"
#include "ToolBar_Xm.h"
#include "SiteBar_Xm.h"
#include "ViewPalette_Xm.h"
#include "StatusBar_Xm.h"
#include "AnchorBox_Xm.h"
#include "TourBox_Xm.h"
#include "GenericMapperBox_Xm.h"


MotifKit::MotifKit(
    XtAppContext app):
    app_(app)
{
#if 0
    XtToolkitInitialize();
    XtSetLanguageProc(NULL, NULL, NULL);
    toplevel = XtAppInitialize(&app_context, "Nattoview", NULL, 0,
			       &argc, argv, NULL, NULL, 0);
    XtAddCallback(toplevel, XmNdestroyCallback, xt_toplevel_destroy, NULL);

    // debugging X (when not static linked)
    if (preference.debug_X) {
	XSynchronize(XtDisplay(toplevel), True);
	XSetErrorHandler(x_error);
    }
#endif
}


GuiObject_imp*
MotifKit::create_LayoutForm(
    GuiObject* obj)
{
    return new LayoutForm_Xm(DCAST(LayoutForm*, obj));
}


GuiObject_imp*
MotifKit::create_MainFrame(
    GuiObject* obj)
{
    return new MainFrame_Xm(DCAST(MainFrame*, obj));
}


GuiObject_imp*
MotifKit::create_NattoView(
    GuiObject* obj)
{
    return new NattoView_Xm(DCAST(NattoView*, obj));
}


GuiObject_imp*
MotifKit::create_ToolBar(
    GuiObject* obj)
{
    return new ToolBar_Xm(DCAST(ToolBar*, obj));
}


GuiObject_imp*
MotifKit::create_SiteBar(
    GuiObject* obj)
{
    return new SiteBar_Xm(DCAST(SiteBar*, obj));
}


GuiObject_imp*
MotifKit::create_ViewPalette(
    GuiObject* obj)
{
    return new ViewPalette_Xm(DCAST(ViewPalette*, obj));
}


GuiObject_imp*
MotifKit::create_StatusBar(
    GuiObject* obj)
{
    return new StatusBar_Xm(DCAST(StatusBar*, obj));
}


GuiObject_imp*
MotifKit::create_AnchorBox(
    GuiObject* obj)
{
    return new AnchorBox_Xm(DCAST(AnchorBox*, obj));
}


GuiObject_imp*
MotifKit::create_TourBox(
    GuiObject* obj)
{
    return new TourBox_Xm(DCAST(TourBox*, obj));
}


GuiObject_imp*
MotifKit::create_GenericMapperBox(
    GuiObject* obj)
{
    return new GenericMapperBox_Xm(DCAST(GenericMapperBox*, obj));
}



void
MotifConfig::clear()
{
    clear_container(args_);
}


MotifConfig&
MotifConfig::add(
    const char* name,
    XtArgVal    value)
{
    Arg a;
    XtSetArg(a, name, value);
    args_.push_back(a);
    return *this;
}


GuiObject_Xm::GuiObject_Xm(
    GuiObject* obj):
    GuiObject_imp(obj), widget_(NULL)
{}


GuiObject_Xm::~GuiObject_Xm()
{
    if (widget_) XtDestroyWidget(widget_);
}


PortKit*
GuiObject_Xm::kit()
{
    return ::portkit;
}


Widget
GuiObject_Xm::widget()
{
    return widget_;
}


void
GuiObject_Xm::widget(
    Widget widget)
{
    widget_ = widget;
}


void
GuiObject_Xm::manage()
{
    XtManageChild(widget_);
}


void
GuiObject_Xm::show()
{
    if (!XtIsManaged(widget_)) XtManageChild(widget_);
    XtRealizeWidget(widget_);
}



LayoutForm_Xm::LayoutForm_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj)
{}


void
LayoutForm_Xm::init(
    GuiConfig* config)
{
    MotifConfig conf;
    if (config) conf = *DCAST(MotifConfig*, config);

    GuiObject_Xm* parent = DCAST(GuiObject_Xm*, obj()->parent()->imp());
    this->widget(
	XmCreateForm(parent->widget(), obj()->name(),
                     conf.args(), conf.size()));
}


LayoutForm*
LayoutForm_Xm::obj()
{
    return DCAST(LayoutForm*, GuiObject_imp::obj());
}


GuiObject_if*
LayoutForm_Xm::interface()
{
    return this;
}




WidgetWrapper::WidgetWrapper(
    Widget widget):
    GuiObject("wrapper", NULL), widget_(widget)
{}


void
WidgetWrapper::create(
    GuiConfig*)
{
    GuiObject::create(new WidgetWrapper_Xm(this));
    DCAST(GuiObject_Xm*, imp())->widget(widget_);
}


WidgetWrapper_Xm::WidgetWrapper_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj)
{}


GuiObject_if*
WidgetWrapper_Xm::interface()
{
    return this;
}
