/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef MOTIFKIT_H
#define MOTIFKIT_H
#ifdef __GNUG__
#pragma interface
#endif

#include <vector>
#include "xtinter.h"
#include "PortKit.h"
//#include "ptrvector.h"


class MotifKit: public PortKit
{
 public:
    explicit MotifKit(XtAppContext);

 protected:
    virtual GuiObject_imp*    create_LayoutForm(GuiObject*);
    virtual GuiObject_imp*    create_MainFrame(GuiObject*);
    virtual GuiObject_imp*    create_NattoView(GuiObject*);
    virtual GuiObject_imp*    create_ToolBar(GuiObject*);
    virtual GuiObject_imp*    create_SiteBar(GuiObject*);
    virtual GuiObject_imp*    create_ViewPalette(GuiObject*);
    virtual GuiObject_imp*    create_StatusBar(GuiObject*);
    virtual GuiObject_imp*    create_AnchorBox(GuiObject*);
    virtual GuiObject_imp*    create_TourBox(GuiObject*);
    virtual GuiObject_imp*    create_GenericMapperBox(GuiObject*);

 private:
    XtAppContext app_;
    DBG_NEW
};



class MotifConfig: public GuiConfig
{
 public:
    void clear();

    const Arg* args() const { return args_.begin(); }
    Arg*       args()       { return args_.begin(); }
    Cardinal   size() const { return args_.size(); }

    MotifConfig& add(const char*, XtArgVal);
    MotifConfig& add(const char* n, Widget v) { return add(n, (XtArgVal) v); }
    MotifConfig& add(const char* n, char*  v) { return add(n, (XtArgVal) v); }
    MotifConfig& add(const char* n, void*  v) { return add(n, (XtArgVal) v); }
    MotifConfig& add(const char* n, int    v) { return add(n, (XtArgVal) v); }
    MotifConfig& add(const char* n, unsigned long v) {
        return add(n, (XtArgVal) v); }
    MotifConfig& add(const char* n, XVisualInfo* v) {
        return add(n, (XtArgVal) v); }
    
 private:
    vector<Arg> args_;
    DBG_NEW
};

inline void
SetValues(Widget w, MotifConfig& c) { XtSetValues(w, c.args(), c.size()); }



class GuiObject_Xm: public GuiObject_imp
{
 public:
    explicit GuiObject_Xm(GuiObject*);
    virtual ~GuiObject_Xm();

    Widget   widget();
    void     widget(Widget);
    
    virtual PortKit* kit();
    virtual void     manage();
    virtual void     show();

 protected:
    Widget widget_;

 private:
    GuiObject_Xm(const GuiObject_Xm&);
    GuiObject_Xm& operator=(const GuiObject_Xm&);
    DBG_NEW
};



class LayoutForm_Xm: public GuiObject_Xm, public LayoutForm_if
{
    typedef LayoutForm_Xm self_type;

 public:
    explicit LayoutForm_Xm(GuiObject*);
    virtual void init(GuiConfig*);

    virtual LayoutForm*   obj();
    virtual GuiObject_if* interface();
    DBG_NEW
};



class WidgetWrapper_Xm: public GuiObject_Xm, public GuiObject_if
{
 public:
    explicit WidgetWrapper_Xm(GuiObject*);
    GuiObject_if* interface();
    DBG_NEW
};


class WidgetWrapper: public GuiObject
{
 public:
    WidgetWrapper(Widget);

    virtual void create(GuiConfig*);

 private:
    Widget widget_;

    WidgetWrapper(const WidgetWrapper&);
    WidgetWrapper& operator=(const WidgetWrapper&);
    DBG_NEW
};


#endif /* MOTIFKIT_H */
