/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define NO_LINK_MENU

#ifdef __GNUG__
#pragma implementation
int dummy_NattoView_Xm_cc = 0;
#endif

#include <Xm/XmAll.h>
#include <Xm/XmP.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "NattoView_Xm.h"

#include <setjmp.h>
#include <strstream.h>
#include <X11/keysym.h>
#include <Xm/XmAll.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include "util.h"
#include "utilstl.h"
#include "utilx.h"
#include "MotifKit.h"
#include "TourBox.h"
#include "Mapper.h"
#include "MainFrame_Xm.h"


NattoView_Xm::NattoView_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj),
    glxcx_(NULL), vi_(NULL),
    cursor_menu_(NULL), pick_menu_(NULL), view_menu_(NULL),
    font_set_(NULL),
    x_drag_start_(0), y_drag_start_(0), x_drag_end_(0), y_drag_end_(0),
    pending_(false)
{}


void
NattoView_Xm::init(
    GuiConfig* config)
{
    MotifConfig conf;
    if (config) conf = *DCAST(MotifConfig*, config);
    
    GuiObject_Xm* parent = DCAST(GuiObject_Xm*, obj()->parent()->imp());

    int w = 0, h = 0;
    XtVaGetValues(parent->widget(), XmNwidth, &w, XmNheight, &h, NULL);
    w -= 20;
    h -= 20;

    conf.clear();
    conf.add(XmNorientation,         XmHORIZONTAL)
        //.add(XmNprocessingDirection, XmMAX_ON_RIGHT)
        .add(XmNprocessingDirection, XmMAX_ON_LEFT)
        .add(XmNminimum,             -100)
        .add(XmNmaximum,              100)
        .add(XmNvalue,               (int) (obj()->trans().x * 100))
	.add(XmNbottomAttachment,    XmATTACH_FORM)
        .add(XmNtopAttachment,       XmATTACH_NONE)
        .add(XmNleftAttachment,      XmATTACH_FORM)
        //.add(XmNrightAttachment,     XmATTACH_SELF)
        .add(XmNwidth,               min(w / 2, 150));
    htransscale_ = XmCreateScale(parent->widget(), "htransscale",
                                 conf.args(), conf.size());
    XtManageChild(htransscale_);
    AddCallback(htransscale_, XmNvalueChangedCallback, htransscale_changed);
    AddCallback(htransscale_, XmNdragCallback,         htransscale_changed);

    conf.clear();
    conf.add(XmNorientation,         XmHORIZONTAL)
        .add(XmNprocessingDirection, XmMAX_ON_RIGHT)
        .add(XmNminimum,             -180)
        .add(XmNmaximum,              180)
        .add(XmNvalue,               (int) obj()->rot().z)
        .add(XmNbottomAttachment,    XmATTACH_FORM)
        .add(XmNtopAttachment,       XmATTACH_NONE)
        .add(XmNleftAttachment,      XmATTACH_WIDGET)
        .add(XmNleftWidget,          htransscale_)
        .add(XmNrightAttachment,     XmATTACH_FORM);
    hrotscale_ = XmCreateScale(parent->widget(), "hrotscale",
                               conf.args(), conf.size());
    XtManageChild(hrotscale_);
    AddCallback(hrotscale_, XmNvalueChangedCallback, hrotscale_changed);
    AddCallback(hrotscale_, XmNdragCallback,         hrotscale_changed);

    conf.clear();
    conf.add(XmNorientation,         XmVERTICAL)
        //.add(XmNprocessingDirection, XmMAX_ON_TOP)
        .add(XmNprocessingDirection, XmMAX_ON_BOTTOM)
        .add(XmNminimum,             -100)
        .add(XmNmaximum,              100)
        .add(XmNvalue,               (int) (obj()->trans().y * 100))
        //.add(XmNbottomAttachment,    XmATTACH_SELF)
        .add(XmNheight,              min(h / 2, 150))
        .add(XmNtopAttachment,       XmATTACH_FORM)
        .add(XmNleftAttachment,      XmATTACH_NONE)
        .add(XmNrightAttachment,     XmATTACH_FORM);
    vtransscale_ = XmCreateScale(parent->widget(), "vtransscale",
                                 conf.args(), conf.size());
    XtManageChild(vtransscale_);
    AddCallback(vtransscale_, XmNvalueChangedCallback, vtransscale_changed);
    AddCallback(vtransscale_, XmNdragCallback,         vtransscale_changed);

    conf.clear();
    conf.add(XmNorientation,         XmVERTICAL)
        .add(XmNprocessingDirection, XmMAX_ON_BOTTOM)
        .add(XmNminimum,             -180)
        .add(XmNmaximum,                0)
        .add(XmNvalue,               (int) obj()->rot().x)
        .add(XmNbottomAttachment,    XmATTACH_FORM)
        .add(XmNtopAttachment,       XmATTACH_WIDGET)
        .add(XmNtopWidget,           vtransscale_)
        .add(XmNleftAttachment,      XmATTACH_NONE)
        .add(XmNrightAttachment,     XmATTACH_FORM);
    vrotscale_ = XmCreateScale(parent->widget(), "vrotscale",
                               conf.args(), conf.size());
    XtManageChild(vrotscale_);
    AddCallback(vrotscale_, XmNvalueChangedCallback, vrotscale_changed);
    AddCallback(vrotscale_, XmNdragCallback,         vrotscale_changed);

    int scale_width = 0;
    XtVaGetValues(vtransscale_, XmNwidth, &scale_width, NULL);
    conf.clear();
    conf.add(XmNrightOffset, scale_width);
    XtSetValues(hrotscale_, conf.args(), conf.size());

    XtVaGetValues(htransscale_, XmNheight, &scale_width, NULL);
    conf.clear();
    conf.add(XmNbottomOffset, scale_width);
    XtSetValues(vrotscale_, conf.args(), conf.size());

    if (!init_aux("glxarea")) return;
    conf.clear();
    conf.add(XmNbottomAttachment, XmATTACH_WIDGET)
        .add(XmNbottomWidget,     hrotscale_)
        .add(XmNtopAttachment,    XmATTACH_FORM)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_WIDGET)
        .add(XmNrightWidget,      vrotscale_);
    XtSetValues(widget(), conf.args(), conf.size());

    if (!obj()->rootview()) {
        AddCallback(widget(), GLwNginitCallback, ginit);
    }
    else {
        NattoView_Xm* rootxm = DCAST(NattoView_Xm*, obj()->rootview()->imp());
        XtVaGetValues(widget(), GLwNvisualInfo, &vi_, NULL);
        glxcx_ = glXCreateContext(
            XtDisplay(widget()), vi_, rootxm->glxcx_, GL_TRUE);
        if (!glxcx_)
            XtAppError(this->app_context(), "Can't create rendering context.");
        obj()->init_graphics();
    }
}


bool
NattoView_Xm::init_aux(
    const char* name)
{
    int dbuf[] = { GLEX_DBLBUF, None };
    int sbuf[] = { GLEX_SGLBUF, None };
    GuiObject_Xm* parent  = DCAST(GuiObject_Xm*, obj()->parent()->imp());
    MainFrame_Xm* framexm = DCAST(MainFrame_Xm*, obj()->frame()->imp());
    Display* dpy = XtDisplay(parent->widget());

    if (obj()->double_buffer())
	vi_ = glXChooseVisual(dpy, DefaultScreen(dpy), dbuf);
    if (!obj()->double_buffer() || !vi_) {
	vi_ = glXChooseVisual(dpy, DefaultScreen(dpy), sbuf);
	if (!vi_) return false;
	obj()->double_buffer(false);
    }

    MotifConfig conf;
    conf.add(GLwNvisualInfo,      vi_)
	.add(GLwNdoublebuffer,    obj()->double_buffer() ? True: False)
	.add(GLwNinstallColormap, obj()->install_cmap()  ? True: False)
	.add(XmNcolormap,         framexm->colormap())
	.add(XmNtraversalOn,      True)
	//.add(XmNsensitive,        True)
	//.add(XmNnavigationType,   XmTAB_GROUP)
	.add(XmNuserData,       (void*) this);
    this->widget(XtCreateManagedWidget(
        name, glwMDrawingAreaWidgetClass, parent->widget(),
        conf.args(), conf.size()));
    AddCallback(widget(), GLwNexposeCallback, expose);
    AddCallback(widget(), GLwNresizeCallback, resize);
    AddCallback(widget(), GLwNinputCallback,  input);

    char** missing_chars = NULL;
    int    missing_count = 0;
    char*  def_string    = NULL;
    font_set_ = XCreateFontSet(
	XtDisplay(widget()), // "-*-*-medium-r-normal--14-*-*-*-*-*-*-*"
	"-adobe-times-medium-r-normal--12-120-75-75-p-64-iso8859-1"
#ifdef JAPANESE
	",-misc-fixed-medium-r-normal--14-130-75-75-c-140-jisx0208.1983-0"
	",-misc-fixed-medium-r-normal--14-130-75-75-c-70-jisx0201.1976-0"
#endif
	, &missing_chars, &missing_count, &def_string);

    POPUP_BEGIN(view_menu_, "topMenu", parent->widget());
#if 1
    ITEM_LBL("title");
    ITEM_SEP("separator");
#endif

    ITEM_BTN("tourbox",    viewmenu_tourbox);
    ITEM_BTN("set_origin", viewmenu_set_origin);

    ITEM_SUB_BEGIN("view");
    ITEM_BTN("home",  viewmenu_home_view);
    ITEM_BTN("top",   viewmenu_top_view);
    ITEM_BTN("focus", viewmenu_focus_view);
    ITEM_SUB_END;

    ITEM_SUB_BEGIN("render");
    ITEM_BTN("wireframe", viewmenu_wireframe);
    ITEM_BTN("surface",   viewmenu_surface);
    ITEM_BTN("no_label",  viewmenu_no_label);
    ITEM_SUB_END;

    ITEM_SEP("separator1");
#if 0
    ITEM_SUB_BEGIN("trans");
    ITEM_BTN("left",    trans_left);
    ITEM_BTN("right",   trans_right);
    ITEM_BTN("up",      trans_up);
    ITEM_BTN("down",    trans_down);
    ITEM_SUB_END;

    ITEM_SUB_BEGIN("rotate");
    ITEM_BTN("left",  rot_left);
    ITEM_BTN("right", rot_right);
    ITEM_BTN("up",    rot_up);
    ITEM_BTN("down",  rot_down);
    ITEM_SUB_END;
    
    ITEM_SUB_BEGIN("zoom");
    ITEM_BTN("zoomIn",  zoom_in);
    ITEM_BTN("zoomOut", zoom_out);
    ITEM_SUB_END;
    ITEM_SEP("separator2");
#endif
    ITEM_BTN("close", viewmenu_close);
    POPUP_END;

    conf.clear();
    obj()->tourbox(new TourBox("tourbox", obj()->parent(), obj()));
    obj()->tourbox()->create(&conf);

    return true;
}


NattoView_Xm::~NattoView_Xm()
{
    Display* dpy = XtDisplay(widget());
    glXDestroyContext(XtDisplay(widget()), glxcx_);
    XFreeFontSet(dpy, font_set_);
    XtDestroyWidget(widget());
    XtDestroyWidget(htransscale_);
    XtDestroyWidget(vtransscale_);
    XtDestroyWidget(hrotscale_);
    XtDestroyWidget(vrotscale_);
}


NattoView*
NattoView_Xm::obj()
{
    return DCAST(NattoView*, GuiObject_imp::obj());
}


GuiObject_if*
NattoView_Xm::interface()
{
    return this;
}


void
NattoView_Xm::adjust_sliders()
{
    ScaleSetValue(hrotscale_,   (int) obj()->rot().z);
    ScaleSetValue(vrotscale_,   (int) obj()->rot().x);
    ScaleSetValue(htransscale_, (int)(obj()->trans().x * 100));
    ScaleSetValue(vtransscale_, (int)(obj()->trans().y * 100));
    glXWaitGL();
}


void
NattoView_Xm::ginit(
    Widget,
    GLwDrawingAreaCallbackStruct* cs)
{
    XtVaGetValues(widget(), GLwNvisualInfo, &vi_, NULL);
    glxcx_ = glXCreateContext(XtDisplay(widget()), vi_, None, GL_TRUE);
    if (!glxcx_)
	XtAppError(this->app_context(), "Can't create rendering context.");
    obj()->init_graphics();
    obj()->resize(0, 0, cs->width, cs->height);
}


void
NattoView_Xm::swap_buffers()
{
    GLwDrawingAreaSwapBuffers(widget());
}


void 
NattoView_Xm::resize(
    Widget,
    GLwDrawingAreaCallbackStruct* cs)
{
    if (!glxcx_) return;

    int old_width  = obj()->width();
    int old_height = obj()->height(); 
    if (!obj()->rootview()) {
	// root view
	obj()->resize(0, 0, cs->width, cs->height);
    }
    else { 
	// subview
	Position x = 0, y = 0;
	XtVaGetValues(widget(), XmNx, &x, XmNy, &y, NULL);
	y = obj()->rootview()->height() - (y + cs->height);
	obj()->resize(x, max(y, (Position) 0), cs->width, cs->height);
    }
    if (cs->width < old_width && cs->height < old_height) {
	// In this case, expose() is not called.
	this->post_redisplay();
    }
    MotifConfig conf;
    conf.clear();
    conf.add(XmNwidth, min(cs->width / 2, 150));
    XtSetValues(htransscale_, conf.args(), conf.size());
    conf.clear();
    conf.add(XmNheight, min(cs->height / 2, 150));
    XtSetValues(vtransscale_, conf.args(), conf.size());
}


void 
NattoView_Xm::expose(
    Widget,
    GLwDrawingAreaCallbackStruct*)
{
    this->post_redisplay();
}


void
NattoView_Xm::post_redisplay()
{
    if (pending_) return;
    pending_ = true;
    AddWorkProc(XtWidgetToApplicationContext(widget()), redisplay);
}


Boolean
NattoView_Xm::redisplay()
{
    obj()->update();
    pending_ = false;
    return True;
}


void*
NattoView_Xm::string_bitmap(
    string str,
    int&   width,
    int&   height,
    int&   base_line)
{
    XImage* image =
	CreateXImageString(XtDisplay(widget()), XtWindow(widget()), font_set_,
			   str.c_str(), width, height, base_line);
    void* bitmap = XImageToGLBitmap(image, width, height);
    XDestroyImage(image);
    return bitmap;
}
    

void
NattoView_Xm::htransscale_changed(
    Widget,
    XmScaleCallbackStruct* cs)
{
    obj()->trans_left_right_abs(cs->value / 100.0);
    this->post_redisplay();
}


void
NattoView_Xm::vtransscale_changed(
    Widget,
    XmScaleCallbackStruct* cs)
{
    obj()->trans_up_down_abs(cs->value / 100.0);
    this->post_redisplay();
}


void
NattoView_Xm::hrotscale_changed(
    Widget,
    XmScaleCallbackStruct* cs)
{
    obj()->rot_left_right_abs(cs->value);
    this->post_redisplay();
}


void
NattoView_Xm::vrotscale_changed(
    Widget,
    XmScaleCallbackStruct* cs)
{
    obj()->rot_up_down_abs(cs->value);
    this->post_redisplay();
}


void
NattoView_Xm::input(
    Widget,
    GLwDrawingAreaCallbackStruct* cs)
{
    switch (cs->event->type) {
     case ButtonPress:   button_press((XButtonEvent*) cs->event);   break;
     case KeyRelease:    key_release((XKeyEvent*) cs->event);       break;
     case ButtonRelease: button_release((XButtonEvent*) cs->event); break;
     case MotionNotify:  motion_notify((XMotionEvent*) cs->event);  break;
    }
    return;
}


void
NattoView_Xm::button_press(
    XButtonEvent* event)
{
    if (pick_menu_) DestroyPopupMenu2(pick_menu_);
    pick_menu_ = NULL;
    clear_container(picked_nodes_);
    clear_container(picked_links_);
    
    switch (event->button) {
     case Button1:
	x_drag_start_ = x_drag_end_ = event->x;
	y_drag_start_ = y_drag_end_ = event->y;
	this->drag_arrow();
	break;
	
     case Button2:
	x_drag_start_ = x_drag_end_ = event->x;
	y_drag_start_ = y_drag_end_ = event->y;
	this->rubberband();
	break;

     default:
	XmMenuPosition(view_menu_, event);
	XtManageChild(view_menu_);
	break;
    }
}


void
NattoView_Xm::button_release(
    XButtonEvent* event)
{
    int w, h;
    // size_t n;
    vector<int> sysobjs;
    int x = x_drag_start_;
    int y = obj()->height() - y_drag_start_;

    switch (event->button) {
     case Button1:
	this->drag_arrow();
	
	// The viewport coordinate is same as the window.
	obj()->pick(x, y, sysobjs, picked_nodes_, picked_links_);

	if (picked_nodes_.empty() && picked_links_.empty()) break;
	pick_menu_ = CreatePopupMenu2(XtParent(widget()), "pickMenu", NULL, 0);

	if (picked_nodes_.size()) {
	    XtManageChild(
		XmCreateLabelGadget(pick_menu_, "nodeTitle", NULL, 0));
	    XtManageChild(
		XmCreateSeparatorGadget(pick_menu_, "separator1", NULL, 0));
	    this->create_node_menu(pick_menu_, picked_nodes_);
	}
#ifndef NO_LINK_MENU
	if (picked_links_.size()) {
	    XtManageChild(
		XmCreateSeparatorGadget(pick_menu_, "separator2", NULL,0));
	    XtManageChild(
		XmCreateLabelGadget(pick_menu_, "linkTitle", NULL, 0));
	    XtManageChild(
		XmCreateSeparatorGadget(pick_menu_, "separator3", NULL,0));
	    this->create_link_menu(pick_menu_, picked_links_);
	}
#endif /* !NO_LINK_MENU */
	// XAllowEvents(XtDisplay(widget()), SyncPointer, CurrentTime);
	// XtVaSetValues(pick_menu_, XmNwhichButton, Button1, NULL);
	
	if (picked_nodes_.size() == 1) {
            obj()->frame()->focus_node(picked_nodes_[0]);
	    if (y_drag_end_ - y_drag_start_ < -10)
		obj()->frame()->lift_up(picked_nodes_[0]);
	    else if (y_drag_end_ - y_drag_start_ > +10)
		obj()->frame()->lift_down(picked_nodes_[0]);
            obj()->frame()->natto()->sync();
	    break;
	}
#ifndef NO_LINK_MENU
	if (picked_nodes_.empty() && picked_links_.size() == 1) {
	    frame()->focus(picked_links_[0]->to());
            obj()->frame()->natto()->sync();
	    break;
	}
#endif /* !NO_LINK_MENU */

	ManagePopupMenu2(pick_menu_, event->x_root, event->y_root);
	break;

     case Button2:
	this->rubberband();
	w = abs(x_drag_end_ - x_drag_start_);
	h = abs(y_drag_end_ - y_drag_start_);
	if (w < 100) w = 100;
	if (h < 100) h = 100;
	{
	    static int n = 0;
	    strstream name;
	    name << "subview" << n++ << ends;
	    obj()->open_subview(
	        name.str(),
		obj()->xpos() + min(x_drag_start_, x_drag_end_),
		obj()->ypos() - max(y_drag_start_, y_drag_end_)
		+ obj()->height(), w, h);
	    name.freeze(false);
	}
	break;
     default:
	break;
    }
}


void
NattoView_Xm::motion_notify(
    XMotionEvent* event)
{
    if (event->state & Button2Mask) {
	this->rubberband();
	x_drag_end_ = event->x;
	y_drag_end_ = event->y;
	this->rubberband();
    }
    else if (event->state & Button1Mask) {
	this->drag_arrow();
	x_drag_end_ = event->x;
	y_drag_end_ = event->y;
	this->drag_arrow();
    }
}


void
NattoView_Xm::create_node_menu(
    Widget                menu,
    vector<Natto::node*>& nodes)
{
    int j = 0;
    for (vector<Natto::node*>::iterator vi = nodes.begin();
	 vi != nodes.end() && j < 10; vi++, j++) {
	Natto::node* vp = *vi;

	URL url = vp->url();
	Widget btn =
            XmCreatePushButtonGadget(menu, url.abs().c_str(), NULL, 0);
	XtVaSetValues(btn, XmNshadowThickness, 0, NULL);
	string label = vp->label() + " (" + url.abs() +')';
	label += ')';
	XmString xms = XmStringCreate(label.c_str(), XmSTRING_DEFAULT_CHARSET);
	
	XtVaSetValues(btn,
		      XmNlabelString, xms,
		      XmNuserData,    vp, NULL);
	XmStringFree(xms);
	AddCallback(btn, XmNactivateCallback, node_menu_item);
	XtManageChild(btn);
    }
}


void
NattoView_Xm::create_link_menu(
    Widget                menu,
    vector<Natto::link*>& links)
{
    int j = 0;
    for (vector<Natto::link*>::iterator ei = links.begin();
	 ei != links.end() && j < 10; ei++, j++) {
	Natto::link* ep = *ei;
	URL url1 = ep->from()->url();
	URL url2 = ep->to()->url();
	string name = url1.abs() + url2.abs();
	
	Widget btn = XmCreatePushButtonGadget(menu, name.c_str(), NULL, 0);
	XtVaSetValues(btn, XmNshadowThickness, 0, NULL);
	string label = ep->label() + " (" + ep->href() + ')';
	XmString xms = XmStringCreate(label.c_str(), XmSTRING_DEFAULT_CHARSET);

	XtVaSetValues(btn,
		      XmNlabelString, xms,
		      XmNuserData,    ep, NULL);
	XmStringFree(xms);
	AddCallback(btn, XmNactivateCallback, link_menu_item);
	XtManageChild(btn);
    }
}


void 
NattoView_Xm::node_menu_item(
    Widget btn,
    XtPointer)
{
    if (pick_menu_) DestroyPopupMenu2(pick_menu_);
    pick_menu_ = NULL;

    Natto::node* vp;
    XtVaGetValues(btn, XmNuserData, &vp, NULL);
    obj()->frame()->focus_node(vp);
    if      (y_drag_end_ - y_drag_start_ < -10) obj()->frame()->lift_up();
    else if (y_drag_end_ - y_drag_start_ > +10) obj()->frame()->lift_down();
}


void 
NattoView_Xm::link_menu_item(
    Widget btn,
    XtPointer)
{
    if (pick_menu_) DestroyPopupMenu2(pick_menu_);
    pick_menu_ = NULL;

    Natto::link* ep;
    XtVaGetValues(btn, XmNuserData, &ep, NULL);
    obj()->frame()->focus_node(ep->to());
}


void
NattoView_Xm::key_release(
    XKeyEvent* event)
{
    char   buf[4];
    KeySym keysym;
    XLookupString(event, buf, 1, &keysym, NULL);
    
    switch (keysym) {
     case XK_Escape:
     case XK_Q:  ExitNattoView();           break;  // -> accelerator

     case XK_C:  obj()->close();            break;  // -> accelerator
     case XK_O:  obj()->set_origin();       break;  // -> accelerator
     case XK_N:  obj()->tourbox();          break;  // -> accelerator

     case XK_F:  obj()->render_draw_icon(); break;  // -> accelerator
     case XK_H:  obj()->home_view();        break;  // -> accelerator
     case XK_T:  obj()->top_view();         break;  // -> accelerator
     case XK_W:  obj()->render_wireframe(); break;  // -> accelerator
     case XK_S:  obj()->render_surface();   break;  // -> accelerator
     case XK_l:  obj()->render_no_label();  break;  // -> accelerator

     case XK_U:  obj()->lift_up();          break;  // -> accelerator
     case XK_D:  obj()->lift_down();        break;  // -> accelerator
     case XK_A:  obj()->lift_down_all();    break;  // -> accelerator
     case XK_B:  obj()->browse();           break;  // -> accelerator
     case XK_L:  obj()->reload();           break;  // -> accelerator

     case XK_w:  obj()->trans_up();         break;  // -> accelerator
     case XK_z:  obj()->trans_down();       break;  // -> accelerator
     case XK_a:  obj()->trans_left();       break;  // -> accelerator
     case XK_s:  obj()->trans_right();      break;  // -> accelerator

     case XK_r:  obj()->rot_up();           break;  // -> accelerator
     case XK_c:  obj()->rot_down();         break;  // -> accelerator
     case XK_d:  obj()->rot_left();         break;  // -> accelerator
     case XK_f:  obj()->rot_right();        break;  // -> accelerator

     case XK_x:  obj()->zoom_in();          break;  // -> accelerator
     case XK_e:  obj()->zoom_out();         break;  // -> accelerator
    }
    this->post_redisplay();
}


void
NattoView_Xm::rubberband()
{
    Display* dpy = XtDisplay(widget());
    GC gc = XtGetGC(widget(), 0, NULL);
    XSetFunction(dpy, gc, GXxor);
    XSetForeground(dpy, gc, ~0U);
    XSetLineAttributes(dpy, gc, 5, LineSolid, CapProjecting, JoinMiter);
    XDrawRectangle(dpy, XtWindow(widget()), gc,
		   min(x_drag_start_, x_drag_end_),
		   min(y_drag_start_, y_drag_end_),
		   abs(x_drag_end_ - x_drag_start_),
		   abs(y_drag_end_ - y_drag_start_));
    XtReleaseGC(widget(), gc);
}


void
NattoView_Xm::drag_arrow()
{
    Display* dpy = XtDisplay(widget());
    GC gc = XtGetGC(widget(), 0, NULL);
    XSetFunction(dpy, gc, GXxor);
    XSetForeground(dpy, gc, ~0U);
    XSetLineAttributes(dpy, gc, 5, LineSolid, CapRound, JoinMiter);
    XDrawLine(dpy, XtWindow(widget()), gc,
	      x_drag_start_, y_drag_start_, x_drag_start_, y_drag_end_);
    if (y_drag_end_ <= y_drag_start_) {
	XDrawLine(dpy, XtWindow(widget()), gc,
		  x_drag_start_ - 10, y_drag_end_ + 10,
		  x_drag_start_,      y_drag_end_);
	XDrawLine(dpy, XtWindow(widget()), gc,
		  x_drag_start_ + 10, y_drag_end_ + 10,
		  x_drag_start_,      y_drag_end_);
    }
    else {
	XDrawLine(dpy, XtWindow(widget()), gc,
		  x_drag_start_ - 10, y_drag_end_ - 10,
		  x_drag_start_,      y_drag_end_);
	XDrawLine(dpy, XtWindow(widget()), gc,
		  x_drag_start_ + 10, y_drag_end_ - 10,
		  x_drag_start_,      y_drag_end_);
    }
    XtReleaseGC(widget(), gc);
}


void
NattoView_Xm::viewmenu_tourbox(Widget, XtPointer)
{
    obj()->tourbox();
}


void
NattoView_Xm::viewmenu_set_origin(Widget, XtPointer)
{
    obj()->set_origin();
}


void
NattoView_Xm::viewmenu_home_view(Widget, XtPointer)
{
    obj()->home_view();
}


void
NattoView_Xm::viewmenu_top_view(Widget, XtPointer)
{
    obj()->top_view();
}


void
NattoView_Xm::viewmenu_focus_view(Widget, XtPointer)
{
    obj()->focus_view();
}


void
NattoView_Xm::viewmenu_wireframe(Widget, XtPointer)
{
    obj()->render_wireframe();
}


void
NattoView_Xm::viewmenu_surface(Widget, XtPointer)
{
    obj()->render_surface();
}


void
NattoView_Xm::viewmenu_no_label(Widget, XtPointer)
{
    obj()->render_no_label();
}


void
NattoView_Xm::viewmenu_close(Widget, XtPointer)
{
    obj()->close();
}


void
NattoView_Xm::under_construction()
{
    msgout << "Not yet implemented." << endl;
}


NattoView*
NattoView_Xm::open_subview(
    const char* name,
    int x, int y,
    int w, int h)
{
    int hscale = 0, vscale = 0;
    XtVaGetValues(htransscale_, XmNheight, &hscale, NULL);
    XtVaGetValues(vtransscale_, XmNwidth,  &vscale, NULL);
    
    MotifConfig conf;
    conf.clear();
    conf.add(XmNfractionBase, 10000)
        .add(XmNx,      x)
        .add(XmNy,      obj()->height() - y - h)
        .add(XmNwidth,  w + hscale)
        .add(XmNheight, h + vscale);
    LayoutForm* form = new LayoutForm("test", obj()->parent());
    form->create(&conf);
    form->manage();
    
    NattoViewGL* view =
        new NattoViewGL("test", form, DCAST(NattoViewGL*, obj()), x, y, w, h);
    view->create(NULL);
    view->manage();
    return view;
}
