/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef NATTOVIEW_XM_H
#define NATTOVIEW_XM_H
#ifdef __GNUG__
#pragma interface
#endif

//////////////////////////////////////////////////////////////////////////////
//  NattoView_Xm.h -- X/Motif (and GL) specific display routines
//

#include <vector>
#include <algorithm>
#include <Xm/Xm.h>
#include <GL/GLwMDrawA.h>
#include "xtinter.h"
#include "NattoViewGL.h"
#include "MotifKit.h"

class MainFrame;


//////////////////////////////////////////////////////////////////////////////
//  NattoView_Xm --
//
class NattoView_Xm: public GuiObject_Xm, public NattoView_if
{
    typedef NattoView_Xm self_type;

 public:
    // primitive functions
    explicit NattoView_Xm(GuiObject*);
    virtual ~NattoView_Xm();

    void          init(GuiConfig*);
    NattoView*    obj();
    GuiObject_if* interface();

    // Xt and GLX interface
    XtAppContext app_context(){ return XtWidgetToApplicationContext(widget());}
    GLXContext   glx_context()  { return glxcx_; }

    // buffer operations (overrided)
    virtual void make_current() { GLwDrawingAreaMakeCurrent(widget(), glxcx_);}
    virtual void swap_buffers();
    virtual void adjust_sliders();

    virtual NattoView* open_subview(const char* name,
                                    int x, int y, int w, int h);
    
 protected:
    Natto* natto() { return obj()->natto(); }
    bool   init_aux(const char* name);

    void create_node_menu(Widget, vector<Natto::node*>&);
    void create_link_menu(Widget, vector<Natto::link*>&);

    // string rasterlization
    virtual void* string_bitmap(string str, int& w, int& h, int& base);
    
    // window callbacks
    DeclCallback(viewmenu_tourbox,    XtPointer);
    DeclCallback(viewmenu_set_origin, XtPointer);
    DeclCallback(viewmenu_home_view,  XtPointer);
    DeclCallback(viewmenu_top_view,   XtPointer);
    DeclCallback(viewmenu_focus_view, XtPointer);
    DeclCallback(viewmenu_wireframe,  XtPointer);
    DeclCallback(viewmenu_surface,    XtPointer);
    DeclCallback(viewmenu_no_label,   XtPointer);
    DeclCallback(viewmenu_close,      XtPointer);

    DeclCallback(ginit,               GLwDrawingAreaCallbackStruct*);
    DeclCallback(resize,              GLwDrawingAreaCallbackStruct*);
    DeclCallback(expose,              GLwDrawingAreaCallbackStruct*);
    DeclCallback(input,               GLwDrawingAreaCallbackStruct*);
    DeclCallback(htransscale_changed, XmScaleCallbackStruct*);
    DeclCallback(vtransscale_changed, XmScaleCallbackStruct*);
    DeclCallback(hrotscale_changed,   XmScaleCallbackStruct*);
    DeclCallback(vrotscale_changed,   XmScaleCallbackStruct*);
    DeclCallback(node_menu_item,      XtPointer);
    DeclCallback(link_menu_item,      XtPointer);

    void button_press(XButtonEvent*);
    void key_release(XKeyEvent*);
    void button_release(XButtonEvent*);
    void motion_notify(XMotionEvent*);
    void rubberband();
    void drag_arrow();

    DeclWorkProc(redisplay);
    void post_redisplay();

    // when selecting invalid menu
    void under_construction();
    
 private:
    GLXContext   glxcx_;         // GLX rendering context 
    XVisualInfo* vi_;
    Widget       htransscale_;
    Widget       vtransscale_;
    Widget       hrotscale_;
    Widget       vrotscale_;
    Widget       cursor_menu_;
    Widget       pick_menu_;
    Widget       view_menu_;
    XFontSet     font_set_;
    XButtonEvent btn_event_;

    Position x_drag_start_;
    Position y_drag_start_;
    Position x_drag_end_;
    Position y_drag_end_;

    bool pending_;
    vector<Natto::node*> picked_nodes_;
    vector<Natto::link*> picked_links_;

    NattoView_Xm(const NattoView_Xm&);
    NattoView_Xm& operator=(const NattoView_Xm&);
    DBG_NEW
};

#endif  /* NATTOVIEW_XM_H */
