/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define USE_PIXMAP

#ifdef __GNUG__
#pragma implementation
int dummy_SiteBar_Xm_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "SiteBar_Xm.h"

#include <Xm/XmAll.h>
#include "ComboBox.h"
#ifdef USE_PIXMAP
#include "utilx.h"
#include "browse.xpm"
#endif


SiteBar_Xm::SiteBar_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj), sitebox_(NULL), browse_pixmap_(0)
{}


void
SiteBar_Xm::init(
    GuiConfig* config)
{
    MotifConfig conf;
    if (config) conf = *DCAST(MotifConfig*, config);
    
    GuiObject_Xm* parent = DCAST(GuiObject_Xm*, obj()->parent()->imp());
    this->widget(XmCreateForm(parent->widget(), obj()->name(),
                              conf.args(), conf.size()));

    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_FORM)
        .add(XmNbottomAttachment, XmATTACH_FORM)
        .add(XmNleftAttachment,   XmATTACH_NONE)
        .add(XmNrightAttachment,  XmATTACH_FORM);
    Widget browse =
        XmCreatePushButton(widget(), "browse", conf.args(), conf.size());
#ifdef USE_PIXMAP
    browse_pixmap_ = CreateLabelPixmap(browse, browse_xpm);
    conf.clear();
    conf.add(XmNlabelType,        XmPIXMAP)
        .add(XmNlabelPixmap,      browse_pixmap_);
    XtSetValues(browse, conf.args(), conf.size());
#endif
    XtManageChild(browse);
    AddCallback(browse, XmNactivateCallback, browse_activate);

    conf.clear();
    conf.add(XmNtopAttachment,    XmATTACH_FORM)
        .add(XmNbottomAttachment, XmATTACH_FORM)
        .add(XmNleftAttachment,   XmATTACH_FORM)
        .add(XmNrightAttachment,  XmATTACH_WIDGET)
        .add(XmNrightWidget,      browse);
    Widget sitecap =
        CreateCaption(widget(), "sitecap", conf.args(), conf.size());
    XtManageChild(sitecap);

    conf.clear();
    conf.add(XmNtype,         XmDROP_DOWN_COMBO_BOX)
        .add(XmNarrowType,    XmMOTIF)
        .add(XmNresizeHeight, True);
    sitebox_ = DtCreateComboBox(sitecap, "sitebox", conf.args(), conf.size());
    XtManageChild(sitebox_);
    AddCallback(sitebox_, XmNactivateCallback,    sitebox_activate);
    AddCallback(sitebox_, XmNselectionCallback,   sitebox_selection);
    AddCallback(sitebox_, XmNfocusCallback,       sitebox_focus);
    AddCallback(sitebox_, XmNlosingFocusCallback, sitebox_losing_focus);
}


SiteBar_Xm::~SiteBar_Xm()
{
    if (browse_pixmap_) XFreePixmap(XtDisplay(widget()), browse_pixmap_);
}


SiteBar*
SiteBar_Xm::obj()
{
    return DCAST(SiteBar*, GuiObject_imp::obj());
}


GuiObject_if*
SiteBar_Xm::interface()
{
    return this;
}


void
SiteBar_Xm::set_node(
    Natto::node* vp)
{
    if (!vp) return;
    Widget textf = NULL;
    XtVaGetValues(sitebox_, XmNtextField, &textf, NULL);
    XmTextFieldSetString(textf, vp->url().abs().c_str());
    XmString xms =
        XmStringCreate(vp->url().abs().c_str(), XmSTRING_DEFAULT_CHARSET);
    DtComboBoxAddItem(sitebox_, xms, 1, True);
}


void
SiteBar_Xm::sitebox_activate(
    Widget, XtPointer)
{
    Widget textf = NULL;
    XtVaGetValues(sitebox_, XmNtextField, &textf, NULL);
    char* str = XmTextFieldGetString(textf);
    obj()->focus(URL(str));
    XtFree(str);
}


void
SiteBar_Xm::browse_activate(
    Widget, XtPointer)
{
    obj()->browse();
}


void
SiteBar_Xm::sitebox_selection(
    Widget, XmListCallbackStruct* ls)
{
    Widget textf = NULL;
    XtVaGetValues(sitebox_, XmNtextField, &textf, NULL);
    char* str;
    if (!XmStringGetLtoR(ls->item, XmSTRING_DEFAULT_CHARSET, &str))
	return;
    XmTextFieldSetString(textf, str);
    obj()->focus(URL(str));
    XtFree(str);
}


void
SiteBar_Xm::sitebox_focus(
    Widget, XtPointer)
{}


void
SiteBar_Xm::sitebox_losing_focus(
    Widget, XtPointer)
{}
