/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define USE_PIXMAP

#ifdef __GNUG__
#pragma implementation
int dummy_ToolBar_Xm_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "ToolBar_Xm.h"

#include <Xm/XmAll.h>
#ifdef USE_PIXMAP
#include "utilx.h"
#include "browse.xpm"
#include "reload.xpm"
#include "stop.xpm"
#endif


ToolBar_Xm::ToolBar_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj),
    browse_pixmap_(0), reload_pixmap_(0), stop_pixmap_(0)
{}


void
ToolBar_Xm::init(
    GuiConfig* config)
{
    MotifConfig conf;
    if (config) conf = *DCAST(MotifConfig*, config);
    
    GuiObject_Xm* parent = DCAST(GuiObject_Xm*, obj()->parent()->imp());
    conf.add(XmNorientation, XmHORIZONTAL);
    this->widget(XmCreateRowColumn(parent->widget(), obj()->name(),
                                   conf.args(), conf.size()));
    
    Widget browse = XmCreatePushButton(widget(), "browse", NULL, 0);
#ifdef USE_PIXMAP
    browse_pixmap_ = CreateLabelPixmap(browse, browse_xpm);
    conf.clear();
    conf.add(XmNlabelType,   XmPIXMAP)
        .add(XmNlabelPixmap, browse_pixmap_);
    XtSetValues(browse, conf.args(), conf.size());
#endif
    XtManageChild(browse);
    AddCallback(browse, XmNactivateCallback, browse_activate);

    Widget reload = XmCreatePushButton(widget(), "reload", NULL, 0);
#ifdef USE_PIXMAP
    reload_pixmap_ = CreateLabelPixmap(reload, reload_xpm);
    conf.clear();
    conf.add(XmNlabelType,   XmPIXMAP)
        .add(XmNlabelPixmap, reload_pixmap_);
    XtSetValues(reload, conf.args(), conf.size());
#endif
    XtManageChild(reload);
    AddCallback(reload, XmNactivateCallback, reload_activate);

    Widget stop = XmCreatePushButton(widget(), "stop", NULL, 0);
#ifdef USE_PIXMAP
    stop_pixmap_ = CreateLabelPixmap(stop, stop_xpm);
    conf.clear();
    conf.add(XmNlabelType,   XmPIXMAP)
        .add(XmNlabelPixmap, stop_pixmap_);
    XtSetValues(stop, conf.args(), conf.size());
#endif
    XtManageChild(stop);
    AddCallback(stop, XmNactivateCallback, stop_activate);
}


ToolBar_Xm::~ToolBar_Xm()
{
    if (browse_pixmap_) XFreePixmap(XtDisplay(widget()), browse_pixmap_);
    if (reload_pixmap_) XFreePixmap(XtDisplay(widget()), reload_pixmap_);
    if (stop_pixmap_)   XFreePixmap(XtDisplay(widget()), stop_pixmap_);
}


ToolBar*
ToolBar_Xm::obj()
{
    return DCAST(ToolBar*, GuiObject_imp::obj());
}


GuiObject_if*
ToolBar_Xm::interface()
{
    return this;
}


void
ToolBar_Xm::browse_activate(
    Widget, XtPointer)
{
    obj()->browse();
}


void
ToolBar_Xm::reload_activate(
    Widget, XtPointer)
{
    obj()->reload();
}


void
ToolBar_Xm::stop_activate(
    Widget, XtPointer)
{
    obj()->stop();
}
