/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1997-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifdef __GNUG__
#pragma implementation
int dummy_ViewPalette_Xm_cc = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "ViewPalette_Xm.h"

#include <Xm/XmAll.h>


ViewPalette_Xm::ViewPalette_Xm(
    GuiObject* obj):
    GuiObject_Xm(obj),
    menu_(NULL), wireframe_(NULL), drawicon_(NULL), drawlabel_(NULL)
{}


void
ViewPalette_Xm::init(
    GuiConfig* config)
{
    MotifConfig conf;
    if (config) conf = *DCAST(MotifConfig*, config);
    
    GuiObject_Xm* parent = DCAST(GuiObject_Xm*, obj()->parent()->imp());
    this->widget(XmCreateFrame(parent->widget(), obj()->name(),
                               conf.args(), conf.size()));

    conf.clear();
    conf.add(XmNorientation, XmVERTICAL);
    menu_ = XmCreateRowColumn(widget(), "menu", conf.args(), conf.size());
    XtManageChild(menu_);

    Widget w;
    XtManageChild(XmCreateLabel(menu_, "lifting", NULL, 0));
    w = XmCreatePushButton(menu_, "liftup", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, liftup_activate);
    w = XmCreatePushButton(menu_, "liftdown", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, liftdown_activate);
    w = XmCreatePushButton(menu_, "downall", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, downall_activate);
    XtManageChild(XmCreateSeparator(menu_, "separator1", NULL, 0));

    XtManageChild(XmCreateLabel(menu_, "views", NULL, 0));
    w = XmCreatePushButton(menu_, "homeview", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, homeview_activate);
    w = XmCreatePushButton(menu_, "topview", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, topview_activate);
    w = XmCreatePushButton(menu_, "topview2d", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, topview2d_activate);
    XtManageChild(XmCreateSeparator(menu_, "separator2", NULL, 0));

    XtManageChild(XmCreateLabel(menu_, "zooming", NULL, 0));
    w = XmCreatePushButton(menu_, "zoomin", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, zoomin_activate);
    w = XmCreatePushButton(menu_, "zoomout", NULL, 0);
    XtManageChild(w);
    AddCallback(w, XmNactivateCallback, zoomout_activate);
    XtManageChild(XmCreateSeparator(menu_, "separator3", NULL, 0));

    XtManageChild(XmCreateLabel(menu_, "rendering", NULL, 0));
    wireframe_ = XmCreateToggleButton(menu_, "wireframe", NULL, 0);
    XtManageChild(wireframe_);
    AddCallback(wireframe_, XmNvalueChangedCallback, wireframe_value_changed);
    drawicon_ = XmCreateToggleButton(menu_, "drawicon", NULL, 0);
    XtManageChild(drawicon_);
    AddCallback(drawicon_, XmNvalueChangedCallback, drawicon_value_changed);
    drawlabel_ = XmCreateToggleButton(menu_, "drawlabel", NULL, 0);
    XtManageChild(drawlabel_);
    AddCallback(drawlabel_, XmNvalueChangedCallback, drawlabel_value_changed);
    XtManageChild(XmCreateSeparator(menu_, "separator4", NULL, 0));
}


ViewPalette*
ViewPalette_Xm::obj()
{
    return DCAST(ViewPalette*, GuiObject_imp::obj());
}


GuiObject_if*
ViewPalette_Xm::interface()
{
    return this;
}


void
ViewPalette_Xm::set_render_mode()
{
    XmToggleButtonSetState(
	wireframe_, obj()->wireframe() ? True: False, False);
    XmToggleButtonSetState(
	drawicon_, obj()->draw_icon() ? True: False, False);
    XmToggleButtonSetState(
	drawlabel_, obj()->draw_label() ? True: False, False);
}


void
ViewPalette_Xm::liftup_activate(
    Widget, XtPointer)
{
    obj()->lift_up();
}


void
ViewPalette_Xm::liftdown_activate(
    Widget, XtPointer)
{
    obj()->lift_down();
}


void
ViewPalette_Xm::downall_activate(
    Widget, XtPointer)
{
    obj()->lift_down_all();
}


void
ViewPalette_Xm::homeview_activate(
    Widget, XtPointer)
{
    obj()->home_view();
}


void
ViewPalette_Xm::topview_activate(
    Widget, XtPointer)
{
    obj()->top_view();
}


void
ViewPalette_Xm::topview2d_activate(
    Widget, XtPointer)
{
    obj()->top_view_2d();
}


void
ViewPalette_Xm::zoomin_activate(
    Widget, XtPointer)
{
    obj()->zoom_in();
}


void
ViewPalette_Xm::zoomout_activate(
    Widget, XtPointer)
{
    obj()->zoom_out();
}


void
ViewPalette_Xm::wireframe_value_changed(
    Widget,
    XmToggleButtonCallbackStruct* cbs)
{
    obj()->wireframe(cbs->set);
}


void
ViewPalette_Xm::drawicon_value_changed(
    Widget,
    XmToggleButtonCallbackStruct* cbs)
{
    obj()->draw_icon(cbs->set);
}


void
ViewPalette_Xm::drawlabel_value_changed(
    Widget,
    XmToggleButtonCallbackStruct* cbs)
{
    obj()->draw_label(cbs->set);
}
