#! /usr/local/bin/perl
#/*
#//  The Natto View
#//
#//  Copyright (c) 1996  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
#//
#//  The code in this file may be used, copied and redistributed under the
#//  same terms as perl itself (see the files README, Copying and Artistic
#//  in the perl distribution).
#*/

while (<>) {
    /undefined/ || next;
    /`(.*)'/ && do {
	my $symbol = $1;
#	1 while ($symbol =~ s/basic_string\s*<\s*char\s*,\s*string_char_traits\s*<\s*char\s*>\s+>/string/g);
	1 while find_class_templates($symbol);
	1 while find_function_templates($symbol);
    };
}


print "#ifdef EXPLATE\n\n";
print "#ifdef EXPLATE_CLASS\n\n";
for (sort @classes) {
    restore_class_names($_);
    next if $seen{$_} || $_ =~ /^basic_string/;
    print "template class $_;\n";
    $seen{$_}++;
}
print "\n#endif /* EXPLATE_CLASS */\n\n";


print "#ifdef EXPLATE_FUNCTION\n\n";
for $fun (sort @functions) {
    my $type;
    local($_) = $fun;
    restore_class_names($fun);
    next if $seen{$fun};
    
    if (/^operator\s*(\!|(<[^<])|(>[^>])|(==)|(<=)|(>=))/) {
        $type = 'bool';
    }
    elsif ($fun =~ /^operator\s*((<<)|(>>))\s*\((.+stream\s*&?)/) {
        $type = $4;
    }
    elsif (/^operator\s*\+\s*\((\01\d+\02)/) {
        my($class) = $1;
	restore_class_names($class);
        $type = $class;
    }
    elsif (/^find((_each)|(_if))?\s*\(([^,]+)/) {
        my($class) = $4;
	restore_class_names($class);
        $type = $class;
    }
    elsif (/^(uninitialized_)?copy(_backward)?\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^(stable_)?sort\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^(uninitialized_)?fill(_n)?\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^(__)?((push)|(pop)|(make)|(sort)_)?heap(_aux)?\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^swap\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^for_each\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^(__)?distance\s*\(([^,]+)/) {
        $type = 'void';
    }
    elsif (/^allocate\s*\([^,]+,([^)]+)/) {
        my($class) = $1;
	restore_class_names($class);
        $type = $class;
    }
    elsif (/^deallocate\s*\(/) {
        $type = 'void';
    }
    elsif (/^construct\s*\(/ || /^destroy\s*\(/) {
        $type = 'void';
    }
    elsif (/^((max)|(min))\s*\(([^,]+)/) {
        my($class) = $4;
	restore_class_names($class);
        $type = $class;
    }
    elsif (/^dfs_((forward)|(backward))(_aux)?\s*\(([^,]+)/) {
        my($class) = $5;
	restore_class_names($class);
        $type = $class;
    }
    elsif (/^clear_container\s*\(/) {
        $type = 'void';
    }
    elsif (/^split\s*\(/) {
        $type = 'void';
    }
    elsif (/^join\s*\(/) {
        $type = 'string';
    }
    else {
        undef $type;
    }
    print "/*? " unless $type;
    print "template $type $fun;";
    print " ?*/" unless $type;
    print "\n";
    $seen{$fun}++;
}
print "\n#endif /* EXPLATE_FUNCTION */\n";
print "\n#endif /* EXPLATE */\n";


sub find_function_templates
{
    push(@functions, $_[0]) if $_[0] =~ /^[^:]+\(/;
    undef;
}

sub find_class_templates
{
    my $class;
    my $ret = undef;
    foreach $match ($_[0] =~ /(\w+\s*<[^<>]+>)[^>]/g) {
	my $id = @classes;
	push(@classes, $match);
	$match =~ s/\*/\\*/g;
	$_[0] =~ s/$match/\01$id\02/g;
	$ret = 1;
    }
    $ret;
}

sub restore_class_names
{
    1 while $_[0] =~ s/\01(\d+)\02/$classes[$1]/g;
}
