/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#define DBG_MODULE  Misc

#ifdef __GNUG__
int dummy_mainx = 0;
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <fstream.h>
#include <strstream.h>
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>
#include <X11/Xmu/Editres.h>
#include <X11/Xmu/Converters.h>
#include <Xm/XmAll.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "util.h"
#include "utilx.h"
#include "GenericMapper.h"
#include "MotifKit.h"
#include "MainFrame.h"
#include "NattoView.h"
#include "Preference.h"
#include "NetManager.h"
#include "NetObject.h"

extern "C" int mkdir2(const char*, mode_t);

static void xt_control_fifo(istream*, int*, XtInputId*);
static void xt_quitbox_ok(Widget, XtPointer, XtPointer);
static void xt_toplevel_destroy(Widget, XtPointer, XtPointer);

static void hack_options(int, char*[]);
static void analyze_options(int, char*[]);
static void cleanup(void);
static int  x_error(Display*, XErrorEvent*);
static void xt_error(char*);
static void core_dump(void);

static XtAppContext app_context   = 0;
static Widget       quitbox       = NULL;
static Natto*       current_natto = NULL;
static MainFrame*   current_frame = NULL;
static char*        fifo_name     = "";
static XtInputId    fifo_id       = 0;

#if 0
static void xt_action_pick(Widget, XEvent*, String[], Cardinal*);
static void xt_action_drag(Widget, XEvent*, String[], Cardinal*);
#endif
static void xt_action_lift_up(Widget, XEvent*, String[], Cardinal*);
static void xt_action_lift_down(Widget, XEvent*, String[], Cardinal*);
static void xt_action_lift_down_all(Widget, XEvent*, String[], Cardinal*);

static XtConvertArgRec converter_table[] = {
    { XtBaseOffset, (char*) XtOffset(Widget, core.parent), sizeof(Widget) }};

static XtActionsRec action_table[] = {
    { "lift-up",        xt_action_lift_up },
    { "lift-down",      xt_action_lift_down },
    { "lift-down-all",  xt_action_lift_down_all },
#if 0
    { "translate",      xt_action_translate },
    { "rotate",         xt_action_rotate },
    { "zoom",           xt_action_zoom },
#endif
};

// Netscape remote control
const char* progname = NULL;
const char* expected_mozilla_version = "1.1";


static void
xt_action_lift_up(Widget, XEvent*, String[], Cardinal*)
{
    current_frame->lift_up();
}


static void
xt_action_lift_down(Widget, XEvent*, String[], Cardinal*)
{
    current_frame->lift_down();
}


static void
xt_action_lift_down_all(Widget, XEvent*, String[], Cardinal*)
{
    current_frame->lift_down_all();
}


static void 
xt_control_fifo(
    istream* in,
    int*, XtInputId*)
{
    strstream buf;
    string cmd;
    string arg, arg2;
    int    para = 0;
    int    ch   = '\0';

    NattoView* view = current_frame->active_view();

    copy_line(*in, buf); buf << ends;
    if (!*in) {
	msgout << "Control fifo broken." << endl;
        DBG("fifo", "control fifo broken");
	XtRemoveInput(fifo_id);
	return;
    }
    DBG("fifo", "control=" << buf.str());
    buf.freeze(false);

    buf >> cmd;
    if (cmd.length() == 0) goto done;
    ch = cmd[0];

    switch (ch) {
	/* Home */     case 'H':  view->home_view();     goto done;
	/* Up */       case 'U':  view->lift_up();       goto done;
	/* Down */     case 'D':  view->lift_down();     goto done;
	/* Down All */ case 'A':  view->lift_down_all(); goto done;
	/* Browse */   case 'B':  view->browse();        goto done;
	/* Reload */   case 'L':  view->reload();        goto done;
	/* Quit */     case 'Q':  ExitNattoView();       goto done;
	/* Exit */     case 'X':  ExitNattoView();       goto done;
    }

    buf >> arg;
    switch (ch) {
     case 'C': /* CURSOR */
	if (arg.length()) view->focus(arg);
	goto done;

     case 'M': /* MARK */
#if 0
	buf >> arg2;
	if (arg.length() && arg2.length()) view->set_mark(arg, arg2);
	else if (arg.length())             view->set_mark(arg);
	else                               view->set_mark();
#endif
	goto done;
	
     case 'T':
	if (!cmd[1] || !cmd[2]) break;
	para = 0;
	if (arg.length()) para = atoi(arg.c_str());

	switch ((toupper(cmd[1]) << 8) | cmd[2]) {
	 case ('X' << 8) | '+':
	    if (para) view->trans_left(para);
	    else      view->trans_left();
	    break;

	 case ('X' << 8) | '-':
	    if (para) view->trans_right(para);
	    else      view->trans_right();
	    break;

	 case ('Y' << 8) | '+':
	    if (para) view->trans_up(para);
	    else      view->trans_up();
	    break;

	 case ('Y' << 8) | '-':
	    if (para) view->trans_down(para);
	    else      view->trans_down();
	    break;
	}
	goto done;

     case 'R':
	if (!cmd[1] || !cmd[2]) break;
	para = 0;
	if (arg.length()) para = atoi(arg.c_str());
	switch ((toupper(cmd[1]) << 8) | cmd[2]) {

	 case ('X' << 8) | '+':
	    if (para) view->rot_up(para);
	    else      view->rot_up();
	    break;

	 case ('X' << 8) | '-':
	    if (para) view->rot_down(para);
	    else      view->rot_down();
	    break;
	 
	 case ('Y' << 8) | '+':
	    if (para) view->rot_left(para);
	    else      view->rot_left();
	    break;
	 
	 case ('Y' << 8) | '-':
	    if (para) view->rot_right(para);
	    else      view->rot_right();
	    break;
	}
	goto done;

     case 'Z':
	if (!cmd[1] || !cmd[2]) break;
	para = 0;
	if (arg.length()) para = atoi(arg.c_str());

	switch ((toupper(cmd[1]) << 8) | cmd[2]) {
	 case ('Z' << 8) | '+':
	    if (para) view->zoom_in(para);
	    else      view->zoom_in();
	    break;

	 case ('Z' << 8) | '-':
	    if (para) view->zoom_out(para);
	    else      view->zoom_out();
	    break;
	}
	goto done;

    }

    return;  /* no match */

  done:;  /* OK! */
    view->update();
}


int
main(
    int   argc,
    char* argv[])
{
    atexit(cleanup);
    // set_terminate(core_dump);
    
    progname= strchr(argv[0], '/');
    if (progname) progname++;
    else	  progname = argv[0];

    // help and version options (before opening display)
    hack_options(argc, argv);

    // Motif
    XtToolkitInitialize();
    XtSetLanguageProc(NULL, NULL, NULL);

    toplevel = XtAppInitialize(&app_context, "Nattoview", NULL, 0,
			       &argc, argv, NULL, NULL, 0);
    ::portkit = new MotifKit(app_context);
    XtAddCallback(toplevel, XmNdestroyCallback, xt_toplevel_destroy, NULL);
    XtAppAddConverter(app_context, XmRString, XmRWindow, XmuCvtStringToWidget,
		      converter_table, XtNumber(converter_table));
    XtAppAddActions(app_context, action_table, XtNumber(action_table));

    // other options
    analyze_options(argc, argv);

    // home page
    if (!preference.home_page) {
        char* home_page = getenv("HOME_PAGE");
	preference.home_page = URL(home_page ? home_page: HOME_PAGE);
        DBG("pref", "homepage=" << preference.home_page.abs());
    }

    // application directory
    if (!preference.app_dir.length()) {
        char* home_dir = getenv("HOME");
	if (home_dir) preference.app_dir = string(home_dir) + "/.nattoview";
	else          preference.app_dir = "/tmp/nattoview";
        DBG("pref", "appdir=" << preference.app_dir);
    }
    mkdir2(preference.app_dir.c_str(), 0700);

    // cache directory
    if (!preference.cache_dir.length()) {
        preference.cache_dir = preference.app_dir + "/cache";
        DBG("pref", "cache=" << preference.cache_dir);
    }
    mkdir2(preference.cache_dir.c_str(), 0700);

    ::netmanager = new NetManager(preference.cache_dir);

    if (!preference.logname.length()) {
	char* logname = getenv("LOGNAME");
	preference.logname = logname ? logname: "libwww";
        DBG("pref", "logname=" << preference.logname);
    }
    if (!preference.tmp_dir.length()) {
	char* tmp_dir = getenv("HOME");
	preference.tmp_dir = tmp_dir ? tmp_dir: "/tmp";
        DBG("pref", "tmp_dir=" << preference.tmp_dir);
    }

    // debugging X (when not static linked)
    if (preference.debug_X) {
	XSynchronize(XtDisplay(toplevel), True);
	XSetErrorHandler(x_error);
	XtSetErrorHandler(xt_error);
    }

    current_natto = new Natto;
    current_frame = new MainFrame("main_frame", NULL, current_natto);
    current_frame->create(NULL);
    XtRealizeWidget(toplevel);

    // open control fifo
    fifo_name = getenv("NATTOVIEW_FIFO");
    if (fifo_name) {
        DBG("fifo", fifo_name << " opening");
	ifstream* fifo = new ifstream(fifo_name, ios::in);
	if (!fifo || !*fifo) {
	    msgout << "Warning: Can't open control fifo "
                   << fifo_name << '.' << endl;
            DBG("fifo", fifo_name << " can't open");
	}
	else {
	    fifo_id = XtAppAddInput(app_context, fifo->rdbuf()->fd(),
				    (XtPointer) XtInputReadMask,
				    (XtInputCallbackProc) xt_control_fifo,
				    (XtPointer) fifo);
	}
    }
    
    // initial access
    current_frame->focus_node(preference.home_page);

    // make "Realy Quit ?" dialog
    quitbox = XmCreateWarningDialog(toplevel, "quitbox", NULL, 0);
    XtUnmanageChild(XmMessageBoxGetChild(quitbox,
					 XmDIALOG_HELP_BUTTON));
    XtAddCallback(quitbox, XmNokCallback, xt_quitbox_ok, NULL);

    // set signal handler
    signal(SIGPIPE, SIG_IGN);
    
    // event loop
    XtAddEventHandler(toplevel, 0, True, _XEditResCheckMessages, NULL);
    for (;;) {
	const XtInputMask no_input = XtIMAll & ~XtIMAlternateInput;
	while (XtAppPending(app_context) & no_input)
	    XtAppProcessEvent(app_context, no_input);
	XtAppProcessEvent(app_context, XtIMAll);
    }
    return EXIT_SUCCESS;  // dummy
}


static void
hack_options(
    int   argc,
    char* argv[])
{
    for (int i = 1; i < argc; i++) {
	if (strncmp(argv[i], "-v",  2) == 0 || strncmp(argv[i], "--v", 3) == 0) {
	    cout << "The Natto View version " NATTOVIEW "\n"
		    "Copyright (c) 1995-1997  Hidekazu Shiozawa,\n"
		    "Bug reports are wellcome!! <shiozawa@myo.inst.keio.ac.jp>\n" << endl;
	    exit(EXIT_SUCCESS);
	}
	if (strncmp(argv[i], "-h",  2) == 0 || strncmp(argv[i], "--h", 3) == 0 ||
	    strncmp(argv[i], "-?",  2) == 0 || strncmp(argv[i], "--?", 3) == 0) {
	    cout << "The Natto View version " NATTOVIEW "\n"
		    "Usage: " << progname << " [-options...] [-toolkit-options...] [URL]\n\n"
		    "Options:\n"
		    "  -v, -version, --version  print version information\n"
		    "  -h, -help,    --help     print this message\n"
		    "  -install                 install colormap\n"
		 << endl;
	    exit(EXIT_SUCCESS);
	}
    }
}


static void
analyze_options(
    int   argc,
    char* argv[])
{
    for (int i = 1; i < argc && argv[i]; i++) {
	if (argv[i][0] != '-') {
	    preference.home_page = argv[i];
	}
	else if (strcmp(argv[i], "-install") == 0) {
	    preference.install_cmap = true;
	}
	else if (strcmp(argv[i], "-xdebug") == 0) {
	    preference.debug_X = true;
	}
	else if (strncmp(argv[i], "-proxy=", 7) == 0) {
	    preference.http_proxy = argv[i] + 7;
	}
	else if (strncmp(argv[i], "-ftpproxy=", 10) == 0) {
	    preference.ftp_proxy = argv[i] + 10;
	}
	else {
	    cout << "Illegal option: " << argv[i] << endl
		<< argv[0] << " -h for help." << endl;
	    exit(EXIT_SUCCESS);
	}
    }
}


static void
xt_toplevel_destroy(Widget, XtPointer, XtPointer)
{
    exit(EXIT_SUCCESS);
}


void 
ExitNattoView(void)
{
    XtManageChild(quitbox);
}


static void
xt_quitbox_ok(Widget, XtPointer, XtPointer)
{
    exit(EXIT_SUCCESS);
}


static void
cleanup(void)
{
    delete ::netmanager;
    delete ::portkit;
    delete ::current_natto;
    delete ::current_frame;
    XtDestroyWidget(toplevel);
    XtDestroyApplicationContext(app_context);
    NetObject::clear_database();
}


// catch X errors
static int
x_error(
    Display*     dpy,
    XErrorEvent* event)
{
    dpy = dpy;
    event = event;
    core_dump();
    return 1;
}


static void
xt_error(
    char* msg)
{
    msg = msg;
    core_dump();
}


// Dump core! in order to stack trace
static void
core_dump(void)
{
    // Segmemtation violation on purpose. 
    char* p = 0;
    *p = 0;
}
