/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1995-1998  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#undef MENU_FRAME

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "utilx.h"

#include <algorithm>
#include <string>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/Xmu.h>
#include <X11/Xmu/StdCmap.h>
#include <Xm/XmAll.h>
#include <X11/xpm.h>
#include "Caption.h"


Colormap
GetShareableColormap(
    Display*     dpy,
    XVisualInfo* vi)
{
    Status status =
	XmuLookupStandardColormap(dpy, vi->screen, vi->visualid, vi->depth,
				  XA_RGB_DEFAULT_MAP, False, True);
    if (status == 1) {
	int n = 0;
	XStandardColormap* std_cmaps;
	status = XGetRGBColormaps(dpy, RootWindow(dpy, vi->screen),
				  &std_cmaps, &n, XA_RGB_DEFAULT_MAP);
	if (status == 1) {
	    for (int i = 0; i < n; i++) {
		if (std_cmaps[i].visualid == vi->visualid) {
		    Colormap cmap = std_cmaps[i].colormap;
		    XFree(std_cmaps);
		    return cmap;
		}
	    }
	}
    }
    return XCreateColormap(dpy, RootWindow(dpy, vi->screen), 
			   vi->visual, AllocNone);
}


int
ListGetSelectedPosFirst(
    Widget w)
{
    int* poslist= new int[512];
    int  count = 0;
    XmListGetSelectedPos(w, &poslist, &count);
    return count > 0 ? poslist[0]: 0;
    delete[] poslist;
}


Pixmap
CreateLabelPixmap(
    Widget widget,
    char*  xpm_data[])
{
    Display* dpy = XtDisplay(widget);
    Window   win = DefaultRootWindow(dpy);
    Pixmap xpm, mask;
    XpmAttributes attr;
    attr.valuemask = 0;
    XpmCreatePixmapFromData(dpy, win, xpm_data, &xpm, &mask, &attr);
    int width  = attr.width;  //33;
    int height = attr.height; //20;
    Pixel fgpixel = 0, bgpixel = 0;
    int depth = 0;
    XtVaGetValues(widget,
		  XmNdepth,      &depth,
		  XmNforeground, &fgpixel,
		  XmNbackground, &bgpixel, NULL);
    GC gc = XCreateGC(dpy, win, 0, 0);
    Pixmap pixmap = XCreatePixmap(dpy, win, width, height, depth);
    XSetForeground(dpy, gc, bgpixel);
    XFillRectangle(dpy, pixmap, gc, 0, 0, width, height);
    XSetClipMask(dpy, gc, mask);
    XCopyArea(dpy, xpm, pixmap, gc, 0, 0, width, height, 0, 0);
    XFreeGC(dpy, gc);
    XFreePixmap(dpy, xpm);
    return pixmap;
}


void
ScaleSetValue(
    Widget scale,
    int    value)
{
    int v;
    XmScaleGetValue(scale, &v);
    if (v != value) XmScaleSetValue(scale, value);
}


Widget
CreatePopupMenu2(
    Widget   parent,
    String   name,
    ArgList  args,
    Cardinal n)
{
    Arg args2[100];
    if (args) {
	for (unsigned i = 0; i < n; i++) args2[i] = args[i];
    }
    else {
	n = 0;
    }
    XtSetArg(args2[n], XmNallowShellResize, True); n++;
    XtSetArg(args2[n], XmNwidth, 10); n++;
    XtSetArg(args2[n], XmNheight, 10); n++;
    
    string wname = "popup_";
    wname += name;
    Widget shell = XtCreatePopupShell(wname.c_str(), xmMenuShellWidgetClass,
				      parent, args2, n);
#ifdef MENU_FRAME
    Widget frame = XmCreateFrame(shell, "frame", NULL, 0);
    XtManageChild(frame);
    Widget menu = XmCreateRowColumn(frame, name, NULL, 0);
#else
    Widget menu = XmCreateRowColumn(shell, name, NULL, 0);
#endif
    XtVaSetValues(menu,
		  // XmNrowColumnType, XmMENU_POPUP,
		  XmNmarginWidth,  0,
		  XmNmarginHeight, 0,
		  XmNspacing,      0, NULL);
    XtManageChild(menu);
    return menu;
}


void
ManagePopupMenu2(
    Widget   menu,
    Position x,
    Position y)
{
#ifdef MENU_FRAME
    Widget shell = XtParent(XtParent(menu));
#else
    Widget shell = XtParent(menu);
#endif
    XtVaSetValues(shell, XmNx, x, XmNy, y, NULL);
    XtPopup(shell, XtGrabNone);
}


void
DestroyPopupMenu2(
    Widget   menu)
{
#ifdef MENU_FRAME
    Widget shell = XtParent(XtParent(menu));
#else
    Widget shell = XtParent(menu);
#endif
    XtDestroyWidget(shell);
}


Widget
CreateCaption(
    Widget   parent,
    String   name,
    ArgList  args,
    Cardinal n)
{
    return XtCreateWidget(name, xbaeCaptionWidgetClass, parent, args, n);
}


XImage*
CreateXImageString(
    Display*    dpy,
    Drawable    win,
    XFontSet    font_set,
    const char* str,
    int&        width,
    int&        height,
    int&        base_line)
{
    XRectangle bounding_box = { 0, 0, 0, 0 };
    XmbTextExtents(font_set, str, strlen(str), NULL, &bounding_box);
    width     = max(bounding_box.width,  (unsigned short) 1);
    height    = max(bounding_box.height, (unsigned short) 1);
    base_line = height + bounding_box.y;

    int bwidth = (width + 7) / 8;  // width in bytes
    Window pixmap = XCreatePixmap(dpy, win, bwidth * 8, height, 1);
    XGCValues values;
    values.background = values.foreground = 0;
    GC gc = XCreateGC(dpy, pixmap, GCBackground | GCForeground, &values);
    XSetForeground(dpy, gc, 0);
    XFillRectangle(dpy, pixmap, gc, 0, 0, bwidth * 8, height);
    XSetForeground(dpy, gc, 1);

    XmbTextItem text;
    text.chars    = str;
    text.nchars   = strlen(str);
    text.delta    = 0;
    text.font_set = font_set;
    XmbDrawText(dpy, pixmap, gc, 0, height - base_line, &text, 1);
    XImage* image =
	XGetImage(dpy, pixmap, 0, 0, bwidth * 8, height, 1, XYPixmap);

    XFreeGC(dpy, gc);
    XFreePixmap(dpy, pixmap);
    return image;
}


void*
XImageToGLBitmap(
    XImage* image,
    int     width,
    int     height)
{
    int bwidth = (width + 7) / 8;  // width in bytes
    unsigned char* bitmap = new unsigned char[bwidth * height];
    memset(bitmap, 0, bwidth * height);

    for (int y = 0; y < height; y++) {
	for (int x = 0; x < width; x++) {
	    if (XGetPixel(image, x, height - 1 - y))
		bitmap[y * bwidth + x / 8] |= 0x80 >> x % 8;
	}
    }
    return bitmap;
}
