/*                                               -*- Mode:C++; tab-width: 8 -*-
//  The Natto View
//
//  Copyright (c) 1996-1997  Hidekazu Shiozawa <shiozawa@myo.inst.keio.ac.jp>
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU *Library* General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
*/

#ifndef XTINTER_H
#define XTINTER_H
#ifdef __GNUG__
#pragma interface
#endif

#include <X11/Intrinsic.h>
#if 1
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>
#endif


#define DeclCallback(callback, calldata_t) \
    static void xt_ ## callback(Widget w, XtPointer me, XtPointer cdata) \
	{ ((self_type*) me)->callback(w, (calldata_t) cdata); } \
    virtual void callback(Widget, calldata_t)

#define AddCallback(w, name, callback) \
    XtAddCallback(w, name, xt_ ## callback, (XtPointer) this)

#define RemoveCallback(w, name, callback) \
    XtRemoveCallback(w, name, xt_ ## callback, (XtPointer) this)

#define DeclEventHandler(callback, calldata_t) \
    static void xt_ ## callback(Widget w, XtPointer me, XEvent* event) \
	{ ((self_type*) me)->callback(w, (calldata_t) event); } \
    virtual void callback(Widget, calldata_t)

#define AddEventHandler(w, mask, flag, callback) \
    XtAddEventHandler(w, mask, flag, xt_ ## callback, (XtPointer) this)

#define RemoveEventHandler(w, mask, flag, callback) \
    XtRemoveEventHandler(w, mask, flag, xt_ ## callback, (XtPointer) this)

#define DeclInput(callback) \
    static void xt_ ## callback(XtPointer me, int* fd, XtInputId* id) \
	{ ((self_type*) me)->callback(*fd, *id); } \
    virtual void callback(int, XtInputId)

#define AddInput(app, fd, cond, callback) \
    XtAppAddInput(app, fd, (XtPointer) cond, xt_ ## callack, (XtPointer) this)

#define RemoveInput(id) \
    XtRemoveInput(id)

#define DeclWorkProc(callback) \
    static Boolean xt_ ## callback(XtPointer me) \
	{ return ((self_type*) me)->callback(); } \
    virtual Boolean callback()

#define AddWorkProc(app, callback) \
    XtAppAddWorkProc(app, xt_ ## callback, (XtPointer) this)

#define RemoveWorkProc(id) \
    XtRemoveWorkProc(id)


#define MENU_BEGIN(widget, name, parent) { \
    Widget menu, btn; \
    XtManageChild(widget = menu = XmCreateMenuBar(parent, name, NULL, 0))

#define MENU_END  }

#define POPUP_BEGIN(widget, name, parent)  { \
    Widget menu, btn; \
    widget = menu = XmCreatePopupMenu(parent, name, NULL, 0)

#define ITEM_LBL(name) \
    XtManageChild(XmCreateLabel/*Gadget*/(menu, name, NULL, 0))

#define POPUP_END  }

#define ITEM_SEP(name) \
    XtManageChild(XmCreateSeparator/*Gadget*/(menu, name, NULL, 0))

#define ITEM_BTN(name, callback) \
    XtManageChild(btn = XmCreatePushButton/*Gadget*/(menu, name, NULL, 0)); \
    AddCallback(btn, XmNactivateCallback, callback)

#define ITEM_SUB_BEGIN(name) \
    XtManageChild(btn = XmCreateCascadeButton/*Gadget*/(menu, name, NULL, 0)); \
    { Widget parent = menu, parbtn = btn; \
      { Widget menu, btn; \
        char* name2 = new char[strlen(name) + 6]; \
	strcpy(name2, name); \
	strcat(name2, "_menu"); \
        menu = XmCreatePulldownMenu(parent, name2, NULL, 0); \
	delete[] name2; \
        XtVaSetValues(parbtn, XmNsubMenuId, menu, NULL)
	  
#define ITEM_SUB_HELP(name) \
    XtManageChild(btn = XmCreateCascadeButton/*Gadget*/(menu, name, NULL, 0)); \
    { Widget parent = menu, parbtn = btn; \
      { Widget menu, btn; \
        char* name2 = new char[strlen(name) + 6]; \
	strcpy(name2, name); \
	strcat(name2, "_menu"); \
        menu = XmCreatePulldownMenu(parent, name2, NULL, 0); \
	delete[] name2; \
        XtVaSetValues(parbtn, XmNsubMenuId, menu, NULL); \
        XtVaSetValues(parent, XmNmenuHelpWidget, parbtn, NULL)

#define ITEM_SUB_END  } }


#endif /* XTINTER_H */
